/****************************************************************************/
/*

DESCRIPTION

	Common constant definitions for ARM processors.
	
AUTHORS

	Copyright 2002, 2004, 2009 Freescale Semiconductor, Inc.
	
*/
/****************************************************************************/

/*
** Memory space defines.
*/

#define KUSEG_BASE				0x00000000
#define KSEG0_BASE				0x80000000
#define KSEG1_BASE				0xa0000000
#define KSEG2_BASE				0xc0000000

/****************************************************************************/
/*
 *	Useful target-specific macros.
 */
/***************************************************************************/

/*
** Address segment/offset definitions.
*/

#define KUSEG					0x0
#define KSEG0					0x4
#define KSEG1					0x5
#define KSEG2					0x6

#define VADDRESS_SEGMENT( x )	(	(((ui32)x) < 0x80000000) ? KUSEG : \
								 	(((ui32)x) >= 0xc0000000) ? KSEG2 : \
								 	((ui32)((((ui32)x) >> 29) & 0x00000007)) )
								 	
#define VADDRESS_SOFFSET( x )	(((ui32)x) & 0x1fffffff)

/*
** Sign-extending oddball values to 32 bits.
*/

#define SIGN_EXTEND_8( x )		(((x) & 0x00000080) ? \
									(((ui32)(x)) | 0xffffff00) : \
										(((ui32)(x)) & 0x000000ff))								
#define SIGN_EXTEND_11( x )		(((x) & 0x00000400) ? \
									(((ui32)(x)) | 0xfffff800) : \
										(((ui32)(x)) & 0x000007ff))
#define SIGN_EXTEND_24( x )		(((x) & 0x00800000) ? \
									(((ui32)(x)) | 0xff000000) : \
										(((ui32)(x)) & 0x00ffffff))


/*
** Useful bit returns
*/

#define	INSTRUC_BIT0( x )		(((ui32)(x) & 0x00000001) >> 0)
#define	INSTRUC_BIT4( x )		(((ui32)(x) & 0x00000010) >> 4)
#define	INSTRUC_BIT5( x )		(((ui32)(x) & 0x00000020) >> 5)
#define	INSTRUC_BIT6( x )		(((ui32)(x) & 0x00000040) >> 6)
#define	INSTRUC_BIT7( x )		(((ui32)(x) & 0x00000080) >> 7)
#define	INSTRUC_BIT15( x )		(((ui32)(x) & 0x00008000) >> 15)
#define	INSTRUC_BIT20( x )		(((ui32)(x) & 0x00100000) >> 20)
#define	INSTRUC_BIT21( x )		(((ui32)(x) & 0x00200000) >> 21)
#define	INSTRUC_BIT22( x )		(((ui32)(x) & 0x00400000) >> 22)
#define	INSTRUC_BIT23( x )		(((ui32)(x) & 0x00800000) >> 23)
#define	INSTRUC_BIT24( x )		(((ui32)(x) & 0x01000000) >> 24)
#define	INSTRUC_BIT25( x )		(((ui32)(x) & 0x02000000) >> 25)
#define	INSTRUC_BIT31( x )		(((ui32)(x) & 0x80000000) >> 31)

#define INSTRUC_COND( x )		(((ui32)(x) & 0xf0000000) >> 28)
#define INSTRUC_IS_COND( x )	((INSTRUC_COND( x ) != 0x0e) && (INSTRUC_COND( x ) != 0x0f))

#define	INSTRUC_26_27( x )		(((ui32)(x) & 0x0c000000) >> 26)
#define	INSTRUC_25_27( x )		(((ui32)(x) & 0x0e000000) >> 25)
#define	INSTRUC_24_27( x )		(((ui32)(x) & 0x0f000000) >> 24)
#define	INSTRUC_23_27( x )		(((ui32)(x) & 0x0f800000) >> 23)
#define INSTRUC_22_24( x )		(((ui32)(x) & 0x01c00000) >> 22)
#define INSTRUC_21_27( x )		(((ui32)(x) & 0x0fe00000) >> 21)
#define INSTRUC_21_24( x )		(((ui32)(x) & 0x01e00000) >> 21)
#define INSTRUC_21_22( x )		(((ui32)(x) & 0x00600000) >> 21)
#define INSTRUC_20_27( x )		(((ui32)(x) & 0x0ff00000) >> 20)
#define INSTRUC_20_21( x )		(((ui32)(x) & 0x00300000) >> 20)
#define	INSTRUC_16_19( x )		(((ui32)(x) & 0x000f0000) >> 16)
#define	INSTRUC_13_15( x )		(((ui32)(x) & 0x0000e000) >> 13)
#define	INSTRUC_12_15( x )		(((ui32)(x) & 0x0000f000) >> 12)
#define	INSTRUC_11_15( x )		(((ui32)(x) & 0x0000f800) >> 11)
#define	INSTRUC_11_12( x )		(((ui32)(x) & 0x00001800) >> 11)
#define	INSTRUC_8_15( x )		(((ui32)(x) & 0x0000ff00) >> 8)
#define	INSTRUC_8_11( x )		(((ui32)(x) & 0x00000f00) >> 8)
#define	INSTRUC_7_15( x )		(((ui32)(x) & 0x0000ff80) >> 7)
#define	INSTRUC_7_11( x )		(((ui32)(x) & 0x00000f80) >> 7)
#define	INSTRUC_6_7( x )		(((ui32)(x) & 0x000000c0) >> 6)
#define	INSTRUC_5_6( x )		(((ui32)(x) & 0x00000060) >> 5)
#define INSTRUC_4_11( x )		(((ui32)(x) & 0x00000ff0) >> 4)
#define	INSTRUC_4_7( x )		(((ui32)(x) & 0x000000f0) >> 4)
#define	INSTRUC_4_6( x )		(((ui32)(x) & 0x00000070) >> 4)
#define	INSTRUC_3_6( x )		(((ui32)(x) & 0x00000078) >> 3)
#define	INSTRUC_3_5( x )		(((ui32)(x) & 0x00000038) >> 3)
#define INSTRUC_0_23( x )		(((ui32)(x) & 0x00ffffff) >> 0)
#define	INSTRUC_0_15( x )		(((ui32)(x) & 0x0000ffff) >> 0)
#define	INSTRUC_0_11( x )		(((ui32)(x) & 0x00000fff) >> 0)
#define	INSTRUC_0_10( x )		(((ui32)(x) & 0x000007ff) >> 0)
#define	INSTRUC_0_7( x )		(((ui32)(x) & 0x000000ff) >> 0)
#define	INSTRUC_0_4( x )		(((ui32)(x) & 0x0000001f) >> 0)
#define	INSTRUC_0_3( x )		(((ui32)(x) & 0x0000000f) >> 0)
#define	INSTRUC_0_2( x )		(((ui32)(x) & 0x00000007) >> 0)

#define PC_MASK( x )			((ui32)(x) & 0xfffffffc)
#define THUMB_PC_MASK( x )		((ui32)(x) & 0xfffffffe)



#define TRK_USR_MODE			0x0
#define TRK_FIQ_MODE			0x1
#define TRK_IRQ_MODE			0x2
#define TRK_SVC_MODE			0x3
#define TRK_ABT_MODE			0x7
#define TRK_UNDEF_MODE			0x0b
#define TRK_SYS_MODE			0x0f
												
																												
/* peices of instructions */
#define INSTRUC_Rd( x )		(INSTRUC_12_15( x ))
#define INSTRUC_Rn( x )		(INSTRUC_16_19( x ))
#define INSTRUC_Rm( x )		(INSTRUC_0_3( x ))
#define INSTRUC_Rs( x)		(INSTRUC_8_11( x ))
#define INSTRUC_SHIFT_IMM( x )  INSTRUC_7_11( x )
#define INSTRUC_ROT_IMM( x )	INSTRUC_8_11( x )
#define INSTRUC_IMMEDIATE( x )	INSTRUC_0_7( x )
#define INSTRUC_REG_LIST( x )	INSTRUC_0_15( x )
#define INSTRUC_OFFSET_12( x ) 	INSTRUC_0_11( x )

#define CARRY					((gCPUState.def[ TRK_DEFAULT_CPSR ] & 0x20000000) >> 29)

/* for addressing mode 1 */
#define IS_ADDR1_IMM( x )		 INSTRUC_BIT25( x )
#define IS_ADDR1_LSL_IMM( x )	(INSTRUC_4_6( x ) == 0x0)
#define IS_ADDR1_LSL_REG( x )	(INSTRUC_4_7( x ) == 0x1)
#define IS_ADDR1_LSR_IMM( x )	(INSTRUC_4_6( x ) == 0x2)
#define IS_ADDR1_LSR_REG( x )	(INSTRUC_4_7( x ) == 0x3)
#define IS_ADDR1_ASR_IMM( x )	(INSTRUC_4_6( x ) == 0x4)
#define IS_ADDR1_ASR_REG( x )	(INSTRUC_4_7( x ) == 0x5)
#define IS_ADDR1_ROR_IMM( x )	(INSTRUC_4_6( x ) == 0x6)
#define IS_ADDR1_ROR_REG( x )	(INSTRUC_4_7( x ) == 0x7)
#define IS_ADDR1_RORX( x )		(INSTRUC_4_11( x ) == 0x6)

	
/* for addressing mode 2 */
#define IS_ADDR2_IMM_OFF( x )		((INSTRUC_24_27( x ) == 0x5) && !INSTRUC_BIT21( x ))
#define IS_ADDR2_REG_OFF( x )		((INSTRUC_24_27( x ) == 0x7) && !INSTRUC_BIT21( x ) && (INSTRUC_4_11( x ) == 0x0))
#define IS_ADDR2_SCL_REG_OFF( x )	((INSTRUC_24_27( x ) == 0x7) && !INSTRUC_BIT21( x ) && !INSTRUC_BIT4( x ))
#define IS_ADDR2_IMM_PRE( x )		((INSTRUC_24_27( x ) == 0x5) &&  INSTRUC_BIT21( x ))
#define IS_ADDR2_REG_PRE( x )		((INSTRUC_24_27( x ) == 0x7) &&  INSTRUC_BIT21( x ) && (INSTRUC_4_11( x ) == 0x0))
#define IS_ADDR2_SCL_REG_PRE( x )	((INSTRUC_24_27( x ) == 0x7) &&  INSTRUC_BIT21( x ) && !INSTRUC_BIT4( x ))
#define IS_ADDR2_IMM_POST( x )		((INSTRUC_24_27( x ) == 0x4) && !INSTRUC_BIT21( x ))
#define IS_ADDR2_REG_POST( x )		((INSTRUC_24_27( x ) == 0x6) && !INSTRUC_BIT21( x ) && (INSTRUC_4_11( x ) == 0x0))
#define IS_ADDR2_SCL_REG_POST( x )	((INSTRUC_24_27( x ) == 0x6) && !INSTRUC_BIT21( x ) && !INSTRUC_BIT4( x ))

	
/*
** ARM instructions that affect the pc and therefore, can branch
*/

#define IS_EXTENDED_INSTR( x )		(!INSTRUC_BIT25( x ) && INSTRUC_BIT4( x ) && INSTRUC_BIT7( x ))


#define IS_INSTRUC_ADD_C( x )		((INSTRUC_22_24( x ) == 0x2 )  &&  (INSTRUC_26_27( x ) == 0x0) && \
								   	 	( INSTRUC_12_15( x ) == 0x0f)    && !IS_EXTENDED_INSTR( x ))
#define IS_INSTRUC_AND( x )			((INSTRUC_21_24( x ) == 0x0 )  &&  (INSTRUC_26_27( x ) == 0x0) && \
								   	 	( INSTRUC_12_15( x ) == 0x0f)    && !IS_EXTENDED_INSTR( x ))
#define IS_INSTRUC_B_BL( x )		( INSTRUC_25_27( x ) == 0x5)
#define IS_INSTRUC_BIC( x )			((INSTRUC_21_24( x ) == 0x0e)  &&  (INSTRUC_26_27( x ) == 0x0) && \
								    	( INSTRUC_12_15( x ) == 0x0f)    && !IS_EXTENDED_INSTR( x ))
#define IS_INSTRUC_BLX( x )			((INSTRUC_COND( x )  == 0x0f)  &&  (INSTRUC_25_27( x ) == 0x5))
#define IS_INSTRUC_BLX2_BX( x )		((INSTRUC_20_27( x ) == 0x12)  &&  (INSTRUC_6_7( x ) == 0x0) && \
										INSTRUC_BIT4( x ))
#define IS_INSTRUC_EOR( x )			((INSTRUC_21_24( x ) == 0x1 )  && \
									    ( INSTRUC_12_15( x ) == 0x0f)    && !IS_EXTENDED_INSTR( x ))
#define IS_INSTRUC_LDM( x )			((INSTRUC_25_27( x ) == 0x4 )  && \
									      INSTRUC_BIT15( x ) 		  &&  INSTRUC_BIT20( x ))
#define IS_INSTRUC_LDR( x )			((INSTRUC_26_27( x ) == 0x1 )  &&   INSTRUC_BIT20( x ) && \
										 !INSTRUC_BIT22( x )  		  && (INSTRUC_COND( x ) != 0x0f) && \
						  			       (INSTRUC_12_15( x ) == 0x0f))
#define IS_INSTRUC_MOV_MVN( x )		((INSTRUC_26_27( x ) == 0x0 )  &&  !IS_EXTENDED_INSTR( x ) && \
										((INSTRUC_21_24( x ) == 0x0d) || (INSTRUC_21_24( x ) == 0x0f)) && \
										   (INSTRUC_12_15( x ) == 0x0f))
#define IS_INSTRUC_ORR( x )			((INSTRUC_24_27( x ) == 0x0e)  &&  (INSTRUC_26_27( x ) == 0x0) && \
										( INSTRUC_12_15( x ) == 0x0f)	  &&  !IS_EXTENDED_INSTR( x ))	
#define IS_INSTRUC_RSB_C( x )		((INSTRUC_21_22( x ) == 0x03)  &&  (INSTRUC_26_27( x ) == 0x0) && \
										 !INSTRUC_BIT24( x ) 		  && (INSTRUC_12_15( x ) == 0x0f) && \
										   !IS_EXTENDED_INSTR( x ))
#define IS_INSTRUC_SUB_C( x )		((INSTRUC_21_22( x ) == 0x02)  &&  (INSTRUC_26_27( x ) == 0x0) && \
										 !INSTRUC_BIT24( x ) 		  && (INSTRUC_12_15( x ) == 0x0f) && \
										   !IS_EXTENDED_INSTR( x ))

									   
/*
** THUMB (16-bit) instructions that affect the pc and therefore, can branch
*/

#define IS_TINSTRUC_ADD4( x )		((INSTRUC_7_15( x ) == 0x89)   &&  (INSTRUC_0_2( x ) == 0x07))
#define IS_TINSTRUC_B1( x )			( INSTRUC_12_15( x ) == 0x0d)
#define IS_TINSTRUC_B2_BL_BLX( x )	( INSTRUC_13_15( x ) == 0x7)
#define IS_TINSTRUC_BLX2_BX( x )	( INSTRUC_8_15( x ) == 0x47)
#define IS_TINSTRUC_MOV3( x )		((INSTRUC_7_15( x ) == 0x8d )  &&  (INSTRUC_0_2( x ) == 0x07))
#define IS_TINSTRUC_POP( x )		((INSTRUC_8_15( x ) == 0x0bd))	

/* peices of instructions */
#define TINSTRUC_Rd( x )			INSTRUC_0_2( x )
#define TINSTRUC_RdH1( x )			(INSTRUC_0_2( x ) | (INSTRUC_BIT7( x ) << 3))
#define TINSTRUC_Rm( x )			INSTRUC_3_5( x )
#define TINSTRUC_RmH2( x )			(INSTRUC_3_5( x ) | (INSTRUC_BIT6( x ) << 3))
#define TINSTRUC_IMMEDIATE8( x )	INSTRUC_0_7( x )
#define TINSTRUC_IMMEDIATE11( x )	INSTRUC_0_10( x )
#define TINSTRUC_REG_LIST( x )		INSTRUC_0_7( x )


/*
** Construct a break instruction with the specified code embedded.
*/
#define BKPT_INSTRUC				0xe7fddefe //0xe1200070	
#define INSTR_BREAKC( x )		((((x) & 0x000fff0) << 4) | (((x) & 0x0000000f) << 0) | \
								 BKPT_INSTRUC )

/*
** Construct a THUMB break instruction with the specified code
** embedded.
*/
#define BRKPT16_INSTRUC				0xdefe //0xbe00
#define INSTR16_BREAKC( x )		(((x) & 0xff) | BRKPT16_INSTRUC)


#define IS_INSTRUC_BKPT( x )		(((x & BKPT_INSTRUC) == BKPT_INSTRUC) || ((x & BRKPT16_INSTRUC) == BRKPT16_INSTRUC))
/*
** Given the first 16-bit word of a THUMB instruction,
** determine whether it is a 32-bit THUMB instruction.
** They are the instructions with the BL or BLX1
** opcodes.
*/

#define IS_32BIT_INSTRUCTION( x )	(INSTRUC_11_15( x ) == 0x1e)


/*
** Determine whether the target program is currently
** running in THUMB mode (true <-> low-order bit of
** target program PC is 1).  This macro also works
** on the destination address for JR/JALR instructions
** to determine whether the jump destination is THUMB
** code.
*/

#define THUMB_MODE		(gCPUState.def[ TRK_DEFAULT_CPSR ] & 0x20)
#define THUMB_PC( pc )		((pc) & THUMB_PC_MASK)
