/****************************************************************************/
/*

DESCRIPTION

	PPC instruction constants & macros.
	
AUTHORS

	Copyright 2002, 2004, 2006, 2009 Freescale Semiconductor, Inc.
	
*/
/****************************************************************************/

#ifndef __ARM_INSTR_H__
#define __ARM_INSTR_H__

/*
** Instruction used for breakpoint.
*/

#define	ARM_BREAK			0xEF9F0001	// xxxx1111 yyyyyyyy yyyyyyyy yyyyyyyy

/*
** Extract the primary opcode from an instruction.  It is in
** bits 0-5.
*/

#define ARM_OPCODE_MASK		0x0E000000
#define ARM_OPCODE( x )		(((ui32)(x) & ARM_OPCODE_MASK) >> 25)

/*
** Conditional Code.
*/

#define ARM_COND_MASK		0xF0000000
#define ARM_COND( x )		(((ui32)(x) & ARM_COND_MASK) >> 28)

/*
** Useful opcode values.
*/

#define ARM_OP_DATA_REG		0		// Data Processing Instructions
#define ARM_OP_DATA_IMM		1

#define ARM_OP_SINGLE_IMM	2		// Single Data Transfer 
#define ARM_OP_SINGLE_REG	3

#define ARM_OP_BLOCK		4		// Block Data Transfer 

#define ARM_OP_BRANCH		5		// Branch Instruction
	
// Generic instructin define
#define ARM_RM(x)			((ui32)(x) & 0x0000000F)			// bit  0- 4
#define ARM_RS(x)			(((ui32)(x) & 0x00000F00) >> 8)		// bit  8-11
#define ARM_RD(x)			(((ui32)(x) & 0x0000F000) >> 12)	// bit 12-15
#define ARM_RN(x)			(((ui32)(x) & 0x000F0000) >> 16)	// bit 16-19
#define ARM_LOAD(x)			(((ui32)(x) & 0x00100000) >> 20)	// bit 20

// Data processing instructin define
#define ARM_DATA_SHIFT(x)	(((ui32)(x) & 0x00000060) >> 5) 	// bit  5- 6
#define ARM_DATA_C(x)		(((ui32)(x) & 0x00000F80) >> 7) 	// bit  7-11
#define ARM_DATA_IMM(x)		((ui32)(x) & 0x000000FF)			// bit  0-7
#define ARM_DATA_ROT(x)		(((ui32)(x) & 0x00000F00) >> 8) 	// bit  8-11
#define ARM_DATA_A(x)		(((ui32)(x) & 0x01E00000) >> 21)	// bit 21-24

// Single date transfer instructin define
#define ARM_SINGLE_IMM(x)	((ui32)(x) & 0x00000FFF)			// bit  0-11
#define ARM_SINGLE_BYTE(x)	(((ui32)(x) & 0x00400000) >> 22)	// bit 22
#define ARM_SINGLE_U(x)		(((ui32)(x) & 0x00800000) >> 23)	// bit 23
#define ARM_SINGLE_PRE(x)	(((ui32)(x) & 0x01000000) >> 24)	// bit 24

// Block data transfer instructin define
#define ARM_BLOCK_REGLIST(x)	((ui32)(x) & 0x0000FFFF)		// bit  0-15
#define ARM_BLOCK_U(x)		(((ui32)(x) & 0x00800000) >> 23)	// bit 23
#define ARM_BLOCK_PRE(x)	(((ui32)(x) & 0x01000000) >> 24)	// bit 24

// Multiplicatoin instructin define
#define ARM_OP_MULTIPLY(x)	(((ui32)(x) & 0x0F600000) >> 22) == 0 && (((ui32)(x) & 0x000000F0) >> 4)

// Branch instructin define
#define ARM_B_ADDR_MASK		0x00FFFFFF
#define ARM_B_ADDR(x)		((x & 0x00800000) ? ((ui32)(x) & ARM_B_ADDR_MASK | 0xFF000000) : (ui32)(x) & ARM_B_ADDR_MASK)

#define ARM_INSTR_B_DEST( x, a )	( ARM_B_ADDR(x) << 2 ) + ((ui32)(a) + 8)

/*
** Extract the LK bit from an instruction.  This is valid for BX, BCX,
** BCCTRX, and BCCLRX instructions.  It is in bit 31.
*/

#define ARM_B_LK_MASK		0x01000000
#define ARM_B_LK( x )		(((ui32)(x) & ARM_B_LK_MASK) >> 24)

/*
** Identify BX and BLX(2) instructions.  These are special case instructions.  Their primary
** opcode matches the ARM_OP_DATA_REG opcode, but they do not fall into that class of
** instructions.
*/

#define ARM_OP_BX_BLX2( x )	(((ui32)(x) & 0x0ff000d0) == 0x01200010)

/*
** Extract the LK bit from a BX or BLX instruction.
*/

#define ARM_BX_LK( x )		(((ui32)(x) & 0x00000020) >> 5)

#endif	// #ifndef __ARM_INSTR_H__
