/****************************************************************************/
/*

DESCRIPTION

	API for target-specific implementation.
	
	Register numbers for ARM 920T Integrator/AP Board.
	
	Registers are accessed via the kDSReadRegisters and kDSWriteRegisters
	messages.  The register numbers for all registers are defined 
	below.  The registers are rougly categorized into blocks.
	All registers are 32 bits, so register data in messages is transmitted
	as a series of 32 bit values in big-endian format.
	
AUTHORS

	Copyright 2002-2004, 2006, 2009 Freescale Semiconductor, Inc.
	
*/

/****************************************************************************/

#ifndef _ARM_REG_H_
#define _ARM_REG_H_

#include "dstypes.h"
#include "arm_defines.h"

/* YOU MAY ALSO NEED TO MODIFY target_asm.s TO REFLECT ANY CHANGES TO THIS FILE */


/****************************************************************************/
/*
 * Default Register Block
 *
 * Note: accessing registers that are not supported on the current
 * processor has no effect and does not result in an error.
 */
/****************************************************************************/

/****************************************************************************/
/*
 *	USR/SYS Mode and non-banked Registers
 *
 *	Note: 
 */
/****************************************************************************/

typedef ui32	DefaultType;

#define TRK_DEFAULT_SIZE			(sizeof( DefaultType ))

#define TRK_DEFAULT_GPR( x )		(x)	
#define TRK_DEFAULT_SP				13	
#define TRK_DEFAULT_LR				14
#define TRK_DEFAULT_PC				15
#define TRK_DEFAULT_CPSR			16

#define ARM_FLAG_C					0x20000000	// bit 30

#define TRK_DEFAULT_MODE_MIN_REGISTER	TRK_DEFAULT_GPR( 0 )
#define TRK_DEFAULT_MODE_MAX_REGISTER	TRK_DEFAULT_CPSR	

/****************************************************************************/
/*
 *	FIQ Mode Registers
 *
 *	Note: 
 */
/****************************************************************************/

#define TRK_FIQ_GPR_8_12( x )	(x+17)	
#define TRK_FIQ_SP				22
#define TRK_FIQ_LR				23
#define TRK_FIQ_SPSR			24

#define TRK_FIQ_MIN_REGISTER	TRK_FIQ_GPR_8_12( 0 )
#define TRK_FIQ_MAX_REGISTER	TRK_FIQ_SPSR	

/****************************************************************************/
/*
 *	IRQ Mode Registers
 *
 *	Note: 
 */
/****************************************************************************/

#define TRK_IRQ_SP				25
#define TRK_IRQ_LR				26
#define TRK_IRQ_SPSR			27

#define TRK_IRQ_MIN_REGISTER	TRK_IRQ_SP
#define TRK_IRQ_MAX_REGISTER	TRK_IRQ_SPSR	

/****************************************************************************/
/*
 *	Supervisor Mode Registers
 *
 *	Note: 
 */
/****************************************************************************/

#define TRK_SVC_SP				28
#define TRK_SVC_LR				29
#define TRK_SVC_SPSR			30

#define TRK_SVC_MIN_REGISTER	TRK_SVC_SP
#define TRK_SVC_MAX_REGISTER	TRK_SVC_SPSR	

/****************************************************************************/
/*
 *	Abort Mode Registers
 *
 *	Note: 
 */
/****************************************************************************/

#define TRK_ABT_SP				31
#define TRK_ABT_LR				32
#define TRK_ABT_SPSR			33

#define TRK_ABT_MIN_REGISTER	TRK_ABT_SP
#define TRK_ABT_MAX_REGISTER	TRK_ABT_SPSR

/****************************************************************************/
/*
 *	Undefined Mode Registers
 *
 *	Note: 
 */
/****************************************************************************/

#define TRK_UNDEF_SP				34
#define TRK_UNDEF_LR				35
#define TRK_UNDEF_SPSR				36

#define TRK_UNDEF_MIN_REGISTER	TRK_UNDEF_SP
#define TRK_UNDEF_MAX_REGISTER	TRK_UNDEF_SPSR

/****************************************************************************/
/*
 *	Exception CPSR 
 *
 *	Note: This is a pseudo register for tracking the source of an exception
 */
/****************************************************************************/ 
#define TRK_EXCEP_CPSR				37



#define TRK_DEFAULT_MIN_REGISTER	TRK_DEFAULT_GPR( 0 )
#define TRK_DEFAULT_MAX_REGISTER	TRK_EXCEP_CPSR	
/* End of Block */


/****************************************************************************/
/*
 * Extended registers block 1.
 *
 * Note: accessing registers that are not supported on the current
 * processor has no effect and does not result in an error.
 */
/****************************************************************************/

typedef ui32	Extended1Type;
typedef ui32	Extended2Type;

#define TRK_EXTENDED1_SIZE( x )		(sizeof( Extended1Type ))
#define TRK_EXTENDED2_SIZE( x )		(sizeof( Extended2Type ))

#define TRK_EXTENDED2_CP15_0		0		/* Register 0, ID Code, read only */
#define TRK_EXTENDED2_CP15_1		1		/* Register 0, Cache Type, read only */
#define TRK_EXTENDED2_CP15_2		2		/* Register 1, Control Register */
#define TRK_EXTENDED2_CP15_3		3		/* Register 2, Translation Table Base */
#define TRK_EXTENDED2_CP15_4		4		/* Register 3, Domain Access Control */
#define TRK_EXTENDED2_CP15_5		5		/* Register 5, Fault Status - Data */
#define TRK_EXTENDED2_CP15_6		6		/* Register 5, Fault Status - Prefetch  */
#define TRK_EXTENDED2_CP15_7		7		/* Register 6, Fault Address */
#define TRK_EXTENDED2_CP15_8		8		/* Register 9, DCache Lockdown Base */
#define TRK_EXTENDED2_CP15_9		9		/* Register 9, ICache Lockdown Base */
#define TRK_EXTENDED2_CP15_10		10		/* Register 10, D TLB Lockdown */
#define TRK_EXTENDED2_CP15_11		11		/* Register 10, I TLB Lockdown */
#define TRK_EXTENDED2_CP15_12		12		/* Register 13, FCSE PID */

#define TRK_EXTENDED2_CP15_13		13		/* Register 7, Invalidate ICache and DCache, write only */
#define TRK_EXTENDED2_CP15_14		14		/* Register 7, Invalidate ICache, write only */
#define TRK_EXTENDED2_CP15_15		15		/* Register 7, Invalidate ICache, single entry MVA, write only */
#define TRK_EXTENDED2_CP15_16		16		/* Register 7, Prefetch ICache Line MVA, write only */
#define TRK_EXTENDED2_CP15_17		17		/* Register 7, Invalidate DCache, write only */
#define TRK_EXTENDED2_CP15_18		18		/* Register 7, Invalidate DCache, single entry MVA, write only */
#define TRK_EXTENDED2_CP15_19		19		/* Register 7, Clean DCache, single entry MVA, write only */
#define TRK_EXTENDED2_CP15_20		20		/* Register 7, Clean and Invalidate DCache entry MVA, write only */
#define TRK_EXTENDED2_CP15_21		21		/* Register 7, Clean DCache, single entry index, write only */
#define TRK_EXTENDED2_CP15_22		22		/* Register 7, Clean and Invalidate DCache entry index, write only */
#define TRK_EXTENDED2_CP15_23		23		/* Register 7, Drain Write Buffer, write only */
#define TRK_EXTENDED2_CP15_24		24		/* Register 7, Wait for Interrupt, write only */

#define TRK_EXTENDED2_CP15_25		25      /* Register 8, Invalidate TLB, write only */
#define TRK_EXTENDED2_CP15_26		26		/* Register 8, Invalidate I TLB, write only */
#define TRK_EXTENDED2_CP15_27		27		/* Register 8, Invalidate I TLB, single entry MVA, write only */
#define TRK_EXTENDED2_CP15_28		28		/* Register 8, Invalidate D TLB, write only */
#define TRK_EXTENDED2_CP15_29		29		/* Register 8, Invalidate D TLB, single entry MVA, write only */

#define TRK_EXTENDED2_CP15_30		30      
#define TRK_EXTENDED2_CP15_31		31		
#define TRK_EXTENDED2_CP15_32		32		
#define TRK_EXTENDED2_CP15_33		33		
#define TRK_EXTENDED2_CP15_34		34		
#define TRK_EXTENDED2_CP15_35		35		
#define TRK_EXTENDED2_CP15_36		36	

#define TRK_EXTENDED2_CP15_37		37		
#define TRK_EXTENDED2_CP15_38		38		
#define TRK_EXTENDED2_CP15_39		39	


#define TRK_EXTENDED2_MIN_REGISTER	TRK_EXTENDED2_CP15_0

#ifdef intel_80321
	#define TRK_EXTENDED2_MAX_REGISTER	TRK_EXTENDED2_CP15_36
#elif iMX21_board 
	#define TRK_EXTENDED2_MAX_REGISTER	TRK_EXTENDED2_CP15_39
#else
	#define TRK_EXTENDED2_MAX_REGISTER	TRK_EXTENDED2_CP15_29
#endif
 
/* End of Block */



/****************************************************************************/
/*
 * Extended registers block 2.
 *
 * Note: accessing registers that are not supported on the current
 * processor has no effect and does not result in an error.
 */
/****************************************************************************/

/****************************************************************************/
/*
 * Floating point register block
 *
 * Note: These blocks are currently not used
 */
/****************************************************************************/

typedef ui32 FPType;

#define TRK_FP_SIZE( x )			(sizeof( FPType ))

#define TRK_FP_FPR( x )				(x)
#define TRK_FP_FPSID				32
#define TRK_FP_FPSCR				33
#define TRK_FP_FPEXC				34	/* read-only */

/****************************************************************************/
/*
 * Double register block
 *
 * Note: These blocks are currently not used
 */
/****************************************************************************/

typedef ui64	DoubleType;

#define TRK_DOUBLE_SIZE( x )		(sizeof( DoubleType ))

#define TRK_DOUBLE_FPR( x )			(x+35)
#define TRK_DOUBLE_FPSID_D			51
#define TRK_DOUBLE_FPSCR_D			52
#define TRK_DOUBLE_FPEXC_D			53	/* read-only */

#define TRK_EXTENDED1_MIN_REGISTER			TRK_FP_FPR( 0 )
#define TRK_EXTENDED1_MAX_REGISTER			TRK_DOUBLE_FPEXC_D
/* End of Block */
#define T_Bit				0x20



/****************************************************************************/
/*
 * Info stored in NotifyStopped message.
 */
/****************************************************************************/


typedef union
{
	ui32	i32;
	ui16	i16[2];
} InstructionType;

typedef ui16		ExceptionCauseType;
#ifndef PCType
typedef ui32		PCType;
#endif
typedef struct StopInfo
{
	PCType				PC;
	InstructionType		PCInstruction;			/* instruction at PC */
	ExceptionCauseType	CR;
} StopInfo;



/****************************************************************************/
/*
 * Info stored in NotifyException message.
 */
/****************************************************************************/
 
typedef StopInfo ExceptionInfo;

#endif


