/****************************************************************************/
/*

DESCRIPTION

	API for target-specific implementation.
	
	Register numbers for M683xx (DragonBall)/ColdFire 
	
	These register blocks are accessed via the kDSReadRegisters and
	kDSWriteRegisters messages.  The register number for a specific
	register is determined by its corresponding element number in these
	structures.  The register data in the messages must conform to
	the structures defined here in big-endian format.
	
AUTHORS

	Copyright 2003, 2006, 2009 Freescale Semiconductor, Inc.
	
*/
/****************************************************************************/

#ifndef TRK_M68K_REG_H
#define TRK_M68K_REG_H

/****************************************************************************/

//	68K instructions are variable length, 1 - 7 words (CPU32 man pg 3-12)


typedef ui32	DefaultType;
typedef ui32	Extended1Type;
typedef ui64	FloatType;
typedef FloatType FPType;	/* TRK core calls it FPType */

/* Default register block (all regs typically requested by debugger) */

typedef struct Default_68K
{
	ui32	Di[8], Ai[8], PC, USP, SSP;
	ui16	filler1;				// so TargetAccessDefault works for SR
	ui16	SR;
	ui16	filler2;				// so TargetAccessDefault works for SR
	ui16	exceptionID;
} Default_68K;

/* Define indices if the above struct were an array of DefaultType */
#define TRK_DEFAULT_DI( x )			(x)
#define TRK_DEFAULT_AI( x )			((x) + 7)
#define TRK_DEFAULT_PC				18
/*16*/
#define TRK_DEFAULT_USP				15
/*17*/
#define TRK_DEFAULT_SSP				15
#define TRK_DEFAULT_SR				17
/*19*/
#define TRK_DEFAULT_EXCEPTIONID		20

#define TRK_DEFAULT_MIN_REGISTER		(TRK_DEFAULT_DI( 0 ))
#define TRK_DEFAULT_MAX_REGISTER		(TRK_DEFAULT_PC)

// Floating point register block not used for m683xx

/* Extended registers block 1 not used for m683xx */

/* full processor state */

typedef struct ProcessorState_68K
{
	Default_68K		Default;		/* Default register block */
} ProcessorState_68K;

/* Info stored in NotifyStopped message */
 
//typedef struct { ui16 inst[7]; }	InstructionType;
typedef ui16	InstructionType;
typedef ui16	BreakType;

typedef DefaultType		PCType;
typedef ui16			ExceptionCauseType;

typedef InstructionType BreakpointRestore;
#define TRK_DEFAULT_SIZE	(sizeof( DefaultType ))
#define TRK_FLOAT_SIZE		(sizeof( FloatType ))

typedef struct StopInfo
{
	PCType				PC;
	ExceptionCauseType	exceptionID;			/* fmtOff ?? */
	InstructionType		PCInstruction;			/* instruction at PC */
} StopInfo;

/* Info stored in NotifyException message */
 
typedef StopInfo ExceptionInfo;


/*
** Useful 68K register defines.
*/

#define SR_TRACE		0x8000
#define SR_SUP			0x2000
#define SR_I2			0x0400
#define SR_I1			0x0200
#define SR_I0			0x0100

#endif /* ndef TRK_M68K_REG_H */
