/******************************************************************************************/
/*
	
DESCRIPTION

	Linux ELF helper routines.
	
COPYRIGHT

	Copyright 2001, 2002, 2004, 2006, 2009 Freescale Semiconductor, Inc.
	
*/
/******************************************************************************************/

#ifndef _LINUX_ELF_H_
#define _LINUX_ELF_H_

/****************************************************************************/
/*
 *	Includes
 */
/****************************************************************************/

#include "dstypes.h"
#include "dserror.h"

#include <elf.h>
#include <sys/procfs.h>
#define _SYS_ELF_H	1		/* prevents libelf from including obsolete file */
#include <libelf/libelf.h>

/****************************************************************************/
/*
 *	Errors
 */
/****************************************************************************/

/****************************************************************************/
/*
 *	Types
 */
/****************************************************************************/

typedef struct TrkLinuxElf
{
	int			fFd;					/* file descriptor */
	Elf			*fElf;					/* libelf handle */
} TrkLinuxElf;

/****************************************************************************/
/*
 *	Public Function Prototypes
 */
/****************************************************************************/

#define __elf_perror( s )			{ \
										const char* errstr; \
										__puts( s ); \
										__puts( ": " ); \
										errstr = elf_errmsg( elf_errno() ); \
										if ( !errstr ) errstr = "unknow"; \
										__puts( errstr ); \
										__puts( "\r\n" ); \
									}


__extern_c

	DSError TrkOpenElfLinux( char *name, TrkLinuxElf *tlElf );

	DSError TrkCloseElfLinux( TrkLinuxElf *tlElf );

	void TrkGetElfSectionRangeLinux( Elf *elf, char *name, void *base,
					PCType *secStart, PCType *secEnd );

	Elf_Scn *TrkFindElfSectionLinux( Elf *elf, char *name );

	Elf32_Sym *TrkFindElfSymbolLinux( Elf *elf, const char *name );

	Elf32_Addr TrkGetElfSymbolOffset( Elf *elf, Elf32_Sym *esym );

__end_extern_c

#endif /* #ifndef _LINUX_ELF_H_ */
