/******************************************************************************************/
/*
	
DESCRIPTION

	main()

COPYRIGHT

	Copyright 2000-2002, 2004-2006, 2009 Freescale Semiconductor, Inc.
	
*/
/******************************************************************************************/
#include "trk_prefix.h"

#include "target.h"
#include "usr_put.h"
#include "msgcmd.h"
#include "main_TRK.h"
#include "dserror.h"
#include "linux_main.h"
#include "linux_comm.h"

#include <setjmp.h>
#include <string.h>

#ifdef __uClinux__

unsigned int gRamsize = 0;

#define MEM_REMIND_MSG "The uClinux doesn't contain the memory protect mechanisim \
in ptrace system call. Thus will cause the CodeWarriorTRK application crash when attemping \
to access a bad memory address.Till now, we have a work around by reading the memory \
infomation stored in ring buffer (which can be get via dmesg) when the kernel start up.\
But the content in ring buffer maybe changed/erased, in this case,we must input the \
highest RAM memeory address manully.\n"

#define RESTART_REMIND_MSG "Please start the CodeWarriorTRK application again with the \
highest RAM address specified. For example, if your ColdFire board has 4M RAM, please input: \
\n\"%s [-v] [:tcpport | serialdevice | '|'] 0x400000\" or \
\n\"%s [-v] [:tcpport | serialdevice | '|'] 4194304\"\n "

#define BADRAM_INPUT_MSG "You didn't input a valid highest RAM address, \
Thus there will be no memory protection in ptrace system call. The CodeWarriorTRK \
Application will crash when it attempts to access an invalid memory location\n"

#endif	//ifdef __uClinux__

static void usage( char *name );
static void version( void );


/****************************************************************************/
/*
 *	main
 *
 *	All of the portable code is in TRK_main().  Linux-specific code goes here.
 */
/****************************************************************************/

int main( int argc, char *argv[] )
{
	DSError err = kNoError;
	int i;
	char *comm_spec = NULL;
	bool ver_switch = false;

#ifdef	__uClinux__
	char *mem_spec = NULL;
#endif	

	/*
	** Iterate through the arguments.
	*/
	
	for (i = 1; i < argc && err == kNoError; i++)
	{
		if (*argv[ i ] == '-')
		{
			if (*(argv[ i ] + 1) == 'v')
				ver_switch = true;
			else
				err = kUserError;
		}
		else
		{
			/*
			** The first non-switch argument specifies the communication channel
			** to use.
			*/
			if (comm_spec == NULL)
				comm_spec = argv[ i ];
#ifdef	__uClinux__
			/*
			** The second non-switch argument specifies the highest RAM address
			** which can be accessed via ptrace function call.
			*/
			else if (mem_spec == NULL)
				mem_spec = argv[i];
#endif
			else
				err = kUserError;
		}
	}
	
	/*
	** Error if nothing was specified.
	*/
	if (!ver_switch && comm_spec == NULL)
		err = kUserError;
	
#ifdef __uClinux__
	if ( err == kNoError )
	{
		if ( !mem_spec ) 	//we didn't specify the highest RAM address
		{			
			err = OsGetPhysicalMemMaxAddress(&gRamsize);	
			if (err == kUserError)	//if err here, that means we must specify the 
									//highest memory address, just exit here.
			{
				printf(MEM_REMIND_MSG);
				printf(RESTART_REMIND_MSG,argv[0],argv[0]);
				return -1;
			}
		}
		else
		{
			gRamsize = strtol(mem_spec,NULL,0);
			if ( !gRamsize )	//input 0 or some invalid argument. 
			{
				printf(BADRAM_INPUT_MSG);
			}
			else
			{
				printf("The input higest RAM address is %u\n", gRamsize);

			}
		}
	}
#endif
	
	/*
	** Display the version and/or open the communication channel &
	** launch TRK.
	*/
	if (err == kNoError)
	{
		if (ver_switch)
			version();
		
		if (comm_spec != NULL)
		{
			err = TrkSetComm( comm_spec );
			if (err == kNoError)
				TRK_main();
		}
	}

	/*
	** If there was a problem with the arguments, print usage.
	*/
	if (err == kUserError)
		usage( argv[0] );

	return( (err == kNoError) ? 0 : -1 );
}

/****************************************************************************/
/*
 *	usage
 *
 *	Display the correct usage.
 */
/****************************************************************************/

void usage( char *name )
{
	char *pname;
	pname = strrchr( name, '/' );
	if (pname == NULL)
		pname = name;
	else
		pname++;
	printf( "Usage: %s [-v] [:tcpport | serialdevice | '|']\r\n", pname );
}

/****************************************************************************/
/*
 *	version
 *
 *	Display the current version.
 */
/****************************************************************************/

void version( void )
{
	printf( "CodeWarrior Target Resident Kernel for %s\n", DS_TARGET_NAME );
	printf( "Version %s implementing CodeWarriorTRK API version %s\n",
		DS_KERNEL_VERSION_STRING, DS_PROTOCOL_VERSION_STRING );
}


#ifdef __uClinux__

/****************************************************************************/
/*
 *	OsGetPhysicalMemRange
 *
 *	If we use uClinux without mmu, we must get the pysical memory range first
 *	to avoide trk access bad memory address via ptrace.( In uClinux, it will 
 *	cause the AppTrk crash, in linux with mmu, the ptrace just return error.)
 */
/****************************************************************************/
int OsGetPhysicalMemRange(int* _ramsize )
{
	int ramsize, bufsize, readsize;
	char *buf;
	char *loc;
	bool found = false;
	static char MatchString[] = "Memory available:";

	ramsize = 0;
	bufsize = readsize = 4096;

	buf = (char*) malloc(bufsize);
	//The memory message should be the following format:
	//"Memory available: 1940k/4096k RAM, 0k/0k ROM (737k kernel code, 212k data)"
	while ( !found )
	{
		if ((readsize = klogctl(3, buf, bufsize)) < 0)
		{			
			__puts("Error calling klogctl() in OsGetPhysicalMemRange\n");
		}
		
		loc = strstr(buf, MatchString);
		if ( loc )
		{
			char intString[32];
			int index = 0;
			while ( *loc != '/' )
				loc++;
			loc++;
					
			while ( *loc >='0' && *loc <='9' )
			{
				intString[index] = *loc;
				loc++;
				index++;
			}
			
			intString[index] = '\0';
			ramsize = atoi(intString) * 1024; 
			found = true;
		}
		
		if ( readsize != bufsize )//means read all the kernel message buffer
			break;
		
		if ( !found )
		{
			bufsize += 4096;
			realloc(buf,bufsize);
		}
		
	}
	
	free(buf);

	if ( !found )
	{
		return kUserError;			
	}
	
	*_ramsize = ramsize;
		

	return kNoError;
}

/****************************************************************************/
/*
 *	OsGetPhysicalMemMaxAddress
 *
 *	If we use uClinux without mmu, we must get the pysical memory offset 
 *	(the address where the SDRAM is mapped) and add it to the physical mem size
 *  in order to obtain the highest memory address for comparison with addresses
 *	used throughout AppTrk.
 */
/****************************************************************************/
int OsGetPhysicalMemMaxAddress(int* _address )
{
	int offset, bufsize, readsize;
	char *buf;
	char *loc;
	bool found = false;
	static char MatchString[] = "Freeing unused kernel memory:";
	DSError error = kNoError;
	unsigned int size = 0;
	int mask = 0xF0000000;
	char i = 6;
	char *endptr;
	
	error = OsGetPhysicalMemRange (&size);
	
	if (error != kNoError)
		return error;
	
	while (!(size & mask) && (i != -1))
	{	
		mask += (16^(i+1)-16^(i));
		i--;
	}
	
	mask -= (16^(i+2)-16^(i+1));
	
	offset = 0;
	bufsize = readsize = 4096;

	buf = (char*) malloc(bufsize);
	//The Kernel free memory message should be the following format:
	//"Freeing unused kernel memory: 24k freed (0xee000 - 0xf3000)"
	while ( !found )
	{
		if ((readsize = klogctl(3, buf, bufsize)) < 0)
		{			
			__puts("Error calling klogctl() in OsGetPhysicalMemMaxAddress\n");			
		}
		loc = strstr(buf, MatchString);
		if ( loc )
		{
			char intString[32];
			int index = 0;
			while ( *loc != '(' )
				loc++;
			loc++;
					
			while (( *loc >='0' && *loc <='9' ) || ( *loc >='a' && *loc <='f' ) || (*loc == 'x'))
			{
				intString[index] = *loc;
				loc++;
				index++;
			}
			
			intString[index] = '\0';
			offset = strtol(intString, endptr, 16);
			found = true;		
		}
		
		if ( readsize != bufsize )//means read all the kernel message buffer
			break;
		
		if ( !found )
		{
			bufsize += 4096;
			realloc(buf,bufsize);
		}
		
	}
	
	free(buf);

	if ( !found )
	{
		return kUserError;			
	}
	
		*_address = (offset & mask) + size; //the highest memory address
	
	return kNoError;
}
#endif //__uClinux__
