/******************************************************************************************/
/*
	
DESCRIPTION

	TRK command handlers for Linux.
	
COPYRIGHT

	Copyright 2000-2006, 2009 Freescale Semiconductor, Inc.
	
*/
/******************************************************************************************/

/* Linux-specific global message descriptors initialized here */
#define TRK_LINUX_MSGCMD_INIT

#include "trk_prefix.h"

/*
**	portable includes
*/
#include "usr_put.h"	/* must appear before msgcmd.h and linux_msgtarg.h */
#include "nubassrt.h"
#include "nubevent.h"
#include "msg.h"
#include "msghndlr.h"
#include "msgparse.h"
#include "linux_msgtarg.h"
#include "linux_msghndlr.h"
#include "linux_targimpl.h"
#include "linux_so.h"		// for TrkSetSoBreakpointsLinux()

/*
**	portable includes for target-specific implementation
*/
#include "targimpl.h"
#include "usr_put.h"

/*
** Linux incudes
*/
#include <sys/ptrace.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>			// for malloc()
#include <string_TRK.h>
#ifndef __uClinux__
#include <pty.h>
#endif
#include <termios.h>
#include <support.h>		// for SuppAccessFile()

/*
** Set this to 1 to enable the use of pseudo-terminals for shunting
** console I/O to the debugger.  If this is disabled, standard pipes
** will be used instead.  Pseudo-terminals are preferred over pipes
** because their behavior is much closer to an actual command line.
*/

#define TRK_LINUX_USE_PTY		(1)


/****************************************************************************/
/*
 *	DoOsItemRequest
 *
 *	Linux supports creation of new processes (exec) and process 
 *	attachments and deletion of processes (kill), process attachments,
 *	and threads (kill);
 *
 */
/****************************************************************************/
DSError DoOsItemRequest( MessageBuffer *b )
{
	DSError err = kNoError;

	/*
	** Parse the item header segment of the message.  This
	** will read the item type from the buffer and store it
	** in the field value array.
	*/

	err = TrkParseMessageSegment( b, &gTrkMsgItemHeader, true );
	
	/*
	** Based on request and item type, dispatch to secondary handling
	** function.
	*/
	
	if (err == kNoError)
	{
		switch( b->fFields.fData[ TRK_MSG_COMMAND_INDEX ] )
		{
			case kDSOSCreateItem:
				switch( b->fFields.fData[ TRK_MSG_ITEM_HEADER_TYPE_INDEX ] )
				{
					case kDSOSProcessItem:
						return( DoOsCreateProcess( b ) );
					case kDSOSProcAttachItem:
						return( DoOsCreateProcAttach( b ) );
					default:
						__puts( "\r\nUnsupported item creation type\r\n" );
						err = kUnsupportedError;
				}
				break;
			case kDSOSDeleteItem:
				switch( b->fFields.fData[ TRK_MSG_ITEM_HEADER_TYPE_INDEX ] )
				{
					case kDSOSProcessItem:
						return( DoOsDeleteProcess( b ) );
					case kDSOSProcAttachItem:
						return( DoOsDeleteProcAttach( b ) );
					case kDSOSThreadItem:
						return( DoOsDeleteThread( b ) );
					default:
						__puts( "\r\nUnsupported item deletion type\r\n" );
						err = kUnsupportedError;
				}
				break;
			default:
				__puts( "\r\nNon-item request in DoOsItemRequest()\r\n" );
				err = kInternalError;
		}
	}
	
	/*
	** It is an error to reach this point.
	*/
	return( StandardACK(b, kDSReplyACK, TrkStandardReplyError( err )) );
}


#ifdef __uClinux__
/****************************************************************************/
/*
 *	DoOsQuery
 *
 *	uCLinux supports query the process text segment start address,
 *	data segment start address and the stack segment start address. Then report 
 *  to Debugger.
 */
/****************************************************************************/
#define IS_TEXT			25
#define IS_DATA			(IS_TEXT + 1)
#define IS_STACK		(IS_DATA + 1)

DSError DoOsQuery( ui32 processId,ui32 *text_start,ui32 *data_start,ui32 *stack_start)
{
	DSError err = kNotFoundError;
	char   buf[400],*r;
	int    fp;
	size_t len,i,j;
	ui32	get_offset;

	*text_start = 0;
	*data_start = 0;
	*stack_start = 0;
	
	r = buf;
	/* Open the Proc stat file for get the infomation needed*/
	sprintf(buf,"/proc/%lu/stat",processId);
	fp = open((const char *)buf, O_RDONLY);
	if ((fp == 0) || (fp == 0xFFFFFFFF)) 
		return 	err;

	/* Read the file till empty*/
	len = read(fp,buf,400);
	/* Close the opened file */
	if (fp)
		close(fp);

	j = 0;
	for (i = 0; i< len ; i++)
	{
		if ((*r++) == ' ')
		{
			j++;
			if (j == IS_TEXT)
			{
				sscanf(r,"%lu ",&get_offset);
				*text_start = get_offset;
			}
			if (j == IS_DATA)
			{
				//this is the code_end wich does not always equal data_start
				sscanf(r,"%lu ",&get_offset);
				
				*data_start = ptrace( PTRACE_PEEKUSER, processId, 200 );
				//see linux/arch/m68knommu/kernel/ptrace.c, sys_ptrace
				//(200>>2 == 50) => ptrace will return the data_offset
			}
			if (j == IS_STACK)
			{
				sscanf(r,"%lu ",&get_offset);
				*stack_start = get_offset;
				err = kNoError;
				break;
			}
		}
	}

	return err;
}
#endif

/****************************************************************************/
/*
 *	DoOsCreateProcess
 *
 *	Create (launch) a new process.
 *
 */
/****************************************************************************/
DSError DoOsCreateProcess( MessageBuffer *b )
{
	DSError err = kNoError;
	ui8 *msgData = NULL;
	char **argv = NULL;
	char **envp = NULL;
	char *consoleNames[kConsoleCount];
	char *wdir = NULL;
	int targetFds[kConsoleCount];
	int trkFds[kConsoleCount];
	int tempFds[2];
	unsigned int msgLength;
	unsigned int dataPos;
	int argc = 0;
	int envc = 0;
	int conc;
	int i;
	int j;
	int oserr;
	int status;
	bool sepFound;
	bool prevNull;
	bool endFound;
#ifndef __uClinux__
	ui32 processId = 0;
#else
	ui32 text_off;
	ui32 processId;
	ui32 data_off;
	ui32 stack_off;
#endif		
	ui32 tempId;
	bool ptyOpen;
	struct termios tio;
	LinuxProcessState *pState;
	LinuxThreadState *tState;


	/*
	** Parse the create-process segment of the message.  This
	** will pull out the options and length field values.  The
	** data field is handled below.
	*/

	err = TrkParseMessageSegment( b, &gTrkMsgCreateProcessLinux, true );
	msgLength = b->fFields.fData[ TRK_MSG_CP_LINUX_LENGTH_INDEX ];

	/*
	** Verify that the data field size matches the length parm.
	*/
	
	dataPos = b->fPosition;
	if (b->fLength != (msgLength + dataPos))
		err = kMessageSizeError;
	
	/*
	** Get the data block from the buffer.  Allocate one extra
	** byte so we can add a '\0' string terminator on the end,
	** in case the host didn't include one.
	*/
	if (err == kNoError)
	{
		msgData = (ui8 *)malloc( msgLength + 1);
		if (msgData == NULL)
		{
			err = kInternalError;
		}
		else
		{
			err = MessageGetBlock_ui8( b, msgData, msgLength );
			msgData[ msgLength ] = '\0';
		}
	}
	
	if (err == kNoError)
	{
		/*
		** Loop through the data field twice.  The first loop
		** counts the # of arguments and environment settings.
		** The second loop allocates argv and envp and populates
		** them.  See "linux_msgtarg.h" for the format of this
		** field.
		*/
		argv = NULL;		// array of argument ptrs
		envp = NULL;		// array of env var ptrs
		
		consoleNames[0] = NULL;
		consoleNames[1] = NULL;
		consoleNames[2] = NULL;
		

		for (j = 0; j < 2; j++)
		{	
			/*
			** On second iteration, allocate argv/envp.	
			*/
			if (j == 1)
			{
				__puts( "args: \r\n" );
				if (argc != 0)
				{
					argv = (char **)malloc( (argc + 1) * sizeof( char * ) );
					if (argv == NULL)
						err = kInternalError;
				}
				else
				{
					/*
					** The first argument is the program name, so
					** we must have at least one argument.
					*/
					err = kParameterError;
				}

				envp = (char **)malloc( (envc + 1) * sizeof( char * ) );
				if (envp == NULL)
					err = kInternalError;
			}
			
			if (err != kNoError)
				break;

			/*
			** On both iterations, scan to find the beginning
			** of each arg/env var.
			*/

			argc = 0;			// # of arguments found
			envc = 0;			// # of env vars found
			conc = 0;
			endFound = false;	// separator between env/console files found
			sepFound = false;	// separator between args/env found
			prevNull = true;	// previous byte was '\0'
			for (i = 0; i < msgLength; ++i)
			{
				if (!endFound)
				{
					if (msgData[ i ] == '\0')
					{
						if (prevNull)
						{
							if (!sepFound)
							{
								sepFound = true;
								if (j == 1)
									__puts( "envs: \r\n" );
							}
							else
							{
								endFound = true;
							}
						}
						else
						{
							prevNull = true;
						}
					}
					else
					{
						if (prevNull)
						{
							if (sepFound)
							{
								if (envp != NULL)
								{
									envp[ envc ] = &msgData[ i ];
									__puts( envp[ envc ] );
									__puts( "\r\n" );
								}
								envc++;
							}
							else
							{
								if (argv != NULL)
								{
									argv[ argc ] = &msgData[ i ];
									__puts( argv[ argc ] );
									__puts( "\r\n" );
								}
								argc++;
							}
							prevNull = false;
						}
					}
				} // if (!endFound)

				if (endFound && (msgData[ i ] == '\0'))
				{
					if (conc < kConsoleCount)
					{
						consoleNames[ conc ] = &msgData[ i + 1 ];
						conc++;
					}
					else if (wdir == NULL)
					{
						wdir = &msgData[ i + 1 ];
					}
				}
			} // inner loop
		} // outer loop
	} // if (err == kNoError)


	/*
	** Set terminating entries in both argv & envp
	*/
	argv[ argc ] = NULL;
	envp[ envc ] = NULL; 
	
	/*
	** Open the console files.
	*/

	ptyOpen = false;
	
	for (i = 0; i < kConsoleCount && err == kNoError; i++)
	{
		if (consoleNames[i] == NULL || TRK_strlen( consoleNames[i] ) == 0)
		{
		
#ifdef TRK_LINUX_USE_PTY	/* pseudo-terminals preferred over pipes */
			/*
			** If the file name is missing or empty, pipe the console
			** file through to the debugger.  Open a pseudo-terminal
			** for this if we don't already have one.  Note: This 
			** forces stdout & stdin through the same terminal.
			*/

			if (ptyOpen ||
				openpty( &tempFds[ 0 ], &tempFds[ 1 ], NULL, NULL, NULL ) == 0)
			{
				ptyOpen = true;
				trkFds[ i ] = tempFds[ 0 ];
				targetFds[ i ] = tempFds[ 1 ];
				fcntl( trkFds[ i ], F_SETFL, O_NONBLOCK );
				if (tcgetattr( trkFds[ i ], &tio ) == 0)
				{
					//tio.c_lflag &= ~ECHO;	/* enable to turn off target-side echo */
					tcsetattr( trkFds[ i ], TCSAFLUSH, &tio );
				}
			}
#else /* #ifdef TRK_LINUX_USE_PTY */
			if (false) {;}			
#endif /* #ifdef TRK_LINUX_USE_PTY */

			/*
			** If the file name is missing or empty, pipe the console
			** file through to the debugger.  Give the target program
			** the read side for stdin; otherwise the write side.
			*/
		   else if (pipe( tempFds ) == 0)
		   {
			    targetFds[i] = tempFds[ (i == kDSStdin) ? 0 : 1 ];
			    trkFds[i] = tempFds[ (i == kDSStdin) ? 1 : 0 ];
			    fcntl( trkFds[ i ], F_SETFL, O_NONBLOCK );
		   }
	
		   else
		   {
				targetFds[i] = -1;
				trkFds[i] = -1;
				err = kOsError;
				__puts( "Failed to open console pty or pipe: " );
				__puthex2( i );
				__puts( "\r\n" );
		    }
		}
		else
		{
			/*
			** If a filename was specified, open that file.  Open it as
			** read-only for stdin; otherwise write-only.
			*/
			
			targetFds[i] = open( consoleNames[i], O_CREAT |
					((i == kDSStdin) ? O_RDONLY : (O_WRONLY || O_TRUNC)),
					S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
			trkFds[i] = -1;
			if (targetFds[i] == -1)
			{
				err = kOsError;
				__puts( "Failed to open console file: " );
				__puthex2( i );
				__puts( " (" );
				__puts( consoleNames[i] );
				__puts( ")\r\n" );
			}
		}
	}

	/*
	** Launch the new process
	*/
	
	if (err == kNoError)
	{
		/*
		** Fork our new process.  On uClinux we will have to use vfork()
		** instead.  However, fiddling with the environment and file
		** descriptors before calling exec() will cause problems with
		** vfork().  Some workaround must be found.
		*/
		
#ifndef __uClinux__
		processId = fork();
#else
		processId = vfork();
#endif				

		
		if (processId == 0) // child process
		{

			#ifndef __uClinux__		
				/* give the child process its own thread group; otherwise it will be in ours (CodeWarriorTRK's) */
				setsid();
			#endif
			
			/*
			** The child inherits the TRK environment, but incorporate
			** the evironment settings from the message.
			*/
			
			for (i = 0; i < envc; i++)
				putenv( envp[ i ] ); 
		
			
			/*
			** Assign the target-side file descriptors to stdin, stdout, and
			** stderr, respectively.  Don't close anything until after
			** all of the dup2() calls because the descriptors in targetFds[]
			** may all be equal.
			*/
			
			for (i = 0; i < kConsoleCount; i++)
			{
				if (targetFds[ i ] != -1)
					dup2( targetFds[ i ], i );
			}

			for (i = 0; i < kConsoleCount; i++)
			{
				if (trkFds[ i ] != -1)
					close( trkFds[ i ] );
				if (targetFds[ i ] != -1)
					close( targetFds[ i ] );
			}
			
			/*
			** If a working directory was specified, set it.
			*/
			if (wdir != NULL && *wdir != '\0')
			{
				chdir( wdir );
			}
			
			/*
			** Set up the attachment to TRK and then exec the program.
			*/
			
			oserr = ptrace(PTRACE_TRACEME);
			
			if (oserr == -1)
			{
				__perror( "ptrace(PT_TRACE_ME) failed" );
				exit( -1 );
			}
		
			oserr = execv(argv[0], argv);
			if (oserr == -1)
			{
				__perror("execve failed, errno");
				exit( -1 );
			}
		}
		
		/*
		** Wait here until the initial stop of the target program.  If
		** we don't get it for some reason, the program didn't run correctly.
		*/
		
		if ((tempId = waitpid( processId, &status, WUNTRACED )) != processId ||
			!WIFSTOPPED( status ))
		{
			err = kOsError;
			__puts( "Initial waitpid after launch failed w/status: p" );
			__puthex8( processId );
			__puts( " wp" );
			__puthex8( tempId );
			__puts( " s" );
			__puthex8( status );
			__puts( "\r\n" );
			if (tempId == -1)
				__perror( "errno" );
		}
	}
#ifdef __uClinux__	
	ptrace(PTRACE_SETOPTIONS,processId,NULL, PTRACE_O_TRACECLONE);
#endif
	/*
	** Close the target-side console fd's.
	*/
	
	for (i = 0; i < kConsoleCount; i++)
	{
		if (targetFds[ i ] != -1)
			close( targetFds[ i ] );
	}

	/*
	** Add this to the list of debugged processes.
	*/

	if (err == kNoError)
		err = OsTargetAddProcess( processId, trkFds, processId, &pState, &tState );
		
	if (err == kNoError)
		err = TrkSetupDebugProcess( pState, tState );
		
#ifdef __uClinux__
	err = DoOsQuery(processId,&text_off,&data_off,&stack_off);
#endif	
		
	/*
	** Build the reply.
	*/

	if (err == kNoError)
	{
		MessageIntoReply( b, kDSReplyACK, kDSReplyNoError );	

		b->fFields.fData[ TRK_MSG_CPR_LINUX_PID_INDEX ] = processId;
#ifndef __uClinux__
		b->fFields.fData[ TRK_MSG_CPR_LINUX_CODEOFF_INDEX ] = 0;
		b->fFields.fData[ TRK_MSG_CPR_LINUX_DATAOFF_INDEX ] = 0;
#else
		b->fFields.fData[ TRK_MSG_CPR_LINUX_CODEOFF_INDEX ] = text_off;
		b->fFields.fData[ TRK_MSG_CPR_LINUX_DATAOFF_INDEX ] = data_off;
#endif

		err = TrkConstructMessage( b, &gTrkMsgCreateProcessReplyLinux );
	}

	/*
	** Free the argument and environment lists.
	*/

	if (msgData != NULL)
		free( msgData );
	if (argv != NULL)
		free( argv );
	if (envp != NULL)
		free( envp );

	/*
	** Send the reply.
	*/
	if (err == kNoError)
	{
		err=SendACK( b );

		/* We send the debugger a "\r\n" to cause it to open a console window */
		PostSendInitConsoleToDebuggerEvent(processId);

	 	return (err);	/* of the SendACK() */
	}

	/*
	** It is an error to reach this point.
	*/
	return( StandardACK(b, kDSReplyACK, TrkStandardReplyError( err )) );

}

/****************************************************************************/
/*
 *	DoOsDeleteProcess
 *
 *	Delete (kill) an existing process.
 *
 */
/****************************************************************************/
DSError DoOsDeleteProcess( MessageBuffer *b )
{
	DSError err = kNoError;
	ui32 processId;

	/*
	** Parse the next segment of the message.
	** This will pull out the process ID.
	*/

	err = TrkParseMessageSegment( b, &gTrkMsgDeleteProcessLinux, true );

	if (err == kNoError)
	{
		processId = b->fFields.fData[ TRK_MSG_DP_LINUX_PID_INDEX ];
		
		err = OsTargetKillProcessLinux( processId );
	}

	/*
	** Reply with the status.
	*/
	return( StandardACK(b, kDSReplyACK, TrkStandardReplyError( err )) );
}

/****************************************************************************/
/*
 *	DoOsCreateProcAttach
 *
 *	Attach to an existing process.
 *
 */
/****************************************************************************/
DSError DoOsCreateProcAttach( MessageBuffer *b )
{
	DSError err = kNoError;
#ifndef __uClinux__
	ui32 processId;
#else
	ui32 processId,text_off,data_off,stack_off;
#endif	
	int oserror;
	int consoleFds[kConsoleCount] = {-1, -1, -1};
	int notifyCount;
	int status;
	int tempId;
	LinuxProcessState *pState;
	LinuxThreadState *tState;

#if COMMON_FOR_MIPS_AND_ARM_ARCHITECTURE
	char str[10];
	size_t count;
	DSIOResult io_result;
#endif


	
	/*
	** Parse the next segment of the message.  This
	** will pull out the options and length field values.  The
	** data field is handled below.
	*/

	err = TrkParseMessageSegment( b, &gTrkMsgCreateProcAttachLinux, true );

	if (err == kNoError)
	{
		processId = b->fFields.fData[ TRK_MSG_CPA_LINUX_PID_INDEX ];
		oserror = ptrace( PTRACE_ATTACH, processId );
		if (oserror == -1)
		{
			err = kOsError;
			__perror( "attach to process failed" );
		}
		else
		{
			/*
			** Wait here until the stop of the target program.  If
			** we don't get it for some reason, the attach didn't work.
			*/

			tempId = waitpid( processId, &status, WUNTRACED );
			if (tempId == -1)
				tempId = waitpid( processId, &status, WUNTRACED | __WCLONE );
			if (tempId != processId || !WIFSTOPPED( status ))
			{
				err = kOsError;
				__puts( "Initial waitpid after attach failed w/status: p" );
				__puthex8( processId );
				__puts( " wp" );
				__puthex8( tempId );
				__puts( " s" );
				__puthex8( status );
				__puts( "\r\n" );
				if (tempId == -1)
					__perror( "errno" );
			}
		}
		
		/*
		** The debugger needs the event notification, so post it to the 
		** event queue.
		*/

		if (err == kNoError)
			PostInterruptEvent( status, processId, 0 );

		/*
		** Add it to the list of controlled processes.
		*/
		
		if (err == kNoError)
			err = OsTargetAddProcess( processId, consoleFds, processId,
					&pState, &tState );
	}

#ifdef __uClinux__
	err = DoOsQuery(processId,&text_off,&data_off,&stack_off);
#endif	

	/*
	** Build the reply.
	*/

	if (err == kNoError)
	{
		MessageIntoReply( b, kDSReplyACK, kDSReplyNoError );	

#ifndef __uClinux__
		b->fFields.fData[ TRK_MSG_CPAR_LINUX_CODEOFF_INDEX ] = 0;
		b->fFields.fData[ TRK_MSG_CPAR_LINUX_DATAOFF_INDEX ] = 0;
#else		
		b->fFields.fData[ TRK_MSG_CPAR_LINUX_CODEOFF_INDEX ] = text_off;
		b->fFields.fData[ TRK_MSG_CPAR_LINUX_DATAOFF_INDEX ] = data_off;
#endif

		err = TrkConstructMessage( b, &gTrkMsgCreateProcAttachReplyLinux );
	}

	/*
	** Send the reply.
	*/
	if (err == kNoError)
	{
		err = SendACK( b );
		
		/*
		** Set the shared library breakpoint and get the shared library state.
		** This call also gets the current thread state if necessary.
		** so there is no need to do that operation separately.  It may
		** send out notifications of any shared libraries and threads that
		** it finds.  That is why this call is made after the ACK is sent.
		*/

		TrkSetSoBreakpointsLinux( pState, tState );
		TrkUpdateSoStateLinux( pState, tState, false, true, &notifyCount );

#if COMMON_FOR_MIPS_AND_ARM_ARCHITECTURE
		/* We send the debugger a "\r\n" to cause it to open a console window */
		PostSendInitConsoleToDebuggerEvent(processId);
#endif

		return( err );
	}

	/*
	** It is an error to reach this point.
	*/
	return( StandardACK(b, kDSReplyACK, TrkStandardReplyError( err )) );
}

/****************************************************************************/
/*
 *	DoOsDeleteProcAttach
 *
 *	Delete (kill) an existing process.
 *
 */
/****************************************************************************/
DSError DoOsDeleteProcAttach( MessageBuffer *b )
{
    DSError err = kNoError;
    ui32 processId;
	int oserror;
	int status;
	int tempId;

    /*
    ** Parse the next segment of the message.  This
    ** will pull out processId field value.
    */

    err = TrkParseMessageSegment( b, &gTrkMsgDeleteProcAttachLinux, true );

    if (err == kNoError)
    {
        processId = b->fFields.fData[ TRK_MSG_DPA_LINUX_PID_INDEX ];

		/*
		** Make sure the process is stopped before proceeding with the detach.
		*/

		if (!OsTargetStopped( processId, 0 ))
		{
			err = OsTargetStop( kDSStopThread, processId, 0 );
			
			/*
			** Wait here until the stop of the target program.  If
			** we don't get it for some reason, the stop didn't work.
			*/

			if (err == kNoError)
			{
				tempId = waitpid( processId, &status, WUNTRACED );
				if (tempId == -1)
					tempId = waitpid( processId, &status, WUNTRACED | __WCLONE );
				if (tempId != processId || !WIFSTOPPED( status ))
				{
					__puts( "waitpid for stop before detach failed w/status: p" );
					__puthex8( processId );
					__puts( " wp" );
					__puthex8( tempId );
					__puts( " s" );
					__puthex8( status );
					__puts( "\r\n" );
					if (tempId == -1)
						__perror( "errno" );
				}
			}
		}

        /*
        ** Clear all breakpoints in the process, purge it from the controlled
        ** process list, and detach.  Note the PTRACE_DETACH operation will
        ** detach and continue execution of the process.
        */

		if (err == kNoError)
            err = ReleaseThreadBreakpoints( processId, DS_MSG_BREAK_THREAD_ID_ALL, true, false );
		
#ifdef CONTINUE_BEFORE_DETACH
		/*
        ** It seems that for some kernels (e.g. PPC)  if we don't continue, the process will
        ** not get detached at all even PTRACE_DETACH is sent.
        */
        oserror = ptrace( PTRACE_CONT, processId, 0, 0 );
      	if (oserror == -1)
      	{
        	__perror( "continue: " );
        	printf("processId: %i\n", processId);
      	} 
#endif		
		
        if (err == kNoError)
			err = PurgeProcess( processId );
        oserror = ptrace( PTRACE_DETACH, processId, 0, 0 );
		if (oserror == -1)
		{
			__perror( "detach from process failed" );
			}
    }
    /*
    ** It is an error to reach this point.
    */
    return( StandardACK(b, kDSReplyACK, TrkStandardReplyError( err )) );

}

/****************************************************************************/
/*
 *	DoOsDeleteThread
 *
 *	Delete (kill) an existing thread.
 *
 */
/****************************************************************************/
DSError DoOsDeleteThread( MessageBuffer *b )
{
	DSError err = kNoError;
	ui32 processId;
	ui32 threadId;

	/*
	** Parse the next segment of the message.  This
	** will pull out the process and thread ID fields values.
	*/

	err = TrkParseMessageSegment( b, &gTrkMsgDeleteThreadLinux, true );

	if (err == kNoError)
	{
		processId = b->fFields.fData[ TRK_MSG_DT_LINUX_PID_INDEX ];
		threadId = b->fFields.fData[ TRK_MSG_DT_LINUX_TID_INDEX ];

		err = OsTargetKillThreadLinux( processId, threadId );
	}
	
	/*
	** Reply with the status.
	*/
	return( StandardACK(b, kDSReplyACK, TrkStandardReplyError( err )) );
}

/****************************************************************************/
/*
 *	DoOsInfoRequest
 *
 *	Linux supports reading of the process list, process state,
 *	thread list, and thread state.
 *
 */
/****************************************************************************/
DSError DoOsInfoRequest( MessageBuffer *b )
{
	DSError err = kNoError;
	ui32 type;

	/*
	** Parse the info header segment of the message.  This
	** will read the info type from the buffer and store it
	** in the field value array.
	*/

	err = TrkParseMessageSegment( b, &gTrkMsgInfoHeader, true );
	
	/*
	** Based on request and info type, dispatch to secondary handling
	** function.
	*/
	
	if (err == kNoError)
	{
		type = b->fFields.fData[ TRK_MSG_INFO_HEADER_TYPE_INDEX ];

		switch( b->fFields.fData[ TRK_MSG_COMMAND_INDEX ] )
		{
			case kDSOSReadInfo:
				/*
				** Dispatch to another function which handles the
				** common features of read-info messages.
				*/
				return( DoOsReadInfo( b, type ) );
				break;
			case kDSOSWriteInfo:
				switch( b->fFields.fData[ TRK_MSG_INFO_HEADER_TYPE_INDEX ] )
				{
					case kDSOSSignalPolicy:
						return( DoOsWriteSignalPolicy( b ) );
					default:
						__puts( "\r\nUnsupported info read type\r\n" );
						err = kUnsupportedError;
				}
				break;
			default:
				__puts( "\r\nNon-info request in DoOsInfoRequest()\r\n" );
				err = kInternalError;
		}
	}
	
	/*
	** It is an error to reach this point.
	*/
	return( StandardACK(b, kDSReplyACK, TrkStandardReplyError( err )) );
}

/****************************************************************************/
/*
 *	DoOsReadInfo
 *
 *	Read info from the target.
 *
 */
/****************************************************************************/
DSError DoOsReadInfo( MessageBuffer *b, ui32 type )
{
	DSError err = kNoError;
	ui32 index;
	ui32 returned;
	ui32 total;
	int headerLen;
	int temp;
	unsigned int dataEnd;

	/*
	** Default values for returned & total for those info types
	** which do not return a list.
	*/
	
	returned = total = 1;

	/*
	** Parse the read-info header segment of the message.  This
	** will read the index from the buffer and store it
	** in the field value array.
	*/

	err = TrkParseMessageSegment( b, &gTrkMsgReadInfoHeader, true );

	if (err == kNoError)
		err = TrkGetMessageTotal( &gTrkMsgReadInfoReplyHeader, 
				&headerLen, &temp );

	/*
	** Dispatch to type-specific secondary function.  This will
	** build the type-specific reply info in the buffer, but will
	** leave space at the beginning for the read-info reply header.
	*/

	if (err == kNoError)
	{
		index =  b->fFields.fData[ TRK_MSG_RI_HEADER_START_INDEX ];
		
		switch( type )
		{
			case kDSOSProcessList:
				err = DoOsReadProcessList( b, headerLen, index,
						&returned, &total );
				break;
			case kDSOSProcessState:
				err = DoOsReadProcessState( b, headerLen, index,
						&returned, &total );
				break;
			case kDSOSThreadList:
				err = DoOsReadThreadList( b, headerLen, index,
						&returned, &total );
				break;
			case kDSOSThreadState:
				err = DoOsReadThreadState( b, headerLen, index,
						&returned, &total );
				break;
			default:
				__puts( "\r\nUnsupported info read type\r\n" );
				err = kUnsupportedError;
		}
	}
	
	/*
	** Build the read-info reply header.  However, this will set the
	** buffer position to the start of the type-specific data.  Need
	** to save & restore the current buffer position.
	*/

	if (err == kNoError)
	{
		dataEnd = b->fPosition;
		MessageIntoReply( b, kDSReplyACK, kDSReplyNoError );	

		b->fFields.fData[ TRK_MSG_RIR_HEADER_RETURNED_INDEX ] = returned;
		b->fFields.fData[ TRK_MSG_RIR_HEADER_TOTAL_INDEX ] = total;

		err = TrkConstructMessage( b, &gTrkMsgReadInfoReplyHeader );
	}
	
	if (err == kNoError)
		err = SetBufferPosition( b, dataEnd, false );

	/*
	** Send the reply.
	*/
	if (err == kNoError)
		return( SendACK( b ) );
	
	return( StandardACK(b, kDSReplyACK, TrkStandardReplyError( err )) );
}

/****************************************************************************/
/*
 *	DoOsReadProcessList
 *
 *	Read the list of processes.
 *
 */
/****************************************************************************/
DSError DoOsReadProcessList( MessageBuffer *b, unsigned int replyStart, 
								ui32 index,	ui32 *returned, ui32 *total )
{
	DSError err = kNoError;
	ui8 options;
	ui32 filter;

	/*
	** Parse the next segment of the message.
	*/

	err = TrkParseMessageSegment( b, &gTrkMsgReadProcessListLinux, true );

	if (err == kNoError)
	{
		options = b->fFields.fData[ TRK_MSG_RPL_LINUX_OPTIONS_INDEX ];
		filter = b->fFields.fData[ TRK_MSG_RPL_LINUX_FILTER_INDEX ];
	}

	/*
	** Set the buffer position to the provided start location
	** for the reply data.
	*/
		
	if (err == kNoError)
		err = SetBufferPosition( b, replyStart, true );

	/*
	** Build the reply information in the buffer.
	*/	

	if (err == kNoError)
		err = TargetAddProcessList( b, options, filter, index,
				returned, total );

	return( err );
}

/****************************************************************************/
/*
 *	DoOsReadProcessState
 *
 *	Read the state of a specific process.
 *
 */
/****************************************************************************/
DSError DoOsReadProcessState( MessageBuffer *b, unsigned int replyStart, 
								ui32 index,	ui32 *returned, ui32 *total )
{
	DSError err = kNoError;

	/*
	** Parse the next segment of the message
	*/

	err = TrkParseMessageSegment( b, &gTrkMsgReadProcessStateLinux, true );

	/* needs to be implemented */
	err = kUnsupportedError;
	
	/*
	** It is an error to reach this point.
	*/
	return( StandardACK(b, kDSReplyACK, TrkStandardReplyError( err )) );
}

/****************************************************************************/
/*
 *	DoOsReadThreadList
 *
 *	Read the list of threads for a specific process.
 *
 */
/****************************************************************************/
DSError DoOsReadThreadList( MessageBuffer *b, unsigned int replyStart, 
								ui32 index,	ui32 *returned, ui32 *total )
{
	DSError err = kNoError;
	ui8 options;
	ui32 filter;

	/*
	** Parse the next segment of the message.
	*/

	err = TrkParseMessageSegment( b, &gTrkMsgReadThreadListLinux, true );

	if (err == kNoError)
	{
		options = b->fFields.fData[ TRK_MSG_RTL_LINUX_OPTIONS_INDEX ];
		filter = b->fFields.fData[ TRK_MSG_RTL_LINUX_FILTER_INDEX ];
	}

	/*
	** Set the buffer position to the provided start location
	** for the reply data.
	*/
	
	if (err == kNoError)
		err = SetBufferPosition( b, replyStart, true );

	if (err == kNoError)
		err = TargetAddThreadList( b, options, filter, index,
				returned, total );

	return( err );
}

/****************************************************************************/
/*
 *	DoOsReadThreadState
 *
 *	Read the state of a specific thread.
 *
 */
/****************************************************************************/
DSError DoOsReadThreadState( MessageBuffer *b, unsigned int replyStart, 
								ui32 index,	ui32 *returned, ui32 *total )
{
	DSError err = kNoError;

	/* needs to be implemented */
	err = kUnsupportedError;
	
	/*
	** It is an error to reach this point.
	*/
	return( StandardACK(b, kDSReplyACK, TrkStandardReplyError( err )) );
}



/****************************************************************************/
/*
 *	DoOsWriteSignalPolicy
 *
 *	Set the signal policy for a process.
 *
 */
/****************************************************************************/
DSError DoOsWriteSignalPolicy( MessageBuffer *b )
{
	DSError err = kNoError;
	ui32 processId;
	LinuxProcessState *pState;

	/*
	** Parse the next segment of the message.
	*/

	err = TrkParseMessageSegment( b, &gTrkMsgWriteSignalPolicyLinux, true );

	if (err == kNoError)
	{
		processId = b->fFields.fData[ TRK_MSG_RSP_LINUX_PID_INDEX ];

		/*
		** Get the process state object for the specified process.
		*/

		if (FindLinuxProcessStateId( processId, NULL, &pState ) != kNoError)
			err = kInvalidProcessId;
	}
	
	/*
	** Read the two masks directly from the buffer.
	*/
	
	if (err == kNoError)
		err = MessageGetBlock_ui8( b, pState->fStopMask,
				sizeof( TrkSignalMaskLinux ) );
				
	if (err == kNoError)
		err = MessageGetBlock_ui8( b, pState->fPassMask,
				sizeof( TrkSignalMaskLinux ) );
	
	/*
	** Send the standard reply.
	*/
	
	return( StandardACK(b, kDSReplyACK, TrkStandardReplyError( err )) );
}

