/******************************************************************************************/
/*

DESCRIPTION

	Contains Linux-specific extensions to the monitor command constants
	and structures defined in msgcmd.h.
	
COPYRIGHT

	Copyright 2000-2002, 2006, 2009 Freescale Semiconductor, Inc.
	
*/
/******************************************************************************************/

#ifndef __LINUX_MSGTARG_H__
#define __LINUX_MSGTARG_H__

#include "msgcmd.h"

#ifdef DESCDECL
	#undef DESCDECL
#endif

#ifdef TRK_LINUX_MSGCMD_INIT
	#define DESCDECL	const
#else
	#define DESCDECL	extern const
#endif

#if __MWERKS__
#pragma mark  Notify Stopped
#endif /* __MWERKS__ */
/****************************************************************************/
/*
 *	kDSNotifyStopped (LINUX-specific)
 *
 *	Send:
 *
 *		....	<Message Header>	  ... (see gTrkMsgHeader)
 *		ui32	PC					- (4) PC of the stopped thread
 *		ui32	lastPC				- (4) previous known execution PC of the stopped thread
 *		ui32	signal				- (4) signal which stopped the thread
 *		ui32	processId			- (4) process ID
 *		ui32	threadId			- (4) thread ID
 *									  <20 + header> total
 *
 *	Reply:
 *
 *		....	<Reply Header>		  ... (see gTrkMsgReplyHeader)
 *									  <header> total
 *	
 *	Sent by the target.  This message will only be sent when the target
 *	is executing (after a kDSContinue or kDSStep command).  It indicates
 *	that the target stopped running.  For Linux, this indicates that
 *	the target thread stopped due to a signal.
 *	Expects an ACK in response.
 *
 */
/****************************************************************************/

DESCDECL TrkMsgDesc gTrkMsgNotifyStoppedLinux
#ifdef TRK_LINUX_MSGCMD_INIT
= {
	__debug_only_parm( "Notify Stopped" )
	&gTrkMsgHeader,								/* previous fragment/header */
	20,											/* # of addl bytes */
	5,											/* # of addl fields */
	{											/* addl field descriptors: */
		__trk_msg_field( "PC", 4 ),
		__trk_msg_field( "Last PC", 4 ),
		__trk_msg_field( "signal", 4 ),
		__trk_msg_field( "process ID", 4 ),
		__trk_msg_field( "thread ID", 4 ),
	}
}
#endif /* #ifdef TRK_MSGCMD_INIT */
;

/* "Index" of fields.  Take # of header fields into account. */
#define TRK_MSG_NS_LINUX_PC_INDEX		(0 + TRK_MSG_HEADER_COUNT)
#define TRK_MSG_NS_LINUX_LASTPC_INDEX	(1 + TRK_MSG_HEADER_COUNT)
#define TRK_MSG_NS_LINUX_SIGNAL_INDEX	(2 + TRK_MSG_HEADER_COUNT)
#define TRK_MSG_NS_LINUX_PID_INDEX		(3 + TRK_MSG_HEADER_COUNT)
#define TRK_MSG_NS_LINUX_TID_INDEX		(4 + TRK_MSG_HEADER_COUNT)



#if __MWERKS__
#pragma mark  Create (Launch) Process
#endif /* __MWERKS__ */
/****************************************************************************/
/*
 *	kDSOSCreateItem:kDSOSProcessItem (LINUX-specific)
 *
 *	Send:
 *
 *		....	<Item Header>		  ... (see gTrkMsgItemHeader)
 *		ui8		options				- (1) see DSLinuxCPOptions
 *		ui16	length				- (2) length (in bytes) of the data field
 *		ui8[]	data				- (0 - 2048) arguments and environment vars
 *									  <3 + length + header> total
 *
 *	Reply
 *
 *		....	<Reply Header>		  ... (see gTrkMsgReplyHeader)
 *		ui32	processId			- (4) process ID
 *		ui32	codeOffset			- (4) code section load offset
 *		ui32	dataOffset			- (4) data section load offset
 *									  <12 + header> total
 *	
 *	Sent by the host.  Launches a new process with the arguments
 *	and environment variables specified in the data field.  The data
 *	field is formatted as follows:
 *		1. A sequence of zero-terminated argument strings, followed by
 *		   a single zero separator byte.
 *		2. A sequence of zero-terminated environment variable strings,
 *		   followed by a single zero separator byte.
 *		3. Three zero-terminated filenames to be used for stdin, stdout,
 *		   and stderr, respectively.  If any one of these names is 
 *		   empty (consists only of the zero-terminator), then the I/O
 *		   for that file will be piped to the debugger via 
 *		   kDSWriteFile/kDSReadFile commands.
 *		4. One zero-terminated path name to be used as the working
 *		   directory for the new process.
 *
 *	For example, the command "ls -a -l" with environment vars "TERM=ansi"
 *	and "LINES=50" would become the byte array, with all stdin & stdout
 *	piped to the debugger, stderr send to the file "err.out", and the 
 *	working directory set to "/tmp"
 *
 *  	'l','s',0,								
 *		'-','a',0,
 *		'-','l',0,
 *		0,										# separator at end of args
 *		'T','E','R','M','=','A','N','S','I',0,
 *		'L','I','N','E','S','=','5','0',0
 *		0,										# separator at end of env. vars
 *		0,										# stdin
 *		0,										# stdout
 *		'e','r','r','.','o','u','t',0,			# stderr
 *		'/','t','m','p',0						# working directory
 *
 *	Note: If there are no environment variable settings, the separator
 *	bytes should still be included.  If the above example had no environment
 *	settings, it would be:
 *
 *  	'l','s',0,								
 *		'-','a',0,
 *		'-','l',0,
 *		0,										# separator at end of args
 *		0,										# separator at end of env. vars
 *		0,										# stdin
 *		0,										# stdout
 *		'e','r','r','.','o','u','t',0,			# stderr
 *		'/','t','m','p',0						# working directory
 *
 *
 */
/****************************************************************************/

/* variable-length data field handled separately */
DESCDECL TrkMsgDesc gTrkMsgCreateProcessLinux
#ifdef TRK_LINUX_MSGCMD_INIT
= {
	__debug_only_parm( "Create Process" )
	&gTrkMsgItemHeader,								/* previous fragment/header */
	3,											/* # of addl bytes */
	2,											/* # of addl fields */
	{											/* addl field descriptors: */
		__trk_msg_field( "options", 1 ),
		__trk_msg_field( "length", 2 )
	}
}
#endif /* #ifdef TRK_MSGCMD_INIT */
;

/* "Index" of fields.  Take # of header fields into account. */
#define TRK_MSG_CP_LINUX_OPTIONS_INDEX	(0 + TRK_MSG_ITEM_HEADER_COUNT)
#define TRK_MSG_CP_LINUX_LENGTH_INDEX	(1 + TRK_MSG_ITEM_HEADER_COUNT)


DESCDECL TrkMsgDesc gTrkMsgCreateProcessReplyLinux
#ifdef TRK_LINUX_MSGCMD_INIT
= {
	__debug_only_parm( "Create Process Reply" )
	&gTrkMsgReplyHeader,						/* previous fragment/header */
	12,											/* # of addl bytes */
	3,											/* # of addl fields */
	{											/* addl field descriptors: */
		__trk_msg_field( "process ID", 4 ),
		__trk_msg_field( "code offset", 4 ),
		__trk_msg_field( "data offset", 4 )
	}
}
#endif /* #ifdef TRK_MSGCMD_INIT */
;

/* "Index" of fields.  Take # of header fields into account. */
#define TRK_MSG_CPR_LINUX_PID_INDEX		(0 + TRK_MSG_REPLY_HEADER_COUNT)
#define TRK_MSG_CPR_LINUX_CODEOFF_INDEX	(1 + TRK_MSG_REPLY_HEADER_COUNT)
#define TRK_MSG_CPR_LINUX_DATAOFF_INDEX	(2 + TRK_MSG_REPLY_HEADER_COUNT)


/*
** No options are defined at this time.
*/

//typedef enum DSLinuxCPOptions {
//}  DSLinuxCPOptions;

#if __MWERKS__
#pragma mark  Delete (Kill) Process
#endif /* __MWERKS__ */
/****************************************************************************/
/*
 *	kDSOSDeleteItem:kDSOSProcessItem (LINUX-specific)
 *
 *	Send:
 *
 *		....	<Item Header>		  ... (see gTrkMsgItemHeader)
 *		ui32	processID			- (4) Process ID
 *									  <8 + header> total
 *
 *	Reply:
 *
 *		....	<Reply Header>		  ... (see gTrkMsgReplyHeader)
 *									  <header> total
 *	
 *	Sent by the host.  Kills (deletes) the specified process.
 *  The reply to this message indicates only that the signal was
 *	sent.  Actual process termination will be indicated by a 
 *	kDSOSNotifyDeleted:kDSOSProcessItem message.
 *
 */
/****************************************************************************/

DESCDECL TrkMsgDesc gTrkMsgDeleteProcessLinux
#ifdef TRK_LINUX_MSGCMD_INIT
= {
	__debug_only_parm( "Delete Process" )
	&gTrkMsgItemHeader,							/* previous fragment/header */
	4,											/* # of addl bytes */
	1,											/* # of addl fields */
	{											/* addl field descriptors: */
		__trk_msg_field( "process ID", 4 ),
	}
}
#endif /* #ifdef TRK_MSGCMD_INIT */
;

/* "Index" of fields.  Take # of header fields into account. */
#define TRK_MSG_DP_LINUX_PID_INDEX		(0 + TRK_MSG_ITEM_HEADER_COUNT)


#if __MWERKS__
#pragma mark  Notify Created Process 
#endif /* __MWERKS__ */
/****************************************************************************/
/*
 *	kDSOSNotifyCreated:kDSOSProcessItem (LINUX-specific)
 *
 *	Send:
 *
 *		....	<Item Header>		  ... (see gTrkMsgItemHeader)
 *		ui32	processId			- (4) process ID
 *		ui32	parentId			- (4) parent process ID
 *									  <8 + header> total
 *
 *	Reply:
 *
 *		....	<Reply Header>		  ... (see gTrkMsgReplyHeader)
 *									  <header> total
 *	
 *	Sent by the target.  Notifies the host that a new process has been
 *	created.  This is only detected when it is the result of a fork() or
 *	clone() system call, so the new process will be a copy of the 
 *	parent.  CodeWarriorTRK removes the parent's breakpoints from the child,
 *	so new breakpoints should be set as needed in the child.
 *
 */
/****************************************************************************/

DESCDECL TrkMsgDesc gTrkMsgNotifyCreatedProcessLinux
#ifdef TRK_LINUX_MSGCMD_INIT
= {
	__debug_only_parm( "Notify Created Process" )
	&gTrkMsgItemHeader,							/* previous fragment/header */
	8,											/* # of addl bytes */
	2,											/* # of addl fields */
	{											/* addl field descriptors: */
		__trk_msg_field( "process ID", 4 ),
		__trk_msg_field( "parent ID", 4 )
	}
}
#endif /* #ifdef TRK_MSGCMD_INIT */
;

/* "Index" of fields.  Take # of header fields into account. */
#define TRK_MSG_NCP_LINUX_PID_INDEX		(0 + TRK_MSG_ITEM_HEADER_COUNT)
#define TRK_MSG_NCP_LINUX_PPID_INDEX	(1 + TRK_MSG_ITEM_HEADER_COUNT)




#if __MWERKS__
#pragma mark  Notify Deleted (Terminated) Process 
#endif /* __MWERKS__ */
/****************************************************************************/
/*
 *	kDSOSNotifyDeleted:kDSOSProcessItem (LINUX-specific)
 *
 *	Send:
 *
 *		....	<Item Header>		  ... (see gTrkMsgItemHeader)
 *		ui8		type				- (1) DSLinuxNDPType
 *		ui32	code				- (4) exit status or signal
 *		ui32	processId			- (4) process ID
 *									  <9 + header> total
 *
 *	Reply:
 *
 *		....	<Reply Header>		  ... (see gTrkMsgReplyHeader)
 *									  <header> total
 *	
 *	Sent by the target.  Notifies the host that an attached process has
 *	either exited normally or been terminated with a signal.  All
 *	breakpoints associated with the process that were set by the
 *	kDSSetBreak command will be automatically purged from the target
 *	system when this message is sent.
 *
 */
/****************************************************************************/

DESCDECL TrkMsgDesc gTrkMsgNotifyDeletedProcessLinux
#ifdef TRK_LINUX_MSGCMD_INIT
= {
	__debug_only_parm( "Notify Deleted Process" )
	&gTrkMsgItemHeader,							/* previous fragment/header */
	9,											/* # of addl bytes */
	3,											/* # of addl fields */
	{											/* addl field descriptors: */
		__trk_msg_field( "NDP type", 1 ),
		__trk_msg_field( "code", 4 ),
		__trk_msg_field( "process ID", 4 )
	}
}
#endif /* #ifdef TRK_MSGCMD_INIT */
;

/* "Index" of fields.  Take # of header fields into account. */
#define TRK_MSG_NDP_LINUX_TYPE_INDEX	(0 + TRK_MSG_ITEM_HEADER_COUNT)
#define TRK_MSG_NDP_LINUX_CODE_INDEX	(1 + TRK_MSG_ITEM_HEADER_COUNT)
#define TRK_MSG_NDP_LINUX_PID_INDEX		(2 + TRK_MSG_ITEM_HEADER_COUNT)


typedef enum DSLinuxNDPType {
	kDSLinuxNDPExit			= 0x00,		/* exited normally */
	kDSLinuxNDPTerminate	= 0x01		/* terminated with a signal */
} DSLinuxDPType;
 

#if __MWERKS__
#pragma mark  Create Process Attach
#endif /* __MWERKS__ */
/****************************************************************************/
/*
 *	kDSOSCreateItem:kDSOSProcAttachItem (LINUX-specific)
 *
 *	Send:
 *
 *		....	<Item Header>		  ... (see gTrkMsgItemHeader)
 *		ui32	processId			- (4) Process ID
 *									  <4 + header> total
 *
 *	Reply:
 *
 *		....	<Reply Header>		  ... (see gTrkMsgReplyHeader)
 *		ui32	codeOffset			- (4) code section load offset
 *		ui32	dataOffset			- (4) data section load offset
 *									  <8 + header> total
 *	
 *	Sent by the host.  Attaches to the specified process for 
 *	debugging purposes.
 *
 */
/****************************************************************************/

DESCDECL TrkMsgDesc gTrkMsgCreateProcAttachLinux
#ifdef TRK_LINUX_MSGCMD_INIT
= {
	__debug_only_parm( "Create Process Attachment" )
	&gTrkMsgItemHeader,							/* previous fragment/header */
	4,											/* # of addl bytes */
	1,											/* # of addl fields */
	{											/* addl field descriptors: */
		__trk_msg_field( "process ID", 4 )
	}
}
#endif /* #ifdef TRK_MSGCMD_INIT */
;

/* "Index" of fields.  Take # of header fields into account. */
#define TRK_MSG_CPA_LINUX_PID_INDEX		(0 + TRK_MSG_ITEM_HEADER_COUNT)



DESCDECL TrkMsgDesc gTrkMsgCreateProcAttachReplyLinux
#ifdef TRK_LINUX_MSGCMD_INIT
= {
	__debug_only_parm( "Create Process Attachment Reply" )
	&gTrkMsgReplyHeader,						/* previous fragment/header */
	8,											/* # of addl bytes */
	2,											/* # of addl fields */
	{											/* addl field descriptors: */
		__trk_msg_field( "code offset", 4 ),
		__trk_msg_field( "data offset", 4 )
	}
}
#endif /* #ifdef TRK_MSGCMD_INIT */
;

/* "Index" of fields.  Take # of header fields into account. */
#define TRK_MSG_CPAR_LINUX_CODEOFF_INDEX	(0 + TRK_MSG_REPLY_HEADER_COUNT)
#define TRK_MSG_CPAR_LINUX_DATAOFF_INDEX	(1 + TRK_MSG_REPLY_HEADER_COUNT)


#if __MWERKS__
#pragma mark  Delete Process Attach
#endif /* __MWERKS__ */
/****************************************************************************/
/*
 *	kDSOSDeleteItem:kDSOSProcAttachItem (LINUX-specific)
 *
 *	Send:
 *
 *		....	<Item Header>		  ... (see gTrkMsgItemHeader)
 *		ui32	processID			- (4) Process ID
 *									  <4 + header> total
 *
 *	Reply:
 *
 *		....	<Reply Header>		  ... (see gTrkMsgReplyHeader)
 *									  <header> total
 *	
 *	Sent by the host.  Detaches from the specified process.
 *
 */
/****************************************************************************/

DESCDECL TrkMsgDesc gTrkMsgDeleteProcAttachLinux
#ifdef TRK_LINUX_MSGCMD_INIT
= {
	__debug_only_parm( "Delete Process Attachment" )
	&gTrkMsgItemHeader,							/* previous fragment/header */
	4,											/* # of addl bytes */
	1,											/* # of addl fields */
	{											/* addl field descriptors: */
		__trk_msg_field( "process ID", 4 )
	}
}
#endif /* #ifdef TRK_MSGCMD_INIT */
;

/* "Index" of fields.  Take # of header fields into account. */
#define TRK_MSG_DPA_LINUX_PID_INDEX		(0 + TRK_MSG_ITEM_HEADER_COUNT)


#if __MWERKS__
#pragma mark  Delete (Kill) Thread
#endif /* __MWERKS__ */
/****************************************************************************/
/*
 *	kDSOSDeleteItem:kDSOSThreadItem (LINUX-specific)
 *
 *	Send:
 *
 *		....	<Item Header>		  ... (see gTrkMsgItemHeader)
 *		ui32	processId			- (4) Process ID
 *		ui32	threadId			- (4) Thread ID
 *									  <8 + header> total
 *
 *	Reply:
 *
 *		....	<Reply Header>		  ... (see gTrkMsgReplyHeader)
 *									  <header> total
 *	
 *	Sent by the host.  Deletes (kills) the specified thread.
 *
 */
/****************************************************************************/

DESCDECL TrkMsgDesc gTrkMsgDeleteThreadLinux
#ifdef TRK_LINUX_MSGCMD_INIT
= {
	__debug_only_parm( "Delete Thread" )
	&gTrkMsgItemHeader,							/* previous fragment/header */
	8,											/* # of addl bytes */
	2,											/* # of addl fields */
	{											/* addl field descriptors: */
		__trk_msg_field( "process ID", 4 ),
		__trk_msg_field( "thread ID", 4 )
	}
}
#endif /* #ifdef TRK_MSGCMD_INIT */
;

/* "Index" of fields.  Take # of header fields into account. */
#define TRK_MSG_DT_LINUX_PID_INDEX		(0 + TRK_MSG_ITEM_HEADER_COUNT)
#define TRK_MSG_DT_LINUX_TID_INDEX		(1 + TRK_MSG_ITEM_HEADER_COUNT)


#if __MWERKS__
#pragma mark  Notify Created Thread
#endif /* __MWERKS__ */
/****************************************************************************/
/*
 *	kDSOSNotifyCreated:kDSOSThreadItem (LINUX-specific)
 *
 *	Send:
 *
 *		....	<Item Header>		  ... (see gTrkMsgItemHeader)
 *		ui32	processId			- (4) Process ID
 *		ui32	threadId			- (4) Thread ID
 *									  <8 + header> total
 *
 *	Reply:
 *
 *		....	<Reply Header>		  ... (see gTrkMsgReplyHeader)
 *									  <header> total
 *	
 *	Sent by the target.  Notifies the host that a new thread has
 *	been created in an attached process.  The new thread will be
 *	in a stopped state until the host sends a kDSContinue or
 *	kDSStep command.
 *
 */
/****************************************************************************/

DESCDECL TrkMsgDesc gTrkMsgNotifyCreatedThreadLinux
#ifdef TRK_LINUX_MSGCMD_INIT
= {
	__debug_only_parm( "Notify Created Thread" )
	&gTrkMsgItemHeader,							/* previous fragment/header */
	8,											/* # of addl bytes */
	2,											/* # of addl fields */
	{											/* addl field descriptors: */
		__trk_msg_field( "process ID", 4 ),
		__trk_msg_field( "thread ID", 4 )
	}
}
#endif /* #ifdef TRK_MSGCMD_INIT */
;

/* "Index" of fields.  Take # of header fields into account. */
#define TRK_MSG_NCT_LINUX_PID_INDEX		(0 + TRK_MSG_ITEM_HEADER_COUNT)
#define TRK_MSG_NCT_LINUX_TID_INDEX		(1 + TRK_MSG_ITEM_HEADER_COUNT)


#if __MWERKS__
#pragma mark  Notify Deleted Thread
#endif /* __MWERKS__ */
/****************************************************************************/
/*
 *	kDSOSNotifyDeleted:kDSOSThreadItem (LINUX-specific)
 *
 *	Send:
 *
 *		....	<Item Header>		  ... (see gTrkMsgItemHeader)
 *		ui32	result				- (4) result value from thread
 *		ui32	processId			- (4) Process ID
 *		ui32	threadId			- (4) Thread ID
 *									  <12 + header> total
 *
 *	Reply:
 *
 *		....	<Reply Header>		  ... (see gTrkMsgReplyHeader)
 *									  <header> total
 *	
 *	Sent by the target.  Notifies the host that a thread in an attached
 *	process has terminated.  All breakpoints associated with the
 *	process that were set by the kDSSetBreak command will be
 *	automatically purged from the target system when this message is
 *	sent.
 *
 */
/****************************************************************************/

DESCDECL TrkMsgDesc gTrkMsgNotifyDeletedThreadLinux
#ifdef TRK_LINUX_MSGCMD_INIT
= {
	__debug_only_parm( "Notify Deleted Thread" )
	&gTrkMsgItemHeader,							/* previous fragment/header */
	12,											/* # of addl bytes */
	3,											/* # of addl fields */
	{											/* addl field descriptors: */
		__trk_msg_field( "result", 4 ),
		__trk_msg_field( "process ID", 4 ),
		__trk_msg_field( "thread ID", 4 )
	}
}
#endif /* #ifdef TRK_MSGCMD_INIT */
;

/* "Index" of fields.  Take # of header fields into account. */
#define TRK_MSG_NDT_LINUX_RESULT_INDEX	(0 + TRK_MSG_ITEM_HEADER_COUNT)
#define TRK_MSG_NDT_LINUX_PID_INDEX		(1 + TRK_MSG_ITEM_HEADER_COUNT)
#define TRK_MSG_NDT_LINUX_TID_INDEX		(2 + TRK_MSG_ITEM_HEADER_COUNT)


#if __MWERKS__
#pragma mark  Notify Created (Loaded) DLL
#endif /* __MWERKS__ */
/****************************************************************************/
/*
 *	kDSOSNotifyCreated:kDSOSDLLItem (LINUX-specific)
 *
 *	Send:
 *
 *		....	<Item Header>		  ... (see gTrkMsgItemHeader)
 *		ui8		remaining			- (1) boolean - more notifications following?
 *		ui32	processId			- (4) ID of process into which lib is loaded
 *		ui32	threadId			- (4) ID of thread doing the load
 *		ui32	offset				- (4) load offset for the section addresses
 *		ui16	length				- (2) length of the name field
 *		ui8[]	name				- (0 - 2048) name of the shared library
 *									  <15 + length + header> total
 *
 *	Reply:
 *
 *		....	<Reply Header>		  ... (see gTrkMsgReplyHeader)
 *									  <header> total
 *	
 *	Sent by the target.  Notifies the host that a new shared library has
 *	been loaded by an attached process.  The threadId indicates which
 *	thread is responsible for loading the shared library, but once it
 *	is loaded the shared library is accessible to all threads in the
 *	process.  The associated thread will be in a stopped state until the
 *	host sends a kDSContinue or kDSStep command.  However, all other
 *	threads in the process will continue running.  It is possible for
 *	a thread to load several shared libraries simultaneously.  This
 *	means it is possible for several sequential notification messages
 *	to be sent, all associated with the same thread.  The remaining
 *	field indicates whether there are remaining unsent notifications for
 *	this thread.  The host should not continue running the thread
 *	until it has received and processed all of the notifications in
 *	the sequence.
 *
 */
/****************************************************************************/

/* variable-length name field handled separately */
DESCDECL TrkMsgDesc gTrkMsgNotifyCreatedDllLinux
#ifdef TRK_LINUX_MSGCMD_INIT
= {
	__debug_only_parm( "Notify Created DLL" )
	&gTrkMsgItemHeader,							/* previous fragment/header */
	15,											/* # of addl bytes */
	5,											/* # of addl fields */
	{											/* addl field descriptors: */
		__trk_msg_field( "remaining", 1 ),
		__trk_msg_field( "process ID", 4 ),
		__trk_msg_field( "thread ID", 4 ),
		__trk_msg_field( "offset", 4 ),
		__trk_msg_field( "length", 2 )
	}
}
#endif /* #ifdef TRK_MSGCMD_INIT */
;

/* "Index" of fields.  Take # of header fields into account. */
#define TRK_MSG_NCD_LINUX_REMAINING_INDEX	(0 + TRK_MSG_ITEM_HEADER_COUNT)
#define TRK_MSG_NCD_LINUX_PID_INDEX			(1 + TRK_MSG_ITEM_HEADER_COUNT)
#define TRK_MSG_NCD_LINUX_TID_INDEX			(2 + TRK_MSG_ITEM_HEADER_COUNT)
#define TRK_MSG_NCD_LINUX_OFFSET_INDEX		(3 + TRK_MSG_ITEM_HEADER_COUNT)
#define TRK_MSG_NCD_LINUX_LENGTH_INDEX		(4 + TRK_MSG_ITEM_HEADER_COUNT)


#if __MWERKS__
#pragma mark  Notify Deleted (Unloaded) DLL
#endif /* __MWERKS__ */
/****************************************************************************/
/*
 *	kDSOSNotifyDeleted:kDSOSDLLItem (LINUX-specific)
 *
 *	Send:
 *
 *		....	<Item Header>		  ... (see gTrkMsgItemHeader)
 *		ui8		remaining			- (1) boolean - more notifications following?
 *		ui32	processId			- (4) ID of process into which lib is unloaded
 *		ui32	threadId			- (4) ID of thread doing the unload
 *		ui16	length				- (2) length of the name field
 *		ui8[]	name				- (0 - 2048) name of the shared library
 *									  <11 + length + header> total
 *
 *	Reply:
 *
 *		....	<Reply Header>		  ... (see gTrkMsgReplyHeader)
 *									  <header> total
 *	
 *	Sent by target.  Notifies the host that a shared library has been
 *	unloaded by an attached process.  The threadId indicates which
 *	thread is responsible for unloading the shared library, but once it
 *	is unloaded the shared library is inaccessible to all threads in the
 *	process.  The associated thread will be in a stopped state until the
 *	host sends a kDSContinue or kDSStep command.  However, all other
 *	threads in the process will continue running.  It is possible for
 *	a thread to unload several shared libraries simultaneously.  This
 *	means it is possible for several sequential notification messages
 *	to be sent, all associated with the same thread.  The remaining
 *	field indicates whether there are remaining unsent notifications for
 *	this thread.  The host should not continue running the thread
 *	until it has received and processed all of the notifications in
 *	the sequence.
 *
 */
/****************************************************************************/

/* variable-length name field handled separately */
DESCDECL TrkMsgDesc gTrkMsgNotifyDeletedDllLinux
#ifdef TRK_LINUX_MSGCMD_INIT
= {
	__debug_only_parm( "Notify Deleted DLL" )
	&gTrkMsgItemHeader,							/* previous fragment/header */
	11,											/* # of addl bytes */
	4,											/* # of addl fields */
	{											/* addl field descriptors: */
		__trk_msg_field( "remaining", 1 ),
		__trk_msg_field( "process ID", 4 ),
		__trk_msg_field( "process ID", 4 ),
		__trk_msg_field( "length", 2 )
	}
}
#endif /* #ifdef TRK_MSGCMD_INIT */
;

/* "Index" of fields.  Take # of header fields into account. */
#define TRK_MSG_NDD_LINUX_REMAINING_INDEX	(0 + TRK_MSG_ITEM_HEADER_COUNT)
#define TRK_MSG_NDD_LINUX_PID_INDEX			(1 + TRK_MSG_ITEM_HEADER_COUNT)
#define TRK_MSG_NDD_LINUX_TID_INDEX			(2 + TRK_MSG_ITEM_HEADER_COUNT)
#define TRK_MSG_NDD_LINUX_LENGTH_INDEX		(3 + TRK_MSG_ITEM_HEADER_COUNT)


#if __MWERKS__
#pragma mark  Notify Created Process Exec
#endif /* __MWERKS__ */
/****************************************************************************/
/*
 *	kDSOSNotifyCreated:kDSOSProcExecItem (LINUX-specific)
 *
 *	Send:
 *
 *		....	<Item Header>		  ... (see gTrkMsgItemHeader)
 *		ui32	processId			- (4) process ID
 *		ui8[]	exePath				- (0-2048) zero-terminated target path to new executable file
 *									  <4 + exePath + header> total
 *
 *	Reply:
 *
 *		....	<Reply Header>		  ... (see gTrkMsgReplyHeader)
 *									  <header> total
 *	
 *	Sent by the target.  Notifies the host that an exec() has occurred in
 *	a process.  The process will be stopped at the beginning of the new
 *	executable.
 *
 */
/****************************************************************************/

DESCDECL TrkMsgDesc gTrkMsgNotifyCreatedProcExecLinux
#ifdef TRK_LINUX_MSGCMD_INIT
= {
	__debug_only_parm( "Notify Created Process Exec" )
	&gTrkMsgItemHeader,							/* previous fragment/header */
	4,											/* # of addl bytes */
	2,											/* # of addl fields */
	{											/* addl field descriptors: */
		__trk_msg_field( "process ID", 4 ),
		__trk_msg_field( "exe path", -1 )
	}
}
#endif /* #ifdef TRK_MSGCMD_INIT */
;

/* "Index" of fields.  Take # of header fields into account. */
#define TRK_MSG_NCPE_LINUX_PID_INDEX		(0 + TRK_MSG_ITEM_HEADER_COUNT)
#define TRK_MSG_NCPE_LINUX_EXEPATH_INDEX	(1 + TRK_MSG_ITEM_HEADER_COUNT)



#if __MWERKS__
#pragma mark  Read Process List
#endif /* __MWERKS__ */
/****************************************************************************/
/*
 *	kDSOSReadInfo:kDSOSProcessList (LINUX-specific)
 *
 *	Send:
 *
 *		....	<Info Header>	      ... (see gTrkMsgInfoHeader)
 *		ui8		options				- (1) reserved (set to 0)
 *		ui32	filter				- (4) reserved (set to 0)
 *									  <5 + header> total
 *
 *	Reply:
 *
 *		....	<Read Info Reply Header> ... (see gTrkMsgReadInfoReplyHeader)
 *		<*>		data				- (0-2048 bytes) list of process info blocks
 *									  <data + header> total
 *	
 *	<*> The data consists of a series of process info blocks, where each block
 *	has the following format:
 *
 *		ui32	id					- (4) process ID
 *		ui8		state				- (1) process state (see DSProcessStateLinux)
 *		ui8		multithreaded		- (1) 1 if known multithreaded; 0 otherwise
 *		ui8[]	name				- (0 - 2040) zero-terminated process name
 *									  <6 + size of name> total
 *
 *
 *	Sent by the host.  Reads the process list from the target.  The complete
 *	process list may not fit in a single CodeWarriorTRK message, but it is
 *	possible to read the list in segments.  When reading the list, the
 *	startIndex should be 0 the first time.  If, in the reply,
 *	totalCount > (startIndex + returnedCount), then the next segment of the
 *	list can be read by sending another request with the new startIndex
 *	set to the previous (startIndex + returnedCount).  
 *
 *	startIndex, returnedCount, and totalCount all refer to the number of
 *	process info blocks, not the number of bytes.  All three fields are
 *	described in the general kDSOSReadInfo header (see msgcmd.h).
 *
 *	The options field may be used in the future to request more detailed
 *	information.  Combined with the filter field it might be used to
 *	filter on things like user ID, session ID, etc.
 *
 */
/****************************************************************************/

#define TRK_MSG_PL_INFO_MIN_LENGTH_LINUX	7	// 6 + at least the zero-terminator

/*
** The process state field can be one of these values.
*/

typedef enum DSProcessStateLinux {
	kDSProcessRunningLinux =	'R',		/* running */
	kDSProcessISleepingLinux =	'S',		/* sleeping in interruptible state */
	kDSProcessUSleepingLinux =	'D',		/* sleeping in uninterruptible state */
	kDSProcessZombieLinux =		'Z',		/* zombie */
	kDSProcessStoppedLinux =	'T',		/* stopped */
} DSProcessStateLinux;

DESCDECL TrkMsgDesc gTrkMsgReadProcessListLinux
#ifdef TRK_LINUX_MSGCMD_INIT
= {
	__debug_only_parm( "Read Process List" )
	&gTrkMsgReadInfoHeader,						/* previous fragment/header */
	5,											/* # of addl bytes */
	2,											/* # of addl fields */
	{											/* addl field descriptors: */
		__trk_msg_field( "options", 1 ),
		__trk_msg_field( "filter", 4 ),
	}
}
#endif /* #ifdef TRK_MSGCMD_INIT */
;

/* "Index" of fields.  Take # of header fields into account. */
#define TRK_MSG_RPL_LINUX_OPTIONS_INDEX	(0 + TRK_MSG_RI_HEADER_COUNT)
#define TRK_MSG_RPL_LINUX_FILTER_INDEX	(1 + TRK_MSG_RI_HEADER_COUNT)


#if __MWERKS__
#pragma mark  Read Process State
#endif /* __MWERKS__ */
/****************************************************************************/
/*
 *	kDSOSReadInfo:kDSOSProcessState
 *
 *	Send:
 *
 *		....	<Info Header>	      ... (see gTrkMsgInfoHeader)
 *		ui8		options				- (1) reserved (set to 0)
 *		ui32	id					- (4) process ID
 *									  <5 + header> total
 *
 *	Reply:
 *
 *		....	<Reply Header>		  ... (see gTrkMsgReplyHeader)
 *		ui16	length				- (2) length of the name field
 *		ui8		state				- (1) process state (see DSProcessStateLinux)
 *		ui8		multithreaded		- (1) 1 if known multithreaded; 0 otherwise
 *		ui8[]	name				- (0 - 2048) process name
 *									  <4 + length + header> total
 *
 *
 *	Sent by the host.  Reads the status for the specified process.
 *
 *	The options field may be used in the future to request more detailed
 *	information.
 *
 *	Currently unsupported.
 */
/****************************************************************************/

DESCDECL TrkMsgDesc gTrkMsgReadProcessStateLinux
#ifdef TRK_LINUX_MSGCMD_INIT
= {
	__debug_only_parm( "Read Process State" )
	&gTrkMsgInfoHeader,							/* previous fragment/header */
	5,											/* # of addl bytes */
	2,											/* # of addl fields */
	{											/* addl field descriptors: */
		__trk_msg_field( "options", 1 ),
		__trk_msg_field( "process ID", 4 )
	}
}
#endif /* #ifdef TRK_MSGCMD_INIT */
;

/* "Index" of fields.  Take # of header fields into account. */
#define TRK_MSG_RPS_LINUX_OPTIONS_INDEX	(0 + TRK_MSG_RI_HEADER_COUNT)
#define TRK_MSG_RPS_LINUX_PID_INDEX		(1 + TRK_MSG_RI_HEADER_COUNT)


/* variable-length name field handled separately */
DESCDECL TrkMsgDesc gTrkMsgReadProcessStateReplyLinux
#ifdef TRK_LINUX_MSGCMD_INIT
= {
	__debug_only_parm( "Read Process State Reply" )
	&gTrkMsgReplyHeader,						/* previous fragment/header */
	4,											/* # of addl bytes */
	3,											/* # of addl fields */
	{											/* addl field descriptors: */
		__trk_msg_field( "length", 2 ),
		__trk_msg_field( "state", 1 ),
		__trk_msg_field( "multithreaded", 1 )
	}
}
#endif /* #ifdef TRK_MSGCMD_INIT */
;

/* "Index" of fields.  Take # of header fields into account. */
#define TRK_MSG_RPSR_LINUX_LENGTH_INDEX		(0 + TRK_MSG_RI_HEADER_COUNT)
#define TRK_MSG_RPSR_LINUX_STATE_INDEX		(1 + TRK_MSG_RI_HEADER_COUNT)
#define TRK_MSG_RPSR_LINUX_MTHREAD_INDEX	(2 + TRK_MSG_RI_HEADER_COUNT)

#if __MWERKS__
#pragma mark  Read Thread List
#endif /* __MWERKS__ */
/****************************************************************************/
/*
 *	kDSOSReadInfo:kDSOSThreadList (LINUX-specific)
 *
 *	Send:
 *
 *		....	<Info Header>	      ... (see gTrkMsgInfoHeader)
 *		ui8		options				- (1) reserved (set to 0)
 *		ui32	processId			- (4) process ID
 *									  <5 + header> total
 *
 *	Reply:
 *
 *		....	<Read Info Reply Header> ... (see gTrkMsgReadInfoReplyHeader)
 *		<*>		data				- (0-2048 bytes) list of thread info blocks
 *									  <data + header> total
 *	
 *	<*> The data consists of an array of thread info blocks, where each block
 *	has the following format:
 *
 *		ui32	id					- (4) thread ID (used by debugger)
 *		ui32	libId				- (4) thread ID assigned by thread lib
 *		ui32	lwpId				- (4) light-weight process ID
 *		ui8		state				- (1) thread state (see DSThreadStateLinux)
 *									  <13> total
 *
 *
 *	Sent by the host.  Reads the thread list for a specific owning process
 *	from the target.  The complete thread list may not fit in a single
 *	CodeWarriorTRK message, but it is possible to read the list in segments.  When
 *	reading the list, the startIndex should be 0 the first time.  If, in the reply,
 *	totalCount > (startIndex + returnedCount), then the next segment of the
 *	list can be read by sending another request with the new startIndex
 *	set to the previous (startIndex + returnedCount).  
 *
 *	startIndex, returnedCount, and totalCount all refer to the number of
 *	thread info blocks, not the number of bytes.  All three fields are
 *	described in the general kDSOSReadInfo header (see msgcmd.h).
 *
 *	The options field may be used in the future to request more detailed
 *	information.  Combined with the filter field it might be used to
 *	filter on things like user ID, session ID, etc.
 *
 */
/****************************************************************************/

#define TRK_MSG_TL_INFO_MIN_LENGTH_LINUX	13	// thread info block size

/*
** The thread state field can be one of these values.
*/

typedef enum DSThreadStateLinux
{
	kDSThreadUnknownLinux,			/* TD_THR_UNKNOWN */
	kDSThreadStoppedLinux,			/* TD_THR_STOPPED */
	kDSThreadRunLinux,				/* TD_THR_RUN */
	kDSThreadActiveLinux,			/* TD_THR_ACTIVE */
	kDSThreadZombieLinux,			/* TD_THR_ZOMBIE */
	kDSThreadSleepLinux,			/* TD_THR_SLEEP */
	kDSThreadStoppedAsleepLinux		/* TD_THR_STOPPED_ASLEEP */
} DSThreadStateLinux;


DESCDECL TrkMsgDesc gTrkMsgReadThreadListLinux
#ifdef TRK_LINUX_MSGCMD_INIT
= {
	__debug_only_parm( "Read Thread List" )
	&gTrkMsgReadInfoHeader,						/* previous fragment/header */
	5,											/* # of addl bytes */
	2,											/* # of addl fields */
	{											/* addl field descriptors: */
		__trk_msg_field( "options", 1 ),
		__trk_msg_field( "process ID", 4 )
	}
}
#endif /* #ifdef TRK_MSGCMD_INIT */
;

/* "Index" of fields.  Take # of header fields into account. */
#define TRK_MSG_RTL_LINUX_OPTIONS_INDEX	(0 + TRK_MSG_RI_HEADER_COUNT)
#define TRK_MSG_RTL_LINUX_FILTER_INDEX	(1 + TRK_MSG_RI_HEADER_COUNT)

#if __MWERKS__
#pragma mark  Read Thread State
#endif /* __MWERKS__ */
/****************************************************************************/
/*
 *	kDSOSReadInfo:kDSOSThreadState
 *
 *	Send:
 *
 *		....	<Info Header>	      ... (see gTrkMsgInfoHeader)
 *		ui8		options				- (1) reserved (set to 0)
 *		ui32	processId			- (4) process ID
 *		ui32	threadId			- (4) thread ID
 *									  <8 + header> total
 *
 *	Reply:
 *
 *		....	<Reply Header>		  ... (see gTrkMsgReplyHeader)
 *		ui16	length				- (2) length of the name field
 *		ui32	libId				- (4) thread ID assigned by thread lib
 *		ui32	lwpid				- (4) light-weight process ID
 *		ui8		state				- (1) thread state (see DSThreadStateLinux)
 *		ui8[]	name				- (0 - 2040) zero-terminated process name
 *									  <11 + header + length> total
 *
 *
 *	Sent by the host.  Reads the status for the specified task.
 *
 *	The options field may be used in the future to request more detailed
 *	information.
 *
 *	Currently unsupported.
 */
/****************************************************************************/
 
#if __MWERKS__
#pragma mark  Write Signal Policy
#endif /* __MWERKS__ */
/****************************************************************************/
/*
 *	kDSOSWriteInfo:kDSOSSignalPolicy (LINUX-specific)
 *
 *	Send:
 *
 *		....	<Info Header>	      ... (see gTrkMsgInfoHeader)
 *		ui32	processId			- (4) ID of owning process
 *		ui8		stopMask[8]			- (8) stop on these signals (TrkSignalMaskLinux)
 *		ui8		passMask[8]			- (8) pass these signals on (TrkSignalMaskLinux)
 *									  <24 + header> total
 *
 *	Reply:
 *
 *		....	<Reply Header>		  ... (see gTrkMsgReplyHeader)
 *									  <header> total
 *
 *
 *	Sent by the host.  Sets the signal-handling policy for the specified
 *	process.  The two fields stopMask and passMask contain 1 bit for
 *	each signal (64 total), where the low-order bit of mask[0] represents
 *	signal 1 and the high-order bit of mask[7] represents signal 64.
 *
 *	Setting the bit for a signal to 1 in stopMask means that any time
 *	a thread for the process stops with that signal it will be reported
 *	to the debugger (via kDSNotifyStopped) and the thread will remain stopped
 *	until the next kDSStep or kDSContinue command is issued.
 *
 *	Setting the bit for a signal to 1 in passMask means that any time
 *	a thread for the process stops with that signal it will be passed on
 *	to the thread the next time that thread continues executing (so the
 *	thread can handle it).
 *	
 */
/****************************************************************************/

typedef ui8	TrkSignalMaskLinux[8];				/* 64 bits total */


/*
** Macros for accessing mask bits.  Warning: macro parameters are
** referenced more than once.
*/

#define TRK_SIGNAL_MAX_LINUX		63

#define TRK_MASK_IS_SET( m, s )		((m)[(((s)-1)/8)%8] & (ui8)(1 << (((s)-1)%8)))
#define TRK_MASK_SET( m, s )		((m)[(((s)-1)/8)%8] |= (ui8)(1 << (((s)-1)%8)))
#define TRK_MASK_CLEAR( m, s )		((m)[(((s)-1)/8)%8] &= (ui8)(~(1 << (((s)-1)%8))))


/* mask fields handled separately */
DESCDECL TrkMsgDesc gTrkMsgWriteSignalPolicyLinux
#ifdef TRK_LINUX_MSGCMD_INIT
= {
	__debug_only_parm( "Write Signal Policy" )
	&gTrkMsgInfoHeader,							/* previous fragment/header */
	4,											/* # of addl bytes */
	1,											/* # of addl fields */
	{											/* addl field descriptors: */
		__trk_msg_field( "process ID", 4 )
	}
}
#endif /* #ifdef TRK_MSGCMD_INIT */
;

/* "Index" of fields.  Take # of header fields into account. */
#define TRK_MSG_RSP_LINUX_PID_INDEX		(0 + TRK_MSG_INFO_HEADER_COUNT)


/****************************************************************************/
/*
** Define extended, target-specific DSOSItemTypes values here.
**
** The pre-defined values are:
**
**   kDSOSProcessItem	= 0x0000
**   kDSOSThreadItem	= 0x0001
**   kDSOSDLLItem		= 0x0002
**   kDSOSAppItem		= 0x0003
**   kDSOSMemBlockItem	= 0x0004
**   kDSOSProcAttachItem= 0x0005
**   kDSOSThreadAttachItem= 0x0006
**   0x0007 - 0x00ff reserved for expansion
**
** Extended item type values should not conflict with the pre-defined
** or reserved values.
*/
/****************************************************************************/

typedef enum {
	kDSOsProcExecItem	= 0x100
} DSTargOSLinuxItemTypes;

/****************************************************************************/
/*
** Define extended, target-specific DSOSInfoTypes values here.
**
** The pre-defined values are:
**
**   kDSOSProcessList	= 0x0000
**   kDSOSProcessState	= 0x0001
**   kDSOSThreadList	= 0x0002
**   kDSOSThreadState	= 0x0003
**   kDSOSDLLList		= 0x0004
**   kDSOSDLLState		= 0x0005
**   0x0006 - 0x00ff reserved for expansion
**
** Extended info type values should not conflict with the pre-defined
** or reserved values.
*/
/****************************************************************************/

typedef enum {
	kDSOSSignalPolicy	= 0x0100
} DSTargOSLinuxInfoTypes;




#endif /* __LINUX_MSGTARG_H__ */

