/******************************************************************************************/
/*

DESCRIPTION

	Event notification from TRK to host debugger, Linux-specific.
	
	Send messages (notification) from TRK to the host debugger.
	
COPYRIGHT

	Copyright 2000, 2002, 2003, 2006, 2009 Freescale Semiconductor, Inc.
	
*/
/******************************************************************************************/

#include "trk_prefix.h"

#include <string.h>

#include "usr_put.h"		// must be included before msgcmd.h
#include "nubassrt.h"
#include "notify.h"
#include "msgbuf.h"
#include "msg.h"
#include "msgcmd.h"
#include "msgparse.h"
#include "targimpl.h"
#include "support.h"
#include "linux_msgtarg.h"
#include "string_TRK.h"
#define NOTIFY_TRY_COUNT 3

/******************************************************************************/
/*
 *	DoOsNotifyProcessCreated - used to send process fork()
 */
/******************************************************************************/

DSError DoOsNotifyProcessCreated( ui32 processId, ui32 parentId )
{
	DSError			err;
	
	MessageBufferId	bufferID;
	MessageBuffer*	buffer;
		
	/*
	 *	Get a send buffer, create a message
	 */
	
	err = GetFreeBuffer(&bufferID, &buffer);
	
	if (err == kNoError) {
		
		/*
		** Construct the message.
		*/
		
		buffer->fFields.fData[ TRK_MSG_COMMAND_INDEX ] = kDSOSNotifyCreated;
		buffer->fFields.fData[ TRK_MSG_SID_INDEX ] = GetNextSendSeqId();
		buffer->fFields.fData[ TRK_MSG_ITEM_HEADER_TYPE_INDEX ] = kDSOSProcessItem;
		buffer->fFields.fData[ TRK_MSG_NCP_LINUX_PID_INDEX ] = processId;
		buffer->fFields.fData[ TRK_MSG_NCP_LINUX_PPID_INDEX ] = parentId;

		err = TrkConstructMessage( buffer, &gTrkMsgNotifyCreatedProcessLinux );

		FAIL(err == kNoError);			/* shouldn't be possible */

		err = DoNotify( buffer );
		ReleaseBuffer(bufferID);		
	}
	
	return( err );
}

/******************************************************************************/
/*
 *	DoOsNotifyProcessDeleted - used to send process exit/termination
 */
/******************************************************************************/

DSError DoOsNotifyProcessDeleted( ui8 type, ui32 code, ui32 processId )
{
	DSError			err;
	
	MessageBufferId	bufferID;
	MessageBuffer*	buffer;
		
	/*
	 *	Get a send buffer, create a message
	 */
	
	err = GetFreeBuffer(&bufferID, &buffer);
	
	if (err == kNoError) {
		
		/*
		** Construct the message.
		*/
		
		buffer->fFields.fData[ TRK_MSG_COMMAND_INDEX ] = kDSOSNotifyDeleted;
		buffer->fFields.fData[ TRK_MSG_SID_INDEX ] = GetNextSendSeqId();
		buffer->fFields.fData[ TRK_MSG_ITEM_HEADER_TYPE_INDEX ] = kDSOSProcessItem;
		buffer->fFields.fData[ TRK_MSG_NDP_LINUX_TYPE_INDEX ] = type;
		buffer->fFields.fData[ TRK_MSG_NDP_LINUX_CODE_INDEX ] = code;
		buffer->fFields.fData[ TRK_MSG_NDP_LINUX_PID_INDEX ] = processId;

		err = TrkConstructMessage( buffer, &gTrkMsgNotifyDeletedProcessLinux );

		FAIL(err == kNoError);			/* shouldn't be possible */

		err = DoNotify( buffer );
		ReleaseBuffer(bufferID);		
	}
	
	return( err );
}

/******************************************************************************/
/*
 *	DoOsNotifyDllCreated - used to send shared object load notification
 */
/******************************************************************************/

DSError DoOsNotifyDllCreated( ui32 offset, char *name, bool remaining,
							ui32 processId, ui32 threadId )
{
	DSError			err;
	
	MessageBufferId	bufferID;
	MessageBuffer*	buffer;
		
	/*
	 *	Get a send buffer, create a message
	 */
	
	err = GetFreeBuffer(&bufferID, &buffer);
	
	if (err == kNoError) {
		
		/*
		** Construct the message.  The name field is added separately.
		*/
		
		buffer->fFields.fData[ TRK_MSG_COMMAND_INDEX ] = kDSOSNotifyCreated;
		buffer->fFields.fData[ TRK_MSG_SID_INDEX ] = GetNextSendSeqId();
		buffer->fFields.fData[ TRK_MSG_ITEM_HEADER_TYPE_INDEX ] = kDSOSDLLItem;
		buffer->fFields.fData[ TRK_MSG_NCD_LINUX_REMAINING_INDEX ] = remaining;
		buffer->fFields.fData[ TRK_MSG_NCD_LINUX_PID_INDEX ] = processId;
		buffer->fFields.fData[ TRK_MSG_NCD_LINUX_TID_INDEX ] = threadId;
		buffer->fFields.fData[ TRK_MSG_NCD_LINUX_OFFSET_INDEX ] = offset;
		buffer->fFields.fData[ TRK_MSG_NCD_LINUX_LENGTH_INDEX ] = TRK_strlen( name );

		err = TrkConstructMessage( buffer, &gTrkMsgNotifyCreatedDllLinux );

		/*
		** Add the name field.
		*/
		
		if (err == kNoError)
			err = MessageAddBlock( buffer, name,
					buffer->fFields.fData[ TRK_MSG_NCD_LINUX_LENGTH_INDEX ] );

		FAIL(err == kNoError);			/* shouldn't be possible */

		err = DoNotify( buffer );
		ReleaseBuffer(bufferID);		
	}
	
	return( err );
}

/******************************************************************************/
/*
 *	DoOsNotifyDllDeleted - used to send shared object unload notification
 */
/******************************************************************************/

DSError DoOsNotifyDllDeleted( char *name, bool remaining,
							ui32 processId, ui32 threadId )
{
	DSError			err;
	
	MessageBufferId	bufferID;
	MessageBuffer*	buffer;
		
	/*
	 *	Get a send buffer, create a message
	 */
	
	err = GetFreeBuffer(&bufferID, &buffer);
	
	if (err == kNoError) {
		
		/*
		** Construct the message.  The name field is added separately.
		*/
		
		buffer->fFields.fData[ TRK_MSG_COMMAND_INDEX ] = kDSOSNotifyDeleted;
		buffer->fFields.fData[ TRK_MSG_SID_INDEX ] = GetNextSendSeqId();
		buffer->fFields.fData[ TRK_MSG_ITEM_HEADER_TYPE_INDEX ] = kDSOSDLLItem;
		buffer->fFields.fData[ TRK_MSG_NDD_LINUX_REMAINING_INDEX ] = remaining;
		buffer->fFields.fData[ TRK_MSG_NDD_LINUX_PID_INDEX ] = processId;
		buffer->fFields.fData[ TRK_MSG_NDD_LINUX_TID_INDEX ] = threadId;
		buffer->fFields.fData[ TRK_MSG_NDD_LINUX_LENGTH_INDEX ] = TRK_strlen( name );

		err = TrkConstructMessage( buffer, &gTrkMsgNotifyDeletedDllLinux );

		/*
		** Add the name field.
		*/
		
		if (err == kNoError)
			err = MessageAddBlock( buffer, name,
					buffer->fFields.fData[ TRK_MSG_NDD_LINUX_LENGTH_INDEX ] );

		FAIL(err == kNoError);			/* shouldn't be possible */

		err = DoNotify( buffer );
		ReleaseBuffer(bufferID);		
	}
	
	return( err );
}

/******************************************************************************/
/*
 *	DoOsNotifyThreadCreated - used to send thread creation notification
 */
/******************************************************************************/
DSError DoOsNotifyThreadCreated( ui32 processId, ui32 threadId )
{
	DSError			err;
	
	MessageBufferId	bufferID;
	MessageBuffer*	buffer;
		
	/*
	 *	Get a send buffer, create a message
	 */
	
	err = GetFreeBuffer(&bufferID, &buffer);
	
	if (err == kNoError) {
		
		/*
		** Construct the message.
		*/
		
		buffer->fFields.fData[ TRK_MSG_COMMAND_INDEX ] = kDSOSNotifyCreated;
		buffer->fFields.fData[ TRK_MSG_SID_INDEX ] = GetNextSendSeqId();
		buffer->fFields.fData[ TRK_MSG_ITEM_HEADER_TYPE_INDEX ] = kDSOSThreadItem;
		buffer->fFields.fData[ TRK_MSG_NCT_LINUX_PID_INDEX ] = processId;
		buffer->fFields.fData[ TRK_MSG_NCT_LINUX_TID_INDEX ] = threadId;

		err = TrkConstructMessage( buffer, &gTrkMsgNotifyCreatedThreadLinux );

		/*
		** Send out the notification.
		*/

		if (err == kNoError)
			err = DoNotify( buffer );
		ReleaseBuffer(bufferID);		
	}
	
	return( err );
}

/******************************************************************************/
/*
 *	DoOsNotifyThreadDeleted - used to send thread deletion notification
 */
/******************************************************************************/
DSError DoOsNotifyThreadDeleted( ui32 result, ui32 processId, ui32 threadId )
{
	DSError			err;
	
	MessageBufferId	bufferID;
	MessageBuffer*	buffer;
		
	/*
	 *	Get a send buffer, create a message
	 */
	
	err = GetFreeBuffer(&bufferID, &buffer);
	
	if (err == kNoError) {
		
		/*
		** Construct the message.
		*/
		
		buffer->fFields.fData[ TRK_MSG_COMMAND_INDEX ] = kDSOSNotifyDeleted;
		buffer->fFields.fData[ TRK_MSG_SID_INDEX ] = GetNextSendSeqId();
		buffer->fFields.fData[ TRK_MSG_ITEM_HEADER_TYPE_INDEX ] = kDSOSThreadItem;
		buffer->fFields.fData[ TRK_MSG_NDT_LINUX_RESULT_INDEX ] = result;
		buffer->fFields.fData[ TRK_MSG_NDT_LINUX_PID_INDEX ] = processId;
		buffer->fFields.fData[ TRK_MSG_NDT_LINUX_TID_INDEX ] = threadId;

		err = TrkConstructMessage( buffer, &gTrkMsgNotifyDeletedThreadLinux );

		/*
		** Send out the notification.
		*/

		if (err == kNoError)
			err = DoNotify( buffer );
		ReleaseBuffer(bufferID);		
	}
	
	return( err );
}

/******************************************************************************/
/*
 *	DoOsNotifyProcExecCreated - used to send process exec
 */
/******************************************************************************/

DSError DoOsNotifyProcExecCreated( ui32 processId, char *exePath )
{
	DSError			err;
	
	MessageBufferId	bufferID;
	MessageBuffer*	buffer;
		
	/*
	 *	Get a send buffer, create a message
	 */
	
	err = GetFreeBuffer(&bufferID, &buffer);
	
	if (err == kNoError) {
		
		/*
		** Construct the message.
		*/
		
		buffer->fFields.fData[ TRK_MSG_COMMAND_INDEX ] = kDSOSNotifyCreated;
		buffer->fFields.fData[ TRK_MSG_SID_INDEX ] = GetNextSendSeqId();
		buffer->fFields.fData[ TRK_MSG_ITEM_HEADER_TYPE_INDEX ] = kDSOsProcExecItem;
		buffer->fFields.fData[ TRK_MSG_NCPE_LINUX_PID_INDEX ] = processId;
		buffer->fFields.fData[ TRK_MSG_NCPE_LINUX_EXEPATH_INDEX ] = (ui32)exePath;

		err = TrkConstructMessage( buffer, &gTrkMsgNotifyCreatedProcExecLinux );

		FAIL(err == kNoError);			/* shouldn't be possible */

		err = DoNotify( buffer );
		ReleaseBuffer(bufferID);		
	}
	
	return( err );
}
