/******************************************************************************************/
/*
	
DESCRIPTION

	Defines specific to Linux, but common across all Linux platforms.

COPYRIGHT

	Copyright 2001-2004, 2006, 2009 Freescale Semiconductor, Inc.
	
*/
/******************************************************************************************/

#ifndef TRK_LINUX_TARGET_H
#define TRK_LINUX_TARGET_H

#include "dstypes.h"
#include "linux_version.h"
#include "linux_comm.h"		// for ReadUARTPollStage2()

/* 
 * TODO: temporary; remove the auto definition once all linux TRK projects have 
 * been updated to specify the compiler switch -DDDS_MALLOC_MAPPING=DS_MALLOC_ANSI
 * but please leave the #warning.
 */
#ifndef DS_MALLOC_MAPPING
//#warning need to specify -DDS_MALLOC_MAPPING=DS_MALLOC_ANSI in your build
#define DS_MALLOC_MAPPING	DS_MALLOC_ANSI
#endif

#include "mem_TRK.h"		// for TRK_calloc

/****************************************************************************/


#define TRK_BAUD_RATE				kBaud115200		/* not used for Linux */

#define DS_PROTOCOL					DS_PROTOCOL_RTOS

/*
** These two define the process and thread IDs associated with
** CodeWarriorTRK.  They are incorporated into the welcome message.
** If there is a "current" thread when CodeWarriorTRK is initialized,
** use that.
*/

#include "unistd.h"
#define TRK_PROCESS_ID				getpid()
#define TRK_THREAD_ID				0

/*
** For Linux, 1 extra buffer is required in order to store
** information in exception events.
*/

#define TRK_EXTRA_BUFFER_COUNT 		1

/*
** For Linux, it is possible for all threads to simultaneously
** generate events.  The event queue really should be dynamically
** allocated, but this should be large enough for most cases.
*/

#define TRK_EXTRA_EVENT_COUNT 		100


/****************************************************************************/
/*
	ReadUARTPoll
	
	Poll for one character from the UART.  If none are available, return
	immediately.
	
	For optimization, ReadUARTPoll is implemented in 2 stages.  The first
	stage (below) is an inline function which gets the next	character
	from the read buffer.  If the read buffer is empty, then stage 2
	in linux_comm.c does the rest of the work.
*/
/****************************************************************************/
#include "UART.h"
extern char gReadBuf[];
extern int gReadPos;
extern int gReadCount;

#define ReadUARTPoll	InlineReadUARTPoll	// avoid compiler warning

__extern_c
	static inline UARTError InlineReadUARTPoll( char *c )
	{
		if (gReadPos < gReadCount)
		{
			*c = gReadBuf[ gReadPos++ ];
			return kUARTNoError;
		}
		
		return( ReadUARTPollStage2( c ) );
	}
__end_extern_c

/*
 * Support Mask - Used to tell the host which commands are supported by
 * this implementation of CodeWarriorTRK.  If using the default, simply include
 * "default_smask.h".  Otherwise, make a local copy of the contents of
 * that file and modify them here.
 */

#include "linux_smask.h" 


#include "linux_timeout.h"


#endif /* ndef TRK_LINUX_TARGET_H */
