/******************************************************************************************/
/*

DESCRIPTION

	Target-specific implementation for LINUX, processor-independent
	portion.
	

COPYRIGHT

	Copyright 2000-2009 Freescale Semiconductor, Inc.
	
*/
/******************************************************************************************/

/*
** TRK includes
*/
#include "trk_prefix.h"

#include "usr_put.h"		// must be included before msgcmd.h
#include "targimpl.h"
#include "linux_targimpl.h"
#include "linux_port_targimpl.h"
#include "linux_msghndlr.h"
#include "linux_msgtarg.h"
#include "linux_comm.h"
#include "dispatch.h"
#include "msg.h"
#include "reset.h"
#include "nubassrt.h"
#include "defidle.h"
#include "nubinit.h"
#include "msgparse.h"
#include "linux_notify.h"
#include "notify.h"
#include "mainloop.h"
#include "break.h"
#include "step.h"
#include "obarray.h"
#include "support.h"
#ifndef __NotSupportSo__
#include "linux_so.h"
#endif
#include "linux_thread.h"


/*
** Linux includes
*/

#include <malloc.h>
#ifndef __uClinux__
#include <sys/ptrace.h>
#endif
#include <errno.h>
#include <signal.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#ifndef __uClinux__
#include <sys/ipc.h>
#include <sys/sem.h>
#else
#include <linux/ipc.h>
#include <linux/sem.h>
#endif
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <utime.h>
#include <dirent.h>
#include <string.h>

/*
** for uClinux without mmu, must ensure that the memory is
** in the range of physical memory.
*/
#ifdef __uClinux__

	extern unsigned int gRamsize;

	#define BEGIN_NOMMU_MEM_VALIDATION(ACCESS_ADDRESS)	if ( ( gRamsize ) && ( ACCESS_ADDRESS > gRamsize ) )	\
														{														\
															err = kInvalidMemory;								\
														}														\
														else													\
														{									
											
	#define END_NOMMU_MEM_VALIDATION					}
										
#else

	#define BEGIN_NOMMU_MEM_VALIDATION(ACCESS_ADDRESS)	//leave it blank
	#define END_NOMMU_MEM_VALIDATION					//leave it blank

#endif //ifdef __uClinux__

/*
** Set this to 1 to get detailed debug output when building the OS process list.
*/
#define DEBUG_LWPID_LIST		(0)
#define DEBUG_PROCESS_LIST		(0)

OsLwpidItem* lwpidList = NULL;
int g_lwpidCount = 0;

#define MIN(x,y)				(((x) > (y)) ? (y) : (x))
#define MAX(x,y)				(((x) > (y)) ? (x) : (y))

/*
** ptrace() operates on words.  Since the declaration has it returning
** "long int", use that to determine what the word size on the current
** architecture.  This may turn out to be too simplistic.
*/

#define PT_TRANSFER_SIZE		sizeof( long int )
#define PT_WORD_BOUND( a )		((a) - ((a) % sizeof( PT_TRANSFER_SIZE )))

/****************************************************************************/
/*
 *	Storage for info about processes.  This is the list of processes we
 *	are currently debugging.
 */
/***************************************************************************/


static LinuxProcessState	gProcessObjects[100];
static ObjectArray			gProcesses;

#define kProcessCount		(sizeof(gProcessObjects)/sizeof(gProcessObjects[0]))

/****************************************************************************/
/*
 *	Default initial signal policy for new processes.
 */
/***************************************************************************/
static TrkSignalMaskLinux	gDefaultStopMask;
static TrkSignalMaskLinux	gDefaultPassMask;

/****************************************************************************/
/*
 *	Linked list element used to store the full list of processes in the
 *	OS.  This is not the same as the list of processes we are currently
 *	debugging.
 */
/***************************************************************************/

typedef struct OsProcessItem
{
	struct OsProcessItem *next;
	ui32	id;
	ui8		state;
	bool	multiThreaded;
	char	*name;
} OsProcessItem;


/****************************************************************************/
/*
 *	Flag for file IO block status.
 */
/***************************************************************************/
static bool gBlockFileIo = false;

/****************************************************************************/
/*
 *	Static functions
 */
/***************************************************************************/

static DSError InitializeLinuxProcessState( ObjectHeader *ob );
static int TrkSelectIo( bool consoleOnly, ui32 processId, struct timeval *timeout );
static DSError	TrkAddChildProcessLinux( int processId );
static DSError TrkHandleExecEvent( LinuxProcessState *pState, LinuxThreadState *tState );
static OsLwpidItem* TRKBuildOsLwpidList(int* listCount);
static OsLwpidItem* TRKBuildOsLwpidListTaskMode(int* listCount);
static int TRKBuildOSLwpidRelationships(OsLwpidItem* lwpidList, int lwpidCount);
static int IsSubthreadOf(OsLwpidItem* parent, OsLwpidItem* child);


/****************************************************************************/
/*
 *	Support Event cmds. For each particular support event, we expose
 *  a dedicated function to post it. And since the handling of the events
 *  are in this compilcation unit as well, only this file needs to know
 *  the actual event details.
 */
/***************************************************************************/
enum 
{
	kSupportEvent_sendInitConsoleToDebugger	= 1
};

/****************************************************************************/
/*
 *	OsTemporarilyStopThread
 *
 *	Called to stop a thread so a memory or a register can be read/written.
 *  The calling function should make sure to automatically resume the thread 
 *  ONLY if the out parameter bStoppedByRequest is set to true. If it's false,
 *  (and kNoError is returned) then that means the thread was stopped, but
 *  coincidentally through some other action (a breakpoint/watchpoint was hit or
 *  something else that had nothing to do with the caller calling this function). 
 *  In that case, we will automatically call PostInterruptEvent to ensure that
 *  the thread stop/exit is handled normally and the debugger is notified; the
 *  caller need not do anything.
 */
/****************************************************************************/
DSError OsTemporarilyStopThread(ui32 processId, LinuxThreadState *tState, bool * bStoppedByRequest)
{
	DSError err = kNoError;
					
	*bStoppedByRequest = false;	// init return param

	err = OsTargetStop(kDSStopThread, processId, tState->fThreadId);
	if (err == kNoError)
	{
		/* Wait here until the thread has actually stopped. */
		int status = 0;

		ui32 tempId = waitpid(tState->fLwProcessId, &status, WUNTRACED);
		if (tempId == -1)
			tempId = waitpid(tState->fLwProcessId, &status, WUNTRACED | __WCLONE);
		if (tempId == -1)
		{
			/* Strange; waitpid failed. Could be any number of reasons, but we know it's not because the thread stopped */
			__perror( "Unable to temporarily stop process to access memory." );				
			return kInvalidThreadId;
		}

		
		/* Because of how we called waitpid, this shouldn't happen. */
		ASSERT(tempId == tState->fLwProcessId);

		/* 
		 * Check to see if the thread has actually stopped, rather than just
		 * exited, which could have just happened by coincidence. Also
		 * check that the reason it stopped is because we forced it to
		 * and not because of a breakpoint or watchpoint, which, again,
		 * may be the case by coincidence.
		 */
		if (WIFSTOPPED(status))
		{
			/* sanity check */
			errno = 0;
			ptrace(PT_READ_U, tState->fLwProcessId, 0, 0);
			ASSERT(errno == 0);

			if (WSTOPSIG(status) == SIGSTOP)
			{
				/* 
				 * Tell the caller the thread stopped because we forced it to.
				 * The caller is responsible for resuming the thread after
				 * he's done doing what he needs to do
				 */
				*bStoppedByRequest = true;
			}
			else
			{
				/* 
				 * The thread stopped for some other reason. We have to let
				 * the debugger know about it. The caller must NOT resume
				 * the thread.
				 */
				*bStoppedByRequest = false;
				
				PostInterruptEvent(status, processId, tState->fThreadId);
			}
		}
		else
		{
			/*
			 * The thread exited, either normally or because of a signal. 
			 * Either way, the thread isn't stopped--it's nonexistent, so
			 * just return kInvalidThreadId, but first post an interrupt
			 * event so the debugger gets notified.
			 */
			PostInterruptEvent(status, processId, tState->fThreadId);
			err = kInvalidThreadId;
		}
	}
	
	return err;
}

/****************************************************************************/
/*
 *	OsTargetAccessMemory
 *
 *	Read/write memory, testing for accessability
 */
/****************************************************************************/
DSError OsTargetAccessMemory(void* data,
						void* virtualAddress,
						size_t* memorySize,
						MemoryAccessOptions accessOptions,
						bool read,
						ui32 processId,
						ui32 threadId )
{	
	DSError err = kNoError;
	unsigned char* ptBuf = NULL;
	lwpid_t lwpid;
	LinuxThreadState *tState;
	

	/*
	** ptrace performs word transfers, so this needs to get both the lower
	** and upper longword boundaries.
	*/
	PCType ptAddr = PT_WORD_BOUND((PCType)virtualAddress);
	long addrOffset = (PCType)virtualAddress - ptAddr;
	long ptCount = PT_WORD_BOUND(addrOffset + *memorySize + PT_TRANSFER_SIZE-1);
	long bytesWritten = 0;
	unsigned long *currBuf;
	unsigned long currAddress;
	bool bResumeThread = false;

	/*
	** Reject if the debugger tries to access TRK memory.
	*/
	if (processId == getpid())
		return( kInvalidProcessId );

	/*
	** Since all the threads for a process share the same memory, a read
	** on the main process should work.  However, ptrace can only access
	** stopped processes.  On top of that, the main process may have died
	** while its threads live on.  For that reason, we grab the first lwpid
	** associated with this process which is accessible via ptrace.
	*/

	if (FindLinuxThreadStateLwAccessible( processId, NULL, &tState ) == kNoError)
	{
		lwpid = tState->fLwProcessId;
	}
	else
	{
		/* 
		 * We're probably here because all threads in the process are running.
		 * Let's grab the first thread and try to stop it. 
		 */
		err = FindLinuxThreadStateLwFirst(processId, NULL, &tState);
		if (err == kNoError)
		{
			err = OsTemporarilyStopThread(processId, tState, &bResumeThread);
			if (err == kNoError)
				lwpid = tState->fLwProcessId;
		}
	}
			
	if (!lwpid)
		return( kInvalidProcessId );
	
	ptBuf = (unsigned char *)malloc( sizeof( unsigned char ) * ptCount );
	if (ptBuf == NULL)
		return( kInternalError );

	currBuf = (unsigned long*)ptBuf;
	currAddress = (unsigned long)ptAddr;

	if (!read)
	{
		//
		// fill the lower word if below the starting point
		//
		if (ptAddr != (PCType)virtualAddress)
		{
			BEGIN_NOMMU_MEM_VALIDATION(ptAddr)
		  	*(int*)ptBuf = ptrace(PT_READ_I, lwpid, ptAddr, 0);
			if (errno)
			{
				err = kOsError;
				__perror( "ptrace memory write backfill failed" );
			}
			END_NOMMU_MEM_VALIDATION
			
			
		}

		//
		// if there are more bytes difference total than there are bytes
		// difference at the beginning, then there are obviously also
		// bytes at the end to get to write back out
		//
		if ((ptCount - *memorySize) > addrOffset)
		{
			BEGIN_NOMMU_MEM_VALIDATION(ptAddr + (ptCount-PT_TRANSFER_SIZE))
			*(int*)(ptBuf+(ptCount-PT_TRANSFER_SIZE))
			  = ptrace(PT_READ_I, lwpid, ptAddr + (ptCount-PT_TRANSFER_SIZE));
			if (errno)
			{
				err = kOsError;
				__perror( "ptrace memory write backfill failed" );
			}
			END_NOMMU_MEM_VALIDATION
		}

		// now, put in the buffer passed by the caller
		memcpy(ptBuf + addrOffset, data, *memorySize);
		
		
	}
	
	// loop, accessing a max of PT_TRANSFER_SIZE bytes of data at a time
	while (ptCount > 0 && err == kNoError)
	{
		// break read/write request into sizeof(int) length chunks
		// (yes, that's right...one word at a time!!)
		if (read)
		{
			BEGIN_NOMMU_MEM_VALIDATION(currAddress)
			*currBuf = ptrace(PT_READ_I, lwpid, currAddress);
			if (errno)
			{
				err = kOsError;
				__perror( "ptrace memory read failed" );
			}
			END_NOMMU_MEM_VALIDATION
		}
		else
		{
			BEGIN_NOMMU_MEM_VALIDATION(currAddress)
			if (-1 == ptrace(PT_WRITE_I, lwpid, currAddress, *currBuf))
			{
				err = kOsError;
				__perror( "ptrace memory write failed" );
			}
			END_NOMMU_MEM_VALIDATION
		}

		if (err == kNoError)
		{
			// update how much to access, how much accessed, our buffer & address
			ptCount -= PT_TRANSFER_SIZE;
			currAddress += PT_TRANSFER_SIZE;
			currBuf ++;
		
			// For writes, we'll want to set the caller's memorSize var to the number of 
			// bytes actually written (from his buffer), even in the case of a failure.
			// For example, say he asks to write 100 bytes, but we encounter a write
			// failure after having written 80 bytes. For reads, it's simpler since we treat
			// that as an all or nothing thing. 
			//
			if (!read)		
				bytesWritten += PT_TRANSFER_SIZE;
		}
	} // while (ptCount > 0 && err == kNoError)

	if (read)
	{
		if (accessOptions == kUserMemory)
		{
			err = HideBreakpoints( ptBuf + addrOffset, (unsigned int)virtualAddress, 
				*memorySize, processId, threadId );
		}
		
		// Put the contents of the local buffer into the caller's buffer. Note
		// that reads are "all or nothing".
		if (err == kNoError)
			memcpy(data, ptBuf+addrOffset, *memorySize);
		else
			*memorySize = 0;
	}
	else
	{
		if (bytesWritten)
		{
			// Determine exactly how many bytes from the caller's buffer were written.
			// 'bytesWritten' is a multiple of PT_TRANSFER_SIZE, so we have to do
			// some math to get the right number...
			
			// Adjust for the caller's start address not falling on a read/write boundary
			bytesWritten -= addrOffset;

			// Adjust for the caller's effective end address not falling on a read/write boundary.
			// Note that if we were unable to write all the words we needed (because of an error),
			// the numer of trailing bytes written that are not from the caller's buffer is zero.
			if (err == kNoError)		
			{
				unsigned long incursionCount = (((unsigned long)virtualAddress + *memorySize) % PT_TRANSFER_SIZE);
				if (incursionCount)
					bytesWritten -= (PT_TRANSFER_SIZE - incursionCount);
			}
		}
		
		// Set the caller's param
		*memorySize = bytesWritten;
	}

	free(ptBuf);		

	
	/* We forced the thread to stop so we could access memory. Now we resume it */
	if (bResumeThread)
		OsTargetContinue(processId, tState->fThreadId);	
		
	return err;
}						

void displayRegParameters(unsigned int firstRegister, unsigned int lastRegister,
						MessageBuffer *b, size_t* registerStorageSize,
						bool read,
						ui32 processId,
						ui32 threadId )
{
	__puts("\n DISPLAY REGISTER-REQ PARAMETERS:");
	__puts("\n firstRegister = ");
	__puthex8(firstRegister);
	__puts("\n lastRegister = ");
	__puthex8(lastRegister);
	__puts("\n MessageBuffer address = ");
	__puthex8((ui32)b);
	__puts("\n registerStorageSize = ");
	__puthex8(*registerStorageSize);
	if ( read )
	  __puts("\n read = true");
	else
	  __puts("\n read = false");
	__puts("\n processId = ");
	__puthex8(processId);
	__puts("\n threadId = ");
	__puthex8(threadId);
	__puts("\n");
	
}

/****************************************************************************/
/*
 *	OsTargetAccessDefault
 *
 *	R/W a range in the default register block.  
 *  W/R the registers in the range directly to/from the given message buffer.  
 *  Return the size of the requested range in bytes.
 */
/****************************************************************************/
DSError	OsTargetAccessDefault(unsigned int firstRegister, unsigned int lastRegister,
						MessageBuffer *b, size_t* registerStorageSize,
						bool read,
						ui32 processId,
						ui32 threadId )
{
	DSError err = kNoError;
	unsigned int reg;
	ui32 value;

    displayRegParameters(firstRegister, lastRegister, b, registerStorageSize, read, processId, threadId);
	/*
	** Reject if the debugger tries to operate on CodeWarriorTRK.
	*/
	if (processId == getpid())
		return( kInvalidProcessId );

	if (firstRegister < TRK_DEFAULT_MIN_REGISTER ||
		lastRegister > TRK_DEFAULT_MAX_REGISTER)
		return( kInvalidRegister );


	ASSERT(firstRegister <= lastRegister);
	
	/*
	** Iterate through the registers.  For each, read/write it
	** into/from the provided buffer.
	*/
	
          

	for (reg = firstRegister, *registerStorageSize = 0;
		 reg <= lastRegister && err == kNoError;
		 reg++)
	{
		*registerStorageSize += sizeof( DefaultType );
		
		if (read)
		{
			err = OsTargetAccessDefaultSingle( reg, &value, read,
					processId, threadId );
			if (err == kNoError)
				err = MessageAdd_ui32( b, value );
		}
		else
		{
			err = MessageGet_ui32( b, &value );
			if (err == kNoError)
				err = OsTargetAccessDefaultSingle( reg, &value, read,
						processId, threadId );
		}

            {
             	__puts(" GPR");
             	__puthex2(reg);
             	__puts(": ");
             	__puthex8( value );          
             	__puts("\t");
             	if ( reg % 5 == 4 )
             	  __puts("\n");
            }
	}


            __puts("\n");
          
	return( err );
}						

/****************************************************************************/
/*
 *	OsTargetAccessDefaultSingle
 *
 *	R/W a register at the specified offset in user memory.
 */
/****************************************************************************/
DSError	OsTargetAccessDefaultSingle(unsigned int reg, DefaultType *value,
						bool read,
						ui32 processId,
						ui32 threadId )
{
	LinuxProcessState *pState;
	LinuxThreadState *tState;
	DSError err = kNoError;

	/*
	** If the process has an associated thread agent, dispatch to a
	** thread-aware implementation.  Otherwise, dispatch to a
	** process-only implementation.
	*/

	if (FindLinuxThreadStateId( processId, threadId, NULL, &tState ) != kNoError)
		err = kInvalidThreadId;
	
	if (err == kNoError)
	{
		pState = TrkGetLinuxProcessState( tState->fProcessStateId );
		ASSERT( pState );
		
#ifndef __NotSupportThread__
		if (pState->fThreadAgent == NULL)
			err = TrkAccessDefaultSingleProcessLinux( reg, value, read, processId );
		else
			err = TrkAccessDefaultSingleThreadLinux( reg, value, read, tState );

#else
		err = TrkAccessDefaultSingleProcessLinux( reg, value, read, processId );
#endif
	}
	
	return( err );
}

/****************************************************************************/
/*
 *	TrkAccessDefaultSingleProcessLinux
 *
 *	R/W a register at the specified offset in user memory.
 *
 *	NOTE:This is a low-level implementation that can be used
 *	on the light-weight process for a thread.  This means that
 *	processId may NOT be the main process ID for the given thread.  This
 *	is important, because many of the TRK functions which accept a
 *	processId will ONLY accept the main process ID.  Avoid calling other
 *	TRK functions with the given process ID.
 */
/****************************************************************************/
DSError	TrkAccessDefaultSingleProcessLinux(unsigned int reg, DefaultType *value,
						bool read,
						ui32 processId)
{
	DSError err = kNoError;
	unsigned int regOffset;
	
	/*
	** Reject if the debugger tries to operate on CodeWarriorTRK.
	*/

	if (processId == getpid())
		return( kInvalidProcessId );
		
	/*
	** Map CodeWarriorTRK register id to Linux register offset.
	*/
	
	regOffset = TrkDefaultIdToPtraceOffset[ reg ];

	/*
	** Use ptrace to access the register at the specified offset.
	** The default registers are all 1 word, so a single ptrace
	** works.
	*/

	errno = 0;

	if (read)
		*value = ptrace(PTRACE_PEEKUSER, processId, regOffset);
	else
		ptrace(PTRACE_POKEUSER, processId, regOffset, *value);

	if (errno != 0)
	{
		err = kOsError;
		__perror( "ptrace register access failed" );
	}
	
	return( err );
}

/****************************************************************************/
/*
 *	OsTargetAccessFP
 *
 *	R/W a range of floating point registers.  W/R the registers in the
 *  range directly to/from the given message buffer.  Returns the size of the
 *  requested range in bytes.
 */
/****************************************************************************/
DSError	OsTargetAccessFP(unsigned int firstRegister, unsigned int lastRegister,
						MessageBuffer *b, size_t* registerStorageSize,
						bool read,
						ui32 processId,
						ui32 threadId )
{
	LinuxProcessState *pState;
	LinuxThreadState *tState;
	DSError err = kNoError;

	/*
	** If the process has an associated thread agent, dispatch to a
	** thread-aware implementation.  Otherwise, dispatch to a
	** process-only implementation.
	*/

	if (FindLinuxThreadStateId( processId, threadId, NULL, &tState ) != kNoError)
		err = kInvalidThreadId;
	
	if (err == kNoError)
	{
		pState = TrkGetLinuxProcessState( tState->fProcessStateId );
		ASSERT( pState );
		
#ifndef __NotSupportThread__
		if (pState->fThreadAgent == NULL)
			err = TrkAccessFPProcessLinux( firstRegister, lastRegister, b,
					registerStorageSize, read, processId );
		else
			err = TrkAccessFPThreadLinux( firstRegister, lastRegister, b,
					registerStorageSize, read, tState );
		
#else
		err = TrkAccessFPProcessLinux( firstRegister, lastRegister, b,
				registerStorageSize, read, processId );
#endif
	}
	
	return( err );
}						

/****************************************************************************/
/*
 *	OsTargetContinue
 *
 *	Continue execution of the target thread.
 */
/****************************************************************************/
DSError	OsTargetContinue( ui32 processId, ui32 threadId )
{	
	LinuxProcessState *pState;
	LinuxThreadState *tState;
	DSError err = kNoError;
	Breakpoint *tdb;
	BreakpointId tdbid;

	__puts( "OsTargetContinue " );
	__puthex8( processId );
	__puts( ":" );
	__puthex8( threadId );
	__puts( "\r\n" );
	
	
	/*
	** Reject if this isn't one of the threads we already know about.
	*/

	if (FindLinuxThreadStateId( processId, threadId, NULL, &tState ) != kNoError)
		err = kInvalidThreadId;
	
	
	
	/*
	** Check for an execution breakpoint at the current PC.
	** If one is found, disable it temporarily and perform
	** a single-step.  Otherwise, use tState->fContinueType
	** to determine whether to do a continue or single-step.
	**
	** NOTE: We do not do this for internally-generated continue
	** events, such as those generated by a multi-line step.
	** We can distinguish these because the thread's stop status
	** will already be false.
	*/
	if (err == kNoError &&
		OsTargetStopped( processId, threadId) &&
		FindExecBreakpoint( &tdbid, &tdb,
							OsTargetGetPC( processId, threadId ),
							1, true, false, processId, threadId ) == kNoError)
	{
		__puts( "TargetContinue: Breakpoint " );
		__puthex8( tdbid );
		__puts( " overlaps PC" );
		__puts( "\r\n" );
	
		TargetSetupTempDisableStep( tdbid, processId, threadId );
	}
	

	if (err == kNoError)
		OsTargetSetStopped( false, processId, threadId );
		
	/*
	** If the process has an associated thread agent, dispatch to a
	** thread-aware implementation.  Otherwise, dispatch to a
	** process-only implementation.
	*/

	if (err == kNoError)
	{
		pState = TrkGetLinuxProcessState( tState->fProcessStateId );
		ASSERT( pState );
		
#ifndef __NotSupportThread__
		if (pState->fThreadAgent == NULL)
			err = TrkContinueProcessLinux( processId, tState );
		else
			err = TrkContinueThreadLinux( tState );
		
#else
		err = TrkContinueProcessLinux( processId, tState );
#endif
	}
	
	return( err );
}

/****************************************************************************/
/*
 *	TrkContinueProcessLinux
 *
 *	Continue execution of the target thread, given a specific thread state
 *	and process ID.
 *
 *	NOTE:This is a low-level implementation that can be used
 *	to continue the light-weight process for a thread.  This means that
 *	processId may NOT be the main process ID for the given thread.  This
 *	is important, because many of the TRK functions which accept a
 *	processId will ONLY accept the main process ID.  Avoid calling other
 *	TRK functions with the given process ID.
 */
/****************************************************************************/
DSError	TrkContinueProcessLinux( ui32 processId, LinuxThreadState *tState )
{
	DSError err = kNoError;
	unsigned long code = 0;
	LinuxProcessState *pState;

	/*
	** Reject if the debugger tries to operate on CodeWarriorTRK.
	*/
	if (processId == getpid())
		return( kInvalidProcessId );

	pState = TrkGetLinuxProcessState( tState->fProcessStateId );
	ASSERT( pState != NULL );
	

	// args to PTRACE
	// arg[0] is the request type:  PT_CONTINUE or PT_SINGLESTEP
	// arg[1] is the pid
	// arg[2] always 1 so arg[3] will be interpreted, as necessary
	// arg[3] if 0, throw away all signals
	//        if <valid SIGno>, continue child with that signal for interrupt;
	// if ptrace doesn't return -1, all is okay; otherwise
	// an error occurred, and errno contains the ERRNUM of the problem
	//
	
	/*
	** Check signal pass mask to determine whether the signal for this
	** process should be passed on.
	** If it is outside the mask bounds, default is to pass on the signal.
	*/
	if (tState->fSignal > TRK_SIGNAL_MAX_LINUX ||
		TRK_MASK_IS_SET( pState->fPassMask, tState->fSignal ))
	{
		__puts( "Continuing with signal " );
		__puthex8( tState->fSignal );
		__puts( "\r\n" );
		code = tState->fSignal;
	}
	
	

	/*
	** Use tState->fContinueType to determine whether to do a continue
	** or single-step.
	*/
	
	if (ptrace(tState->fContinueType, processId, 1, code) == -1)
	{
		err = kOsError;
		__perror( "failed to continue process" );
	}
	
	return( err );
}						

/****************************************************************************/
/*
 *	OsTargetStopped
 *
 *	Is the specified thread stopped?
 */
/****************************************************************************/
bool OsTargetStopped( ui32 processId, ui32 threadId )
{
	LinuxThreadState *tState;
	
	/*
	** Reject if the specified thread is not in the list of controlled threads.
	*/
	if (FindLinuxThreadStateId( processId, threadId, NULL, &tState ) != kNoError)
		return( false );

	return( tState->fStopped );
}						

/****************************************************************************/
/*
 *	OsTargetStop
 *
 *	Stop the specified thread(s).
 */
/****************************************************************************/
DSError OsTargetStop( ui8 options, ui32 processId, ui32 threadId )
{
	DSError err = kNoError;
	LinuxProcessState *pState;
	LinuxThreadState *tState;
	ObjectId i;

	switch (options)
	{
		case kDSStopThread:
			/*
			** If the process has an associated thread agent, dispatch to a
			** thread-aware implementation.  Otherwise, dispatch to a
			** process-only implementation.
			*/
		
			if (FindLinuxThreadStateId( processId, threadId, &i, &tState ) != kNoError)
				err = kInvalidThreadId;
			
			if (err == kNoError)
			{
				pState = TrkGetLinuxProcessState( tState->fProcessStateId );
				ASSERT( pState );
				
				if (pState->fThreadAgent == NULL)
					err = TrkStopProcessLinux( processId );
				else
					err = TrkStopThreadLinux( tState );
			}
			break;
		case kDSStopProcess:
		case kDSStopSystem:
		default:
			err = kUnsupportedError;
			break;
	}
	
	return( err );
}						

/****************************************************************************/
/*
 *	TrkStopProcessLinux
 *
 *	Stop the specified process given only the process ID. 
 *
 *	NOTE:This is a low-level implementation that can be used
 *	to stop the light-weight process for a thread.  This means that
 *	processId may NOT be the main process ID for the given thread.  This
 *	is important, because many of the TRK functions which accept a
 *	processId will ONLY accept the main process ID.  Avoid calling other
 *	TRK functions with the given process ID.
 */
/****************************************************************************/
DSError TrkStopProcessLinux( ui32 processId )
{
	kill( processId, SIGSTOP );
	return( kNoError );
}

/****************************************************************************/
/*
 *	OsTargetSetStopped
 *
 *	Sets the stopped status of the target process.  Set to true if
 *	the target process is stopped, false if it is temporarily
 *	suspended.
 */
/****************************************************************************/
void OsTargetSetStopped( bool stop_state, ui32 processId, ui32 threadId )
{
	LinuxThreadState *tState;

	/*
	** Reject if the specified thread is not in the list of controlled threads.
	*/
	if (FindLinuxThreadStateId( processId, threadId, NULL, &tState ) != kNoError)
		return;

	tState->fStopped = stop_state;
}

/****************************************************************************/
/*
 *	OsTargetAddStopInfo
 *
 *  Building NotifyStopped message to send to the host.  Add target-specific
 *  block to the message buffer.
 */
/****************************************************************************/
DSError OsTargetAddStopInfo( MessageBuffer* b, ui32 processId, ui32 threadId )
{
	DSError err;
	
	LinuxThreadState *tState;
	
	/*
	** Reject if the specified process is not in the list of controlled processes.
	*/
	if (FindLinuxThreadStateId( processId, threadId, NULL, &tState ) != kNoError)
		return( kInvalidThreadId );

	b->fFields.fData[ TRK_MSG_NS_LINUX_PC_INDEX ] =
		(ui32)OsTargetGetPC( processId, threadId );
	b->fFields.fData[ TRK_MSG_NS_LINUX_LASTPC_INDEX ] = tState->fLastPc;
	b->fFields.fData[ TRK_MSG_NS_LINUX_SIGNAL_INDEX ] = tState->fSignal;
	b->fFields.fData[ TRK_MSG_NS_LINUX_PID_INDEX ] = processId;
	b->fFields.fData[ TRK_MSG_NS_LINUX_TID_INDEX ] = threadId;

	err = TrkConstructMessage( b, &gTrkMsgNotifyStoppedLinux );

	return( err );
}

/****************************************************************************/
/*
 *	__reset
 *
 *	Reset system to initial state.
 */
/****************************************************************************/
void __reset( void )
{
	/*
	** Do nothing.  We are debugging applications, not the OS.  At
	** this level, the debugger shouldn't have the ability to reset
	** the hardware.
	*/
}

/****************************************************************************/
/*
 *	OsTargetValidStep
 *
 *	Verify that stepping is OK in the specified thread.
 */
/****************************************************************************/
DSError OsTargetValidStep( ui32 processId, ui32 threadId )
{
	LinuxThreadState *tState;
	
	/*
	** Reject if the debugger tries to operate on CodeWarriorTRK.
	*/
	if (processId == getpid())
		return( kInvalidProcessId );
	

	/*
	** Reject if the specified process is not in the list of controlled processes.
	*/
	if (FindLinuxThreadStateId( processId, threadId, NULL, &tState ) != kNoError)
		return( kInvalidThreadId );

	return( kNoError );
}

/****************************************************************************/
/*
 *	InitializeTarget
 *
 *	Performs initialization of target-program state that is done at startup.
 *
 */
/****************************************************************************/
int wakePipe[2];
#ifdef __uClinux__
#undef SIGRTMIN
#undef SIGRTMAX
#define SIGRTMIN SIGUNUSED
#define SIGRTMAX SIGUNUSED
#endif

void SigWake( int s /*, siginfo_t *i, void *p */)
{
	/*
	** Writing to this pipe will force select() to wakeup when 
	** TRK waits for input from the debugger.  This is needed
	** in case the signal happens after the call waitpid() but
	** before the call to select().
	*/
	__puts( "\r\nSigWake: ***wakeup event***\r\n" );
	__puts( "SigWake: The awaken proccess has the following PID = 0x");
	__puthex4(getpid());
	__puts("\n\r");

	write( wakePipe[1], "\0", 1 );	
}
DSError InitializeTarget()
{
	DispatchEntry entry;
	struct sigaction act;
	sigset_t sset;
	int i;
	
	/*
	** Add Linux-specific handler functions to the command
	** dispatch table.
	*/
	
	entry.fn = &DoOsItemRequest;
	OverrideDispatch( kDSOSCreateItem, &entry, NULL );
	entry.fn = &DoOsItemRequest;
	OverrideDispatch( kDSOSDeleteItem, &entry, NULL );
	entry.fn = &DoOsInfoRequest;
	OverrideDispatch( kDSOSReadInfo, &entry, NULL );
	entry.fn = &DoOsInfoRequest;
	OverrideDispatch( kDSOSWriteInfo, &entry, NULL );
	
	/*
	** Set up the default signal policy.  By default, stop on
	** everything except real-time signals.  The pass-mask is 
	** the inverse of that, don't pass on anything except the
	** real-time signals.
	*/  
	
	memset( &gDefaultStopMask, 0xff, sizeof( gDefaultStopMask ) );
	memset( &gDefaultPassMask, 0x00, sizeof( gDefaultPassMask ) );
	for (i = MIN( SIGRTMIN, TRK_SIGNAL_MAX_LINUX ); 
		 i <= MIN( SIGRTMAX, TRK_SIGNAL_MAX_LINUX );
		 i++)
	{
		TRK_MASK_CLEAR( gDefaultStopMask, i );
		TRK_MASK_SET( gDefaultPassMask, i );
	}

	/*
	** Initialize the process status array.
	*/
	InitializeObjectArray( &gProcesses, kProcessCount, sizeof(gProcessObjects[0]),
							InitializeLinuxProcessState, &gProcessObjects );
							
	/*
	** Initialize the thread status array.
	*/

	InitializeThreadArray();
	
	/*
	** This should prevent us from dying from real-time signals when
	** debugging threaded apps.
	*/
	
	sigemptyset( &sset );
	for (i = SIGRTMIN; i <= SIGRTMAX; i++)
		sigaddset( &sset, i );

	sigprocmask( SIG_BLOCK, &sset, NULL );
	
	/*
	** Setup SIGCHLD signals to wakeup TRK when it is idle.  This will
	** ensure that the select() routine (used by TrkSelectIo() in Idle())
	** will return when a child process stops or dies.
	**
	** Open the wakeup pipe and set both ends to non-blocking mode.
	*/

	pipe( wakePipe );
	fcntl( wakePipe[0], F_SETFL, O_NONBLOCK );
	fcntl( wakePipe[1], F_SETFL, O_NONBLOCK );

	sigemptyset( &act.sa_mask );
	act.sa_flags = SA_RESTART;
	act.sa_handler = SigWake;
	sigaction( SIGCHLD, &act, NULL );
    	
	/*
	** Don't die when pipes break.
	*/

	signal( SIGPIPE, SIG_IGN );
	
	/*
	** Initialize the thread support routines.
	*/

#ifndef __NotSupportThread__
	TrkInitializeThreadSupportLinux();
#endif	

	return( kNoError );
}

/****************************************************************************/
/*
 *	CleanupTarget
 *
 *	Performs cleanup of target-program state to prepare it for a new 
 *	debug session.  Removes any residual state from the previous session,
 *	such as running processes and breakpoints.
 *
 */
/****************************************************************************/
DSError CleanupTarget()
{
	DSError err = kNoError;
	ObjectId i;
	LinuxProcessState *pState;

	/*
	** Iterate over all the processes in the list of controlled processes
	** and kill them.
	*/

	for (i = IterateObjectArrayFirst( &gProcesses, (ObjectHeader **)&pState );
	     i != kInvalidObjectId && err == kNoError;
	     i = IterateObjectArrayNext( &gProcesses, i, (ObjectHeader **)&pState ))
	{
		err = OsTargetKillProcessLinux( pState->fProcessId );
	}
	
	/*
	** Reset the file IO blocking flag.
	*/

	gBlockFileIo = false;

	return( err );
}

/****************************************************************************/
/*
 *	PostInterruptEvent
 *
 *	Post an "interrupt/task stopped" event to the event queue.  This
 *	function can be called at any time, allowing concurrent execution to be
 *	synchronized through the event loop.
 */
/****************************************************************************/
void PostInterruptEvent( int status, ui32 processId, ui32 threadId )
{
	NubEvent		event;

	ConstructEvent(&event, kExceptionEvent);

	/*
	** Store relevant info in the event.  Store the status as the
	** event sub-type.
	*/

	event.fInfo[0] = status;	
	event.fProcessId = processId;
	event.fThreadId = threadId;

	/*
	 *	Post an event to be handled in the main loop
	 */
	PostEvent(&event);
}

/****************************************************************************/
/*
 *	Idle
 *
 *	Find a productive way to kill time while waiting for something
 *	interesting to happen.
 */
/****************************************************************************/
void Idle( void )
{
	pid_t processId;
	int status;
	LinuxThreadState *tState;
	LinuxProcessState *pState;
	char c;
	struct timeval timeout;
	
	/*
	** TRK is done with all of its processing.  Check to see if a child
	** has stopped or terminated.  If one has, post an event for it and
	** return.  Otherwise, wait for something to happen.
	** 
	** Note: The order of these two waitpid() calls is important.  This
	** ordering prevents zombies from piling up when the threads 
	** die.  If we call with the __WCLONE option first, then it seems
	** we may not get termination status for all of them.
	*/
	
	/*
	** Clear out the wakeup pipe.
	*/


	while (read( wakePipe[0], &c, 1 ) > 0)
		;

	processId = waitpid( -1, &status, WNOHANG | WUNTRACED );
	if (processId <= 0)
	{
		processId = waitpid( -1, &status, WNOHANG | __WCLONE );
	}
		
	if (processId > 0)
	{
		/*
		** If the process ID matches the LW process ID for one of
		** the threads, post the event against that thread and its
		** primary process ID.  Otherwise, post it against the given
		** process ID with a default thread ID.
		*/

		if (FindLinuxThreadStateLwId( processId, NULL, &tState ) == kNoError)
		{
			pState = TrkGetLinuxProcessState( tState->fProcessStateId );
			ASSERT( pState );
			PostInterruptEvent( status, pState->fProcessId, tState->fThreadId );
			
			__puts( "Posting interrupt to thread " );
			__puthex8( pState->fProcessId );
			__puts( ":" );
			__puthex8( tState->fThreadId );
			__puts( " (" );
			__puthex8( processId );
			__puts( ")\r\n" );
		}
		else
		{
			PostInterruptEvent( status, processId, 0 );
			
			__puts( "Posting interrupt to process " );
			__puthex8( processId );
			__puts( "\r\n" );
		}
	}
	else
	{
		/*
		** Use select() on the input stream.  The only parameter to this function
		** is the timeout.  This function will return when input is available,
		** SIGCHLD is caught (via wakePipe), or when there is console IO 
		** activity for one of the target programs.
		**
		** Note: Under normal circumstances, this shouldn't require a timeout.
		** However, when debugging TRK with TRK, the behavior of the SIGCHLD
		** signal seems to be unreliable.  In such a case, select() might not
		** be interrupted when a child process stops, necessitating the timeout.
		*/

		timeout.tv_sec = 1;
		timeout.tv_usec = 0;
		TrkSelectIo( false, -1, &timeout );
	}
}

/****************************************************************************/
/*
 *	TargetInterrupt
 *
 *	Interrupt/breakpoint/signal occurred.  Event was posted by the Idle()
 *	function based on the result of wait().
 *	Service the event, sending notification to the host debugger
 */
/****************************************************************************/
DSError TargetInterrupt(NubEvent* event)
{
	ui32 processId;
	ui32 threadId;
	ui32 status;
	bool invisibleStop = false;
	PCType lastPc;
	Breakpoint *bp;
	DSError result = kNoError;
	LinuxThreadState *tState;
	LinuxProcessState *pState;
	ObjectId tStateId;
	int notifyCount, mEvent;

	switch (event->fType) {
		case kBreakpointEvent:
		case kExceptionEvent:
			/*
			** Get the satus from the event as the event sub-type.
			** Also get the corresponding process & thread IDs.
			** Use the status to determine the appropriate notification
			** type (or continue stepping without notification).
			*/
			status = event->fInfo[0];
			processId = event->fProcessId;
			threadId = event->fThreadId;

			/*
			** If a signal arrives for an unknown process, assume it is
			** due to a fork() using CLONE_PTRACE.  Attach to the new
			** process.
			*/
				
			if (FindLinuxThreadStateId( processId, threadId, &tStateId, &tState ) !=
				kNoError)
			{
#ifdef __uClinux__
				//set TRACECLONE options in case the new process clones
				ptrace(PTRACE_SETOPTIONS,processId,NULL, PTRACE_O_TRACECLONE);
#endif //__uClinux__
				result = TrkAddChildProcessLinux( processId );
				break;
			}
								
			/*
			** Get the associated process state object as well.
			*/

			pState = TrkGetLinuxProcessState( tState->fProcessStateId );

			if (WIFEXITED( status ))
			{
				/*
				** Thread light-weight process exited normally.
				** Call the thread death handling routine with the associated
				** process death type and code.  We can't get a useful thread
				** return value in this case, so just use NULL.
				*/

				TrkHandleThreadDeathLinux( NULL,
						kDSLinuxNDPExit, WEXITSTATUS( status ),
						processId, threadId );
			}
			else if (WIFSIGNALED( status ))
			{
				/*
				** Thread light-weight process terminated with a signal.
				** Call the thread death handling routine with the associated
				** process death type and code.  We can't get a useful thread
				** return value in this case, so just use NULL.
				*/
				
				TrkHandleThreadDeathLinux( NULL,
						kDSLinuxNDPTerminate, WTERMSIG( status ),
						processId, threadId );
			}
			else
			{
				/*
				** Thread stopped with a signal.
				*/
				ASSERT(WIFSTOPPED(status));
#ifdef __uClinux__				
				if (WIFSTOPPED(status))
				{			
					if (WSTOPSIG(status) == SIGTRAP)
					{
						
						mEvent = (status >> 16) & 0xffff;
						switch (mEvent)
						{
							case PTRACE_EVENT_CLONE:
								
								PostContinueEvent( processId, 0 );
								return result;
								break;
							
							default:
								break;
						}
					}
				}
#endif //__uClinux__			

				tState->fSignal = WSTOPSIG( status );

				/*
				** Breakpoint release is delayed until the process stops.  Release
				** them now (or reduce the delay by 1).
				*/

				ReleaseDelayedBreakpoints( processId, DS_MSG_BREAK_THREAD_ID_ALL );
				
				/*
				** When a traced program execs, the result will be a SIGTRAP.
				** Detect the exec by looking for a change in the exe file path.
				*/
				
				if (tState->fSignal == SIGTRAP)
				{
					char buffer[PATH_MAX];
					size_t s;
					sprintf( buffer, "/proc/%lu/exe", (ui32)tState->fLwProcessId );
					s = readlink( buffer, buffer, PATH_MAX - 1 );
					if (s != -1)
					{
						buffer[s] = '\0';
						if (pState->fExe == NULL ||
							strcmp( pState->fExe, buffer ) != 0)
						{
							__puts( "exec detected: " );
							__puts( pState->fExe );
							__puts( ":" );
							__puts( buffer );
							__puts( "\r\n" );
							result = TrkHandleExecEvent( pState, tState );
							break;
						}
					}
				}

				/*
				** If it is a SIGTRAP or SIGILL signal, the thread is
				** stopped at a breakpoint.  On some platforms (e.g., ix86)
				** the PC at this point may be at the end of the BP
				** instruction rather than before.  Fix the PC before
				** reporting back to the debugger so that it points to
				** the true BP address.
				*/

				if (tState->fSignal == SIGTRAP || tState->fSignal == SIGILL)
				{
					OsTargetAdjustBreakPC( processId, threadId );
				}

				lastPc = OsTargetGetPC( processId, threadId );
				
				__puts( "Stopped with signal " );
				__puthex8( tState->fSignal );
				__puts( "  at " );
				__puthex8( lastPc );
				__puts( "\r\n" );
				
				
				if (tState->fSignal > TRK_SIGNAL_MAX_LINUX ||
					!TRK_MASK_IS_SET( pState->fStopMask, tState->fSignal ))
				{
					__puts( "Auto-continue signal " );
					__puthex2( tState->fSignal );
					__puts( "\r\n" );
					invisibleStop = true;
					PostContinueEvent( processId, threadId );
				}

				/*
				** Compare the PC to the shared-object breakpoint addr.
				** If it matches:
				**   - Process shared object status and send out appropriate
				**     notifications to the host
				**   - Don't do the step-check or stop notification below.
				**     This stop should be invisible to the user, which
				**     means that it should not interfere with the current
				**     stepping operation.  Note: If the user tries to step
				**     through the specific instruction at which this bp is
				**     set, the step behavior will be wrong.  The potential
				**     fixes for this are ugly, and this should be a rare
				**     case, so it has not been fixed.
				*/

				else
				{
#ifndef __NotSupportSo__
					if (pState->fSoBreakpointId != kInvalidBreakpointId)
					{
					
						bp = GetBreakpoint( pState->fSoBreakpointId );
						ASSERT( bp );
						if ((PCType)bp->fStart == lastPc)
						{
							__puts( "Shared object breakpoint hit.\r\n" );							
														
							
							TrkUpdateSoStateLinux( pState, tState, true, true, &notifyCount );
											
							
							/* Delayed process threads attachment hack for RedHat 9.0. See explanation elsewhere; search on this macro */
							#ifdef TRK_DELAY_ATTACHING_TO_PROCESS_THREADS
								if (notifyCount == 0)
									pState->fReadyForDelayedProcThreadsAttach = true;
							#endif

							invisibleStop = true;
							
							
							/*
							** If no notifications were sent, the debugger will
							** not know that the process is stopped.  Automatically
							** continue running.
							*/
							if (notifyCount == 0)
								PostContinueEvent( processId, threadId );
						}
					}
#endif					
					
					/*
					** Check for thread creation event.
					*/
					if (pState->fThreadCreateBreakpointId != kInvalidBreakpointId)
					{
						bp = GetBreakpoint( pState->fThreadCreateBreakpointId );
						ASSERT( bp );
						if ((PCType)bp->fStart == lastPc)
						{
							__puts( "Thread creation breakpoint hit.\r\n" );
							TrkHandleThreadCreateEventLinux( pState );
							invisibleStop = true;
							PostContinueEvent( processId, threadId );
						}
					}
					
					/*
					** Check for thread death event.
					*/
					if (pState->fThreadDeathBreakpointId != kInvalidBreakpointId)
					{
						bp = GetBreakpoint( pState->fThreadDeathBreakpointId );
						ASSERT( bp );
						if ((PCType)bp->fStart == lastPc)
						{
							__puts( "Thread death breakpoint hit.\r\n" );
							TrkHandleThreadDeathEventLinux( pState );
							invisibleStop = true;
							PostContinueEvent( processId, threadId );
						}
					}
				}
				
				if (invisibleStop)
				{
				
					OsTargetSetStopped( true, processId, threadId );

				}
				if (!invisibleStop)
				{
					/*
					** Don't notify the debugger of the individual trace
					** breakpoints that occur during stepping - just the
					** last one.
					**
					** Just check the step status.  If it is active, don't
					** notify the debugger.  If it is not, then either we
					** are not stepping at all or we just finished, so
					** send out the notification.
					*/
			
					if (!TargetCheckStep( pState->fInvisibleStepRangeList,
							 &invisibleStop,
							processId, threadId ))
					{
						/*
						** Notify the debugger that the target process is
						** stopped.  Also, set the target stop status
						** to indicate that it is stopped.
						*/
					
						OsTargetSetStopped( true, processId, threadId );
						DoNotifyStopped( kDSNotifyStopped, processId, threadId );
						
						/*
						** Save the current PC.  The next time we do a NotifyStopped,
						** we send both the current and previous PC.  This saved
						** value is what will be sent as the previous PC.
						*/
					
						tState->fLastPc = lastPc;
						
					}
				}

			}
			
			break;
		
		default:
			ASSERT(0);			/* not a valid exception */
			break;
	}
	return result;
}

/****************************************************************************/
/*
 *	TargetVersions
 *
 *	Fill in version struct for this target.
 */
/****************************************************************************/
DSError TargetVersions( DSVersions *versions )
{
	versions->kernelMajor = DS_KERNEL_MAJOR_VERSION;
	versions->kernelMinor = DS_KERNEL_MINOR_VERSION;
	versions->protocolMajor = DS_PROTOCOL_MAJOR_VERSION;
	versions->protocolMinor = DS_PROTOCOL_MINOR_VERSION;
	
	return kNoError;
}

/****************************************************************************/
/*
 *	TargetSupportMask
 *
 *	Fill in support mask for this target.
 */
/****************************************************************************/
DSError TargetSupportMask( DSSupportMask *mask )
{
	(*mask)[0] = DS_SUPPORT_MASK_00_07;
	(*mask)[1] = DS_SUPPORT_MASK_08_0F;
	(*mask)[2] = DS_SUPPORT_MASK_10_17;
	(*mask)[3] = DS_SUPPORT_MASK_18_1F;
	(*mask)[4] = DS_SUPPORT_MASK_20_27;
	(*mask)[5] = DS_SUPPORT_MASK_28_2F;
	(*mask)[6] = DS_SUPPORT_MASK_30_37;
	(*mask)[7] = DS_SUPPORT_MASK_38_3F;
	(*mask)[8] = DS_SUPPORT_MASK_40_47;
	(*mask)[9] = DS_SUPPORT_MASK_48_4F;
	(*mask)[10] = DS_SUPPORT_MASK_50_57;
	(*mask)[11] = DS_SUPPORT_MASK_58_5F;
	(*mask)[12] = DS_SUPPORT_MASK_60_67;
	(*mask)[13] = DS_SUPPORT_MASK_68_6F;
	(*mask)[14] = DS_SUPPORT_MASK_70_77;
	(*mask)[15] = DS_SUPPORT_MASK_78_7F;
	(*mask)[16] = DS_SUPPORT_MASK_80_87;
	(*mask)[17] = DS_SUPPORT_MASK_88_8F;
	(*mask)[18] = DS_SUPPORT_MASK_90_97;
	(*mask)[19] = DS_SUPPORT_MASK_98_9F;
	(*mask)[20] = DS_SUPPORT_MASK_A0_A7;
	(*mask)[21] = DS_SUPPORT_MASK_A8_AF;
	(*mask)[22] = DS_SUPPORT_MASK_B0_B7;
	(*mask)[23] = DS_SUPPORT_MASK_B8_BF;
	(*mask)[24] = DS_SUPPORT_MASK_C0_C7;
	(*mask)[25] = DS_SUPPORT_MASK_C8_CF;
	(*mask)[26] = DS_SUPPORT_MASK_D0_D7;
	(*mask)[27] = DS_SUPPORT_MASK_D8_DF;
	(*mask)[28] = DS_SUPPORT_MASK_E0_E7;
	(*mask)[29] = DS_SUPPORT_MASK_E8_EF;
	(*mask)[30] = DS_SUPPORT_MASK_F0_F7;
	(*mask)[31] = DS_SUPPORT_MASK_F8_FF;

	return kNoError;
}

/****************************************************************************/
/*
 *	TargetFlushCache
 *
 *	Flushes the cache corresponding to the specified memory range using 
 *	the given options.
 *
 */
/****************************************************************************/
DSError TargetFlushCache( ui8 options,
						  void *start, void *end )
{
	return( kNoError );
}

/****************************************************************************/
/*
 *	TargetCPUType
 *
 *	Fill in cpu type struct for this target.
 */
/****************************************************************************/
DSError TargetCPUType( DSCPUType *cpuType )
{
	cpuType->cpuMajor = DS_CPU_MAJOR_TYPE;
	
#ifdef PPC
	if ( (GetVersionRegister() & 0xFFFF0000) == E300C2 )
	  cpuType->cpuMinor = DS_CPU_PPC_E300C2;
	else if ( (GetVersionRegister() & 0xFFF80000) == E300 )
	  cpuType->cpuMinor = DS_CPU_PPC_E300;
	else if ( (GetVersionRegister() & 0xFFFE0000) == E500 )
	  cpuType->cpuMinor = DS_CPU_PPC_E500;
	else if ( (GetVersionRegister() & 0xFFFF0000) == E500MC )
	  cpuType->cpuMinor = DS_CPU_PPC_E500MC;
	else if ( (GetVersionRegister() & 0xFFFF0000) == E600 )
	  cpuType->cpuMinor = DS_CPU_PPC_E600;
	else
#endif
	  cpuType->cpuMinor = DS_CPU_MINOR_TYPE;
	
	cpuType->bigEndian = gBigEndian;
	cpuType->defaultTypeSize = DS_CPU_DEFAULT_SIZE;
	cpuType->fpTypeSize = DS_CPU_FP_SIZE;
	cpuType->extended1TypeSize = DS_CPU_EXTENDED1_SIZE;
	cpuType->extended2TypeSize = DS_CPU_EXTENDED2_SIZE;
	
	return kNoError;
}

/****************************************************************************/
/*
 *	OsTargetSetupTraceNoBreakpoints
 *
 *	Sets up a single instruction trace without using breakpoints by setting
 *	up singlestep mode.
 */
/****************************************************************************/
DSError OsTargetSetupTraceNoBreakpoints( StepStatus *status,
										ui32 processId, ui32 threadId )
{
	DSError err = kNoError;
	LinuxThreadState *tState;
	
	/*
	** Reject if the specified process is not in the list of controlled processes.
	*/
	if (FindLinuxThreadStateId( processId, threadId, NULL, &tState ) != kNoError)
		return( kInvalidThreadId );

	/*
	** Don't check the step type.  This function is only called when
	** single step mode is desired.
	*/

	tState->fContinueType = PTRACE_SINGLESTEP;
	
	return( err );
}

/****************************************************************************/
/*
 *	OsTargetCleanupTraceNoBreakpoints
 *
 *	Cleans up a single instruction trace by removing breakpoint instructions
 *	and replacing them with the original instructions.
 */
/****************************************************************************/
DSError OsTargetCleanupTraceNoBreakpoints( StepStatus *status,
											ui32 processId, ui32 threadId )
{
	DSError err = kNoError;
	LinuxThreadState *tState;
	
	/*
	** Reject if the specified process is not in the list of controlled processes.
	*/
	if (FindLinuxThreadStateId( processId, threadId, NULL, &tState ) != kNoError)
		return( kInvalidThreadId );

	tState->fContinueType = PTRACE_CONT;

	return( err );
}

/****************************************************************************/
/*
 *	OsAtTraceBreakPoint
 *
 *	Determines whether the target program is currently stopped at a
 *	trace breakpoint.
 */
/****************************************************************************/
bool OsAtTraceBreakPoint( StepStatus *status, ui32 processId, ui32 threadId )
{
	bool result = false;
	int i;
	Breakpoint *bp;
	LinuxThreadState *tState;
	PCType pc;
	
	/*
	** Reject if the specified process is not in the list of controlled processes.
	*/
	if (FindLinuxThreadStateId( processId, threadId, NULL, &tState ) != kNoError)
		return( kInvalidThreadId );
	
	/*
	** Compare the PC against any temporary trace breakpoints.
	*/
	
	pc = OsTargetGetPC( processId, threadId );
	
	for (i = 0; i < kTraceBreakpointCount && !result; i++)
	{
		if (status->traceBreakpoints[ i ] != kInvalidBreakpointId)
		{
			bp = GetBreakpoint( status->traceBreakpoints[ i ] );
			ASSERT( bp != NULL );
			if (bp->fStart == pc)
				result = true;
		}
	}

	/*
	** If no match, but we're using single-step mode, check to
	** see if the signal type is consistent w/single-step.
	*/
#ifndef RedStripe
	if (!result && tState->fContinueType == PTRACE_SINGLESTEP 
		&& tState->fSignal == SIGTRAP)
#else
	if (!result && tState->fContinueType == PTRACE_SINGLESTEP )
#endif
		result = true;

		
	
	return(result);
}

/****************************************************************************/
/*
 *	PurgeProcess
 *
 *	Purges all internally stored state for the given process.  Do not
 *	attempt to alter the process itself.  This function may be called
 *	after the process has died or after unattaching, so fiddling with
 *	the process at this point could cause problems.
 */
/****************************************************************************/
DSError PurgeProcess( ui32 processId )
{
	LinuxProcessState *pState;
	ObjectId i;
	int x;
	
	/*
	** Release all breakpoints for the process.
	**
	** Do not call ClearThreadBreakpoints() here.  That function will
	** try to un-set each breakpoint, which is no good if the process
	** has already died.
	*/
	ReleaseThreadBreakpoints( processId, DS_MSG_BREAK_THREAD_ID_ALL, false, false );

	/*
	** If the specified process is not in the list of controlled processes,
	** there is nothing left to do.
	*/
	if (FindLinuxProcessStateId( processId, &i, &pState ) != kNoError)
		return( kInvalidProcessId );

	/*
	** Purge all threads associated with the process.
	*/
	TrkPurgeProcessThreadsLinux( pState );

	/*
	** Clear out everything in the process state object.
	*/
	
#ifndef __NotSupportSo__
	TrkFreeSoListLinux( pState->fSoList );
#endif	
	
	TrkFreeInvisibleRangeList(&pState->fInvisibleStepRangeList);
	


	for (x = 0; x < kConsoleCount; x++)
	{
		if (pState->fConsoleFds[ x ] != -1)
			close( pState->fConsoleFds[ x ] );
	}

	/*
	** Free the exec file name.
	*/
	if (pState->fExe != NULL)
		free( pState->fExe );
		
	/*
	** Release the process state object.
	*/

	ReleaseArrayObject( &gProcesses, i );

	return( kNoError );
}

DSError PurgeProcessesOnly( ui32 processId )
{
	LinuxProcessState *pState;
	ObjectId i;
	int x;
	
	/*
	** Release all breakpoints for the process.
	**
	** Do not call ClearThreadBreakpoints() here.  That function will
	** try to un-set each breakpoint, which is no good if the process
	** has already died.
	*/
	ReleaseThreadBreakpoints( processId, DS_MSG_BREAK_THREAD_ID_ALL, false, false );

	/*
	** If the specified process is not in the list of controlled processes,
	** there is nothing left to do.
	*/
	if (FindLinuxProcessStateId( processId, &i, &pState ) != kNoError)
		return( kInvalidProcessId );

	/*
	** Clear out everything in the process state object.
	*/
	
	TrkFreeSoListLinux( pState->fSoList );
	
	TrkFreeInvisibleRangeList(&pState->fInvisibleStepRangeList);
	


	for (x = 0; x < kConsoleCount; x++)
	{
		if (pState->fConsoleFds[ x ] != -1)
			close( pState->fConsoleFds[ x ] );
	}

	/*
	** Free the exec file name.
	*/
	if (pState->fExe != NULL)
		free( pState->fExe );
		
	/*
	** Release the process state object.
	*/

	ReleaseArrayObject( &gProcesses, i );

	return( kNoError );
}


/****************************************************************************/
/*
 *	OsTargetGetStepStatii
 *
 *	Gets the step status objects for the specified process & thread.
 */
/****************************************************************************/
StepStatii *OsTargetGetStepStatii( ui32 processId, ui32 threadId )
{
	LinuxThreadState *tState;
	
	/*
	** Reject if the specified process is not in the list of controlled processes.
	*/
	if (FindLinuxThreadStateId( processId, threadId, NULL, &tState ) != kNoError)
		return( NULL );

	return( &(tState->fStepStatii) );
}

/****************************************************************************/
/*
 *	OsTargetAddProcess
 *
 *	Add a new process to the list of debugged processes.
 */
/****************************************************************************/
DSError OsTargetAddProcess( ui32 processId, int consoleFds[kConsoleCount],
							ui32 consoleParentId,
							LinuxProcessState **resultPstate, 
							LinuxThreadState **resultTstate )
{
	DSError err = kNoError;
	LinuxProcessState *pState;
	ObjectId i;
	ObjectId tStateId;
	LinuxThreadState *tState;
	ui32 threadId;
	int x;
	char buffer[PATH_MAX];

	/*
	** Get a free process state object and assign it the process ID
	** and console file descriptors.
	*/
	
	err = GetFreeArrayObject( &gProcesses, &i, (ObjectHeader **)&pState );
	if (err == kNoError)
	{
		pState->fProcessId = processId;
		for (x = 0; x < kConsoleCount; x++)
			pState->fConsoleFds[ x ] = consoleFds[ x ];
		pState->fConsoleParentId = consoleParentId;
		pState->fBlockStdin = true;
		
		/*
		** Get the path to the executable file from the exe link in /proc.
		*/
		
		sprintf( buffer, "/proc/%lu/exe", processId );
		x = readlink( buffer, buffer, PATH_MAX - 1 );
		if (x != -1)
		{
			buffer[x] = '\0';
			pState->fExe = strdup( buffer );
		}
	}
		
	/*
	** Add a default thread thread to the process.  TRK will assign an
	** internal thread ID to it.  Initially there is no debug thread ID
	** (the thread ID we get from the thread debug library).  This will
	** be initialized, if possible, by TrkUpdateProcessThreadsLinux() 
	** below or some time in the future.  The lwpid for the default thread
	** is the same as the main process ID.
	*/

	if (err == kNoError)
		err = TrkAddThreadLinux( i, 0, processId, &threadId, &tStateId );

	if (err == kNoError)
	{
		if (resultPstate != NULL)
			*resultPstate = pState;
		if (resultTstate != NULL)
		{
			tState = TrkGetLinuxThreadState( tStateId );
			ASSERT( tState );
			*resultTstate = tState;
		}
	}

	return( err );	
}

/****************************************************************************/
/*
 *	TrkSetupDebugProcess
 *
 *	Set up a process for debugging.  Attach to the process before calling
 *	this function.
 */
/****************************************************************************/
DSError TrkSetupDebugProcess( LinuxProcessState *pState, LinuxThreadState *tState )
{
	DSError err = kNoError;
	
	/*
	** Set breakpoints needed to support shared-object debugging.  Then 
	** get additional thread status, if any.
	*/
	
	if (err == kNoError)
	{

#ifndef __NotSupportSo__
		TrkSetSoBreakpointsLinux( pState, tState );
#endif
#ifndef __NotSupportThread__
		TrkInitializeProcessThreadsLinux( pState );
#endif
	}
	
	return( err );
}

/****************************************************************************/
/*
 *	OsTargetKillProcessLinux
 *
 *	"Takes care" of the specified process.  This is a high-level function 
 *	that kills all of the associated threads and purges any internally-
 *	stored state.
 */
/****************************************************************************/
DSError OsTargetKillProcessLinux( ui32 processId )
{
	DSError err = kNoError;

	/*
	** Kill all of the threads in the process, then purge its state.
	*/

	err = TrkKillProcessThreadsLinux( processId );

//!!!should happen automatically when the death signal arrives	PurgeProcess( processId );
	
	return( err );
}

/****************************************************************************/
/*
 *	OsTargetKillThreadLinux
 *
 *	"Takes care" of the specified thread.  This is a high-level function
 *	that determines the appropriate method with which to kill the thread
 *	and purges any internally stored state associated with it.
 */
/****************************************************************************/
DSError OsTargetKillThreadLinux( ui32 processId, ui32 threadId )
{
	DSError err = kNoError;
	LinuxThreadState *tState;
	LinuxProcessState *pState;
	
	/*
	** If the process has an associated thread agent, dispatch to a
	** thread-aware implementation.  Otherwise, dispatch to a
	** process-only implementation.
	*/

	if (FindLinuxThreadStateId( processId, threadId, NULL, &tState ) != kNoError)
		err = kInvalidThreadId;

	if (err == kNoError)
	{
		pState = TrkGetLinuxProcessState( tState->fProcessStateId );
		ASSERT( pState );
	
#ifndef __NotSupportThread__
		if (pState->fThreadAgent == NULL)
		{
			err = TrkKillProcessLinux( pState->fProcessId );
		}
		else
		{
			err = TrkKillThreadLinux( tState );
		}
#else
		err = TrkKillProcessLinux( pState->fProcessId );
#endif

	}

	return( err );
}

/****************************************************************************/
/*
 *	TrkKillProcessLinux
 *
 *	"Takes care" of the specified process.  This is a low-level function 
 *	that simply kills the Linux process with the given ID.
 *
 *	NOTE:This is a low-level implementation that can be used
 *	to stop the light-weight process for a thread.  This means that
 *	processId may NOT be the main process ID for the given thread.  This
 *	is important, because many of the TRK functions which accept a
 *	processId will ONLY accept the main process ID.  Avoid calling other
 *	TRK functions with the given process ID.
 */
/****************************************************************************/
DSError TrkKillProcessLinux( ui32 processId )
{
	DSError err = kNoError;
	
	/*
	** Don't wait but monitor for the process termination status because we
	** don't want a blocking call here.  That will be handled by the main loop.
	** Note: On some systems, PTRACE_KILL doesn't work when the target
	** process is running.  Raising a SIGKILL signal beforehand takes
	** care of that case and can't be blocked.
	*/
	__puts("TrkKillProcessLinux: Current (parent) process PID = 0x");
	__puthex4(getpid());
	__puts(".\n\r");
	
	/*
	** If the child process does not exists anymore then return
	** from this function.
	*/

	/*
	** Atempt to kill the child process by sending the SIGKILL signal.
	*/
	
	__puts("TrkKillProcessLinux: Kill the child process PID = 0x");
	__puthex4(processId);
	__puts(".\n\r");
	kill( processId, SIGKILL );

	
	/*
	** If the child process is still alive then try to kill it using the
	** ptrace( PTRACE_KILL,...) syscall.
	*/

     __puts("TrkKillProcessLinux: The child PID = 0x");
     __puthex4(processId);
     __puts(" is still alive. Trying to kill it using ptrace(PTRACE_KILL, processId).\n\r");
	if ( ptrace( PTRACE_KILL, processId, 0, 0 ) != 0 )
		err = kOsError;
	
	return( err );
}

/****************************************************************************/
/*
 *	InitializeLinuxProcessState
 *
 *	Initialize one of the LinuxProcessState objects.  This is called by
 *	the general-purpose object array routines, so the passed in type 
 *	assumes the header only.  Note that the header itself should NOT be
 *	initialized by this routine, only the additional object-specific fields.
 */
/****************************************************************************/
static DSError InitializeLinuxProcessState( ObjectHeader *ob )
{
	LinuxProcessState *pState = (LinuxProcessState *)ob;
	
	ASSERT( pState );
	
	/*dynamic list for invisible step ranges as any number of shared objects can 
	be associated with an application.*/
	pState->fInvisibleStepRangeList = NULL;
	pState->fInvisibleStepRangeCount = 0;
	
	pState->fSoBreakpointId = kInvalidBreakpointId;
	pState->fSoDebugPtr = NULL;
	pState->fInvisibleStepRangeCount = 0;
	pState->fSoList = NULL;
#ifndef __NotSupportThread__
	pState->fThreadAgent = NULL;
#endif
	pState->fThreadCreateBreakpointId = kInvalidBreakpointId;
	pState->fThreadDeathBreakpointId = kInvalidBreakpointId;
	pState->fThreadSignalBreakpointId = kInvalidBreakpointId;
	pState->fNextThreadId = 0;
	pState->fPrimaryThreadDead = false;
#ifndef __NotSupportThread__
	pState->fAllowThreadDb = true;
#else
	pState->fAllowThreadDb = false;
#endif	
	pState->fDlWarned = false;
	pState->fThreadLibWarned = false;
	pState->fExe = NULL;
	
	memcpy( &pState->fStopMask, &gDefaultStopMask,
			sizeof( TrkSignalMaskLinux ) );
	memcpy( &pState->fPassMask, &gDefaultPassMask,
			sizeof( TrkSignalMaskLinux ) );

	return kNoError;
}

/****************************************************************************/
/*
 *	MatchLinuxProcessStateId
 *
 *	Compare two process state objects to determine if they have the same
 *	process Ids.
 */
/****************************************************************************/
static bool MatchLinuxProcessStateId( ObjectHeader *ob1, ObjectHeader *ob2 )
{
	return( ((LinuxProcessState *)ob1)->fProcessId ==
			((LinuxProcessState *)ob2)->fProcessId );
}

/****************************************************************************/
/*
 *	FindLinuxProcessStateId
 *
 *	Find the process state structure corresponding to the specified process ID.
 *	The resultObId parameter can be NULL.
 */
/****************************************************************************/
DSError FindLinuxProcessStateId( ui32 processId,
										ObjectId *resultObId,
										LinuxProcessState **result )
{
	DSError err;
	ObjectId i;
	
	LinuxProcessState key;
	
	key.fProcessId = processId;
	err = FindArrayObject( &gProcesses, &i, (ObjectHeader **)result, 
							(ObjectHeader *)&key, &MatchLinuxProcessStateId );
	if (resultObId != NULL)
		*resultObId = i;

	return( err );
}

/****************************************************************************/
/*
 *	TrkGetLinuxProcessState
 *
 *	Get the process state object associated with the specified process state
 *	ID.
 */
/****************************************************************************/
LinuxProcessState *TrkGetLinuxProcessState( ObjectId i )
{
	return( (LinuxProcessState *)GetArrayObject( &gProcesses, i ) );
}

/****************************************************************************/
/*
 *	TrkGetLinuxProcessStateId
 *
 *	Get the process state object ID associated with the specified process
 *	state.
 */
/****************************************************************************/
ObjectId TrkGetLinuxProcessStateId( LinuxProcessState *pState )
{
	return( GetArrayObjectId( &gProcesses, (ObjectHeader *)pState ) );
}

/****************************************************************************/
/*
	TrkHandleProcessDeathLinux
	
	When a process dies, update internal structures and send a notification
	back to the host.
	
	This function may be called for thread death due to a normal
	exit, termination by signal, or death of the last associated
	thread.

*/
/****************************************************************************/
DSError TrkHandleProcessDeathLinux( ui8 type, ui32 code,
								ui32 processId )
{
	DSError err = kNoError;
	struct timeval timeout;
	
	/*
	** Handle any pending console I/O so that it doesn't get lost.  After
	** PurgeProcess(), we won't be checking for it anymore.
	*/
	
	timeout.tv_sec = 0;
	timeout.tv_usec = 0;
	while (TrkSelectIo( true, processId, &timeout ) > 0) {;}
	
	/*
	** Purge all internal info on the process and send a 
	** NotifyItemDeleted:ProcessItem msg with the exit
	** type status.
	*/

	PurgeProcess( processId );
	err = DoOsNotifyProcessDeleted( type, code, processId );
	
	return( err );
}

/****************************************************************************/
/*
	TargetWriteFile
	
	Write data to a file.  The length parameter is both an input and output
	parameter.  On return it should reflect the actual amount of data written
	to the file.  The return value should be one of the DSIOResult enumerated
	values.

*/
/****************************************************************************/
DSIOResult TargetWriteFile( ui32 handle, ui8 *data, size_t *length )
{
	DSIOResult result = kDSIONoError;
	ssize_t fresult;
	
	fresult = write( handle, data, *length );

	if (fresult >= 0)
	{
		*length = fresult;
	}
	else
	{
		*length = 0;
		result = kDSIOError;
	}

	return( result );
}

/****************************************************************************/
/*
	TargetReadFile
	
	Read data from a file.  The length parameter is both an input and output
	parameter.  On return it should reflect the actual amount of data read
	from the file.  The return value should be one of the DSIOResult enumerated
	values.

*/
/****************************************************************************/
DSIOResult TargetReadFile( ui32 handle, ui8 *data, size_t *length )
{
	DSIOResult result = kDSIONoError;
	ssize_t fresult;
	
	fresult = read( handle, data, *length );

	if (fresult > 0)
	{
		*length = fresult;
	}
	else
	{
		*length = 0;
		result = (fresult == 0) ? kDSIOEOF : kDSIOError;
	}

	return( result );
}

/****************************************************************************/
/*
	TargetOpenFile
	
	Open a file.  The handle parameter is an output parameter.  The return 
	value should be one of the DSIOResult enumerated values.

*/
/****************************************************************************/
DSIOResult TargetOpenFile( DSFileOpenModes modes, unsigned char *name,
		ui32 *handle, ui32 *timestamp )
{
	DSIOResult result = kDSIONoError;
	int fflags = 0;
	mode_t fmode;
	struct stat fst;
	
	*handle = 0;
	*timestamp = 0;
	
	/*
	** The fflags must be based on one of these 3 values.
	*/
	
	if ((modes & kDSFileOpenRead) && (modes & kDSFileOpenWrite))
		fflags = O_RDWR;
	else if (modes & kDSFileOpenRead)
		fflags = O_RDONLY;
	else if (modes & kDSFileOpenWrite)
		fflags = O_WRONLY;
	else
		ASSERT(0);

	/*
	** If opening in write mode and not appending, truncate the file.
	*/
	
	if (!(modes & kDSFileOpenAppend) && (modes & kDSFileOpenWrite))
		fflags |= O_TRUNC;

	/*
	** The create mode maps directly to one of the OS flags.
	*/

	if (modes & kDSFileOpenCreate)
		fflags |= O_CREAT;

	/*
	** Don't allow blocking IO.
	*/
	
	fflags |= O_NONBLOCK;
	
	/*
	** The fmode sets the permissions for new files.  If the exec mode 
	** is specified, give us rwx permission.  Otherwise, just rw.
	*/
	
	if (modes & kDSFileOpenExec)
	{
		fmode = S_IRWXU | S_IRWXG | S_IRWXO;
	}
	else
	{
		fmode = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH;
	}

	/*
	** Open the file.
	*/
	
	__puts( "Opening file " );
	__puts( name );
	__puts( "\r\n" );
	*handle = open( name, fflags, fmode );

	/*
	** Set the initial file position to the end of the file for append
	** mode, and the start of the file in all other cases.
	*/
	
	if (*handle != -1)
	{
		if (modes & kDSFileOpenAppend)
			lseek( *handle, 0, SEEK_END );
		else
			lseek( *handle, 0, SEEK_SET );

		/*
		** Get the timestamp and file permissions.  If the
		** caller requested exec permission and the file doesn't
		** have it, set it.
		*/
		
		if (fstat( *handle, &fst ) == 0)
		{
			ASSERT( sizeof( *timestamp ) >= sizeof( fst.st_mtime ) );
			*timestamp = fst.st_mtime;
			__puts( "File time: " );
			__puts( ctime( &fst.st_mtime ) );
			__puts( "\r\n" );

			if ((modes & kDSFileOpenExec) && !(fst.st_mode & S_IXUSR))
			{
				fchmod( *handle, fst.st_mode | S_IXUSR | S_IXGRP | S_IXOTH );
			}
		}
	}
	else
	{
		result = kDSIOError;
	}

	return( result );
}

/****************************************************************************/
/*
	TargetCloseFile
	
	Close a file. The return value should be one of the DSIOResult enumerated
	values.

*/
/****************************************************************************/
DSIOResult TargetCloseFile( ui32 handle, ui32 timestamp )
{
	DSIOResult result = kDSIONoError;
	int flags;
	struct utimbuf ftime;
	#define TCF_BUFSIZE 4096
	char buf[ TCF_BUFSIZE ]; /* hopefully enough for long file names */
	
	/*
	** Set the file's timestamp, but only if we have write access.
	*/

	flags = fcntl( handle, F_GETFL );
	
	if ((flags & O_ACCMODE) != O_RDONLY)
	{
		ftime.actime = timestamp;
		ftime.modtime = timestamp;

		__puts( "File time: " );
		__puts( ctime( &timestamp ) );
		__puts( "\r\n" );
		
		/*
		** I can't find a way to do this without getting the path to the
		** file.  There is a link to it in /dev/fd/??? where ??? is the
		** value of the handle.
		*/

		sprintf( buf, "/proc/self/fd/%d", (int)handle );
		
		utime( buf, &ftime );
	}

	/*
	** Close the file.
	*/
	
	if (close( handle ) != 0)
		result = kDSIOError;
	
	return( result );
}

/****************************************************************************/
/*
	TargetPositionFile
	
	Set a file's position.  The length parameter is both an input and output
	parameter.  On return it should reflect the actual amount of data written
	to the file.  The return value should be one of the DSIOResult enumerated
	values.

*/
/****************************************************************************/
DSIOResult TargetPositionFile( DSFilePositionModes mode, ui32 handle, ui32 offset )
{
	DSIOResult result = kDSIONoError;
	int fwhence;
	
	/*
	** Convert mode to native parameter and set the position accordingly.
	*/

	switch( mode )
	{
		default:
		case kDSFileSeekSet: fwhence = SEEK_SET; break;
		case kDSFileSeekCur: fwhence = SEEK_CUR; break;
		case kDSFileSeekEnd: fwhence = SEEK_END; break;
	}
	
	if (lseek( handle, offset, fwhence ) == -1)
		result = kDSIOError;

	return( result );
}

/****************************************************************************/
/*
	TrkFreeOsProcessList
	
	Free the given list.

*/
/****************************************************************************/
static void TrkFreeOsProcessList( OsProcessItem *list )
{
	OsProcessItem *curr;
	OsProcessItem *next;
	
	for (curr = list; curr != NULL; curr = next)
	{
		next = curr->next;
		if (curr->name != NULL)
			free( curr->name );
		free( curr );
	}
}

/****************************************************************************/
/*
	TrkBuildOsProcessList
	
	Build a new process list based on the current system state.

*/
/****************************************************************************/
static int SelectProcessDirent( const struct dirent *d )
{
	/*
	** Select all of the directory entries which consist entirely
	** of numeric characters.
	*/
	int length = strlen( d->d_name );
	
	return( strspn( d->d_name, ".0123456789" ) == length && strspn( d->d_name, "." ) != length );
}

static int CompareProcessDirent( const void *a, const void *b )
{
	/*
	** Convert the directory entry names to numbers and compare those.
	*/
	const char * a_dirname = (*((const struct dirent **)a))->d_name;
	const char * b_dirname = (*((const struct dirent **)b))->d_name;

	int va, vb;

	/*
	 * Starting with Linux Kernel 2.14.20 (RedHat 9.0), all the lightweight 
	 * processes of a thread group except the main one are given a hidden
	 * directory in /proc
	 */	
	if (*a_dirname == '.') a_dirname++;
	if (*b_dirname == '.') b_dirname++;
	
	va = atol( a_dirname );
	vb = atol( b_dirname );
	if (va < vb)
		return( -1 );
	else if (va == vb)
		return( 0 );
	return( 1 );
}


#ifdef __uClinux__
/* 
 * ColdFire Linux does not seem to fully support process groups. All applications
 * that are launched from a shell end up belonging to the process group of the shell
 * rather than new dedicated process groups. That makes breaking down lightweight
 * processes into POSIX process/thread categories impossible. We must report
 * all lightweight processes as main processes. This is consistent with how
 * ps -e is working on the ColdFire uClinux implementation; that is, it too
 * is unable to differentiate between POSIX processes and threads so it lists
 * everything as a process. We make an exception for targeted applications; we can
 * determine the lightweight processes that belong to it, and hence report them
 * as threads.
 */
static OsProcessItem *TrkBuildOsProcessList( int *listCount )
{
	OsProcessItem * pList = NULL;
	int i = 0;
	struct dirent **dList = NULL;
	int lwpidCount = 0;
	*listCount = 0;
				
	/*
	 * Get all of the process entries from the /proc dir, sorted
	 * in numeric order by process ID.
	 */
	lwpidCount = scandir("/proc", &dList, &SelectProcessDirent, &CompareProcessDirent);
	
	if (!lwpidCount)
	{
		ASSERT(0);	// something is seriously wrong
		return NULL;
	}
	
	/* Iterate over the process entries from /proc	*/
	for (i = 0; i < lwpidCount; i++)
	{
		ui32 pid = 0;
		
		struct dirent * entry = dList[i];
		ASSERT(entry);

		/*
		 * Starting with Linux Kernel 2.14.20 (RedHat 9.0), all the lightweight 
		 * processes of a thread group except the main one are given a hidden
		 * directory in /proc
		 */	
		pid = atol((entry->d_name[0] == '.') ? &entry->d_name[1] : entry->d_name);

		/*
		 * pid 0 represents the root of the process tree, not a real process; pid 1 
		 * is the 'init' process, and is also ignored
		 */
		if (pid > 1)
		{
			DSError err = 0;
			OsProcessItem * newItem;
			char processName[512];
			ui8 processState;
			LinuxThreadState *tState = NULL;
			LinuxProcessState *pState = NULL;			

			/*
			 * Exclude processes which are associated with a group of threads
			 * in our controlled thread list but are not the main process for 
			 * that group.
			 */
			if (FindLinuxThreadStateLwId(pid, NULL, &tState) == kNoError)
			{
				pState = TrkGetLinuxProcessState(tState->fProcessStateId);
				ASSERT(pState);
				if (pState->fProcessId != pid)
					continue;	// skip this thread
			}

			processName[0] = '\0';
			err = GetProcessInfo(pid, processName, &processState, NULL, NULL, NULL, NULL);
			if (err == kNoError)
			{
				/* Add a new item to the head of the list. */
				newItem = (OsProcessItem *)malloc(sizeof(OsProcessItem));
				if (newItem)
				{
					(*listCount)++;

					/* Insert the new item at the beginning of the list */				
					newItem->next = pList;
					pList = newItem;

					pList->name = strdup(processName);
					pList->id = pid;
					pList->state = processState;
										
					/* in uClinux, every lightweight process is considered its own top level process */
					pList->multiThreaded = 0;

					#if DEBUG_PROCESS_LIST
						__puts( "   i:" );
						__puthex8( pList->id );
						__puts( " s:" );
						__putchar( pList->state );
						__puts( " m:" );
						__puthex2( pList->multiThreaded );
						__puts( " n:" );
						__puts( pList->name );
						__puts( "\r\n" );
					#endif
				}
			}
		}
	}
	
	return pList;
}
#else
static OsProcessItem *TrkBuildOsProcessList( int *listCount )
{
	LinuxProcessState* pState = NULL;
	LinuxThreadState* tState = NULL;
	
	OsProcessItem* pList = NULL;
	OsProcessItem* pItem = NULL;
	
	int lwpidCount;
	int index;
	int taskMode;
	
	struct dirent** dEntry = NULL;
	
	/*
	 * Whenever we create a new process list we free the old list of lwpids.
	 */
	if (lwpidList)
		free(lwpidList);
	
	/*
	 * The new kernels keep the process threads info in a task directory (/proc/<pid>/task)
	 */
	taskMode = scandir( "/proc/1/task", &dEntry, &SelectProcessDirent, &CompareProcessDirent);
	if (taskMode > 0)
	{
		while(taskMode--)
			free(dEntry[taskMode]);
		free(dEntry);	
		lwpidList = TRKBuildOsLwpidListTaskMode(&lwpidCount);
	}
	else	
		lwpidList = TRKBuildOsLwpidList(&lwpidCount);
	
	g_lwpidCount = lwpidCount;
	if (!lwpidList)
			  return (NULL);

	if (TRKBuildOSLwpidRelationships(lwpidList, lwpidCount))
	{
		free(lwpidList);
		return (NULL);
	}
	
	#if DEBUG_PROCESS_LIST
		__puts( "Building process list:\r\n" );
	#endif

	for (*listCount = 0, index = 0; index < lwpidCount; index++)
	{
		/*
		 * Do not include the pid=0 process.
		 */
		 if (lwpidList[index].pid == 0)
		 	continue;
		
		/* 
		 * We look first in the controlled list of processes.
		 */
		
		if (FindLinuxThreadStateLwId(lwpidList[index].pid, NULL, &tState) == kNoError)
		{
			pState = TrkGetLinuxProcessState(tState->fProcessStateId);
			
			if (pState->fProcessId == lwpidList[index].pid)
			{
			
				pItem = (OsProcessItem*)malloc(sizeof(OsProcessItem));
				if (!pItem)
				{
					free(lwpidList);
					return (NULL);
				}
				memset(pItem, 0, sizeof(OsProcessItem));
				
				pItem->id = lwpidList[index].pid;
				pItem->state = lwpidList[index].state;
				pItem->name = lwpidList[index].name;
				pItem->multiThreaded = lwpidList[index].threads ? true : false;

				pItem->next = pList;
				pList = pItem;
				(*listCount)++;
				
								
				#if DEBUG_PROCESS_LIST
					__puts( "   id:" );
					__puthex8( pList->id );
					__puts( "	state:" );
					__putchar( pList->state );
					__puts( "	multithread: " );
					__puthex2( pList->multiThreaded );
					__puts( "	name:" );
					__puts( pList->name );
					__puts( "	controled" );
					__puts( "\r\n" );
				#endif
				
				continue;
			}
		}
		  
		/*
		 * Do not include the lwpids marked as subthreads.
		 */
		if (lwpidList[index].isSubthread)
				  continue;

		pItem = (OsProcessItem*)malloc(sizeof(OsProcessItem));
		if (!pItem)
		{
			free(lwpidList);
			return (NULL);
		}
		memset(pItem, 0, sizeof(OsProcessItem));

		/*
		 * Add the lwpid as process in the processes list.
		 */
		pItem->id = lwpidList[index].pid;
		pItem->state = lwpidList[index].state;
		pItem->name = lwpidList[index].name;
		pItem->multiThreaded = lwpidList[index].threads ? true : false;

		pItem->next = pList;
		pList = pItem;
		(*listCount)++;

		#if DEBUG_PROCESS_LIST
			__puts( "   id:" );
			__puthex8( pList->id );
			__puts( "	state:" );
			__putchar( pList->state );
			__puts( "	multithread: " );
			__puthex2( pList->multiThreaded );
			__puts( "	name:" );
			__puts( pList->name );
			__puts( "	non-controled\n" );
			__puts( "\r\n" );
		#endif

	}
	
	
	return (pList);
}
#endif // __uClinux__

/****************************************************************************/
/*
	TargetAddProcessList
	
	Add process list data to the given buffer, starting at the given
	index in the list.  If the start index is 0, refreshes the list.
	Otherwise, uses previously stored list.

*/
/****************************************************************************/
DSError TargetAddProcessList( MessageBuffer *b, ui8 options, ui32 filter,
							ui32 startIndex,
                            ui32 *returnedCount,
                            ui32 *totalCount )
{
	DSError err = kNoError;
	unsigned int currentIndex;
	OsProcessItem *currentItem;
	size_t nameLen;
	
	static OsProcessItem *list = NULL;
	static unsigned int total = 0;

	/*
	** If the requested start index is 0, then read a fresh process
	** list.  Otherwise, it is an index into the last-read list.
	*/
	
	if (startIndex == 0)
	{
		TrkFreeOsProcessList( list );
		list = TrkBuildOsProcessList( &total );
	}
	
	if (err == kNoError)
	{
		*totalCount = total;	
		*returnedCount = 0;

		if (startIndex < *totalCount)
		{
			/*
			** Find the item corresponding to the requested index.
			*/
			
			for (currentItem = list, currentIndex = 0;
				 currentItem != NULL && currentIndex < startIndex;
				 currentItem = currentItem->next, currentIndex++)
				;
					
			ASSERT( currentItem != NULL );
					
			/*
			** Add info to the buffer until we run out of processes or
			** buffer space.
			*/
			
			while (currentItem != NULL)
			{
				nameLen = strlen( currentItem->name );
				
				if (GetBufferRemaining( b ) <
					(nameLen + TRK_MSG_PL_INFO_MIN_LENGTH_LINUX))
					break;

				err = MessageAdd_ui32( b, currentItem->id );
				if (err == kNoError)
					err = MessageAdd_ui8( b, currentItem->state );
				if (err == kNoError)
					MessageAdd_ui8( b, currentItem->multiThreaded );
				if (err == kNoError)
					MessageAddBlock( b, currentItem->name, nameLen + 1 );

				(*returnedCount)++;
				currentItem = currentItem->next;
			}
		}
	}

	return( err );
}

/****************************************************************************/
/*
	OsTargetNotifyFileInput
	
	The host has notified CodeWarriorTRK that it has new input for one of
	the target program's input files.

*/
/****************************************************************************/
DSError OsTargetNotifyFileInput( ui32 handle, ui32 processId, ui32 threadId )
{
	DSError err = kNoError;
	LinuxProcessState *pState;
	
	/*
	** Assume this is for stdin.  Find the process state for the given
	** process ID and make sure its stdin pipe isn't being blocked.
	*/
	
	if (handle == kDSStdin)
	{
		if (FindLinuxProcessStateId( processId, NULL, &pState ) == kNoError)
			pState->fBlockStdin = false;
		else
			err = kInvalidProcessId;
	}
	else
	{
		err = kParameterError;
	}
	
	return( err );
}

/****************************************************************************/
/*
 *	TrkSelectIo
 *
 *	Uses select() to wait for IO events indicating one of 3 things:
 *		- input arrived from the debugger
 *		- a SIGCHLD signal was caught (input in wakePipe)
 *		- IO is ready on one of the console pipes for one of our
 *		  processes being debugged
 *
 *	The function also manages the console IO when it occurs, sending
 *	appropriate messages to the debugger.
 *
 *	Parameters:
 *		consoleOnly - if true, then do not watch for input from the
 *			debugger or SIGCHLD signals.  This is useful when the
 *			caller only wants to handle pending console IO.
 *		processId - if -1, then wait for &handle console IO for all
 *			processes.  Otherwise only for the specified process.
 *		timeout - Passed to select() as the timeout value.
 *		
 *
 *	The output is essentially the same as the output from select(),
 *	although errno may not be set correctly.
 */
/****************************************************************************/
#define kMaxConsoleIoLen	MIN( DS_MAXREADWRITELENGTH, PIPE_BUF )
static int TrkSelectIo( bool consoleOnly, ui32 processId, struct timeval *timeout )
{
	DSError err = kNoError;
	fd_set readFds;
	fd_set writeFds;
	fd_set exceptFds;
	ObjectId i;
	int maxFd = 0;
	int x;
	char buf[ kMaxConsoleIoLen ];
	int res;
	size_t count;
	DSIOResult ioResult;
	LinuxProcessState *pState;
	int scount = 0;

	FD_ZERO( &readFds );
	FD_ZERO( &writeFds );
	FD_ZERO( &exceptFds );

	if (!consoleOnly)
	{
		/*
		** Need input or exception notification for our remote connection
		** to the debugger.
		*/
		
		FD_SET( gCommInFd, &readFds );
		FD_SET( gCommInFd, &exceptFds );
		FD_SET( gCommOutFd, &exceptFds );
		
		/*
		** Need input notification for the wakePipe.
		*/
		
		FD_SET( wakePipe[ 0 ], &readFds );
		
		maxFd = MAX( gCommInFd, MAX( gCommOutFd, wakePipe[ 0 ] ) );
	}
	
	/*
	** Iterate over all the processes in the list of controlled processes
	** and add the TRK-side of their console pipes.
	*/

	if (!gBlockFileIo || consoleOnly)
	{
		for (i = IterateObjectArrayFirst( &gProcesses, (ObjectHeader **)&pState );
		     i != kInvalidObjectId;
		     i = IterateObjectArrayNext( &gProcesses, i, (ObjectHeader **)&pState ))
		{
			if (processId == -1 || pState->fProcessId == processId)
			{
				for (x = 0; x < kConsoleCount; x++)
				{
					if (pState->fConsoleFds[ x ] != -1)
					{
						if (x == kDSStdin)
						{
							if (!pState->fBlockStdin)
							{
								__puts( "Watching " );
								__puthex8( pState->fConsoleFds[ x ] );
								__puts( " for input-readiness\r\n" );
								FD_SET( pState->fConsoleFds[ x ], &writeFds );
							}
						}
						else
						{
							__puts( "Watching " );
							__puthex8( pState->fConsoleFds[ x ] );
							__puts( " for output\r\n" );
							FD_SET( pState->fConsoleFds[ x ], &readFds );
						}
						maxFd = MAX( maxFd, pState->fConsoleFds[ x ] );
					}
				}
			}
		}
	}
	
	/*
	** Wait for activity.  This should return as soon as TRK receives
	** input from the debugger, a SIGCHLD signal is caught, any
	** one of the target program console pipes needs attention,
	** or it times out.
	*/
	
	scount = select( maxFd + 1, &readFds, &writeFds, &exceptFds, timeout );
	if (scount == -1 && errno != EINTR)
	{
		err = kOsError;
		__perror( "select failed" );
	}

	/*
	** Iterate over all the processes in the list of controlled processes.
	** If one of the console pipes is ready, handle it.
	*/

	for (i = IterateObjectArrayFirst( &gProcesses, (ObjectHeader **)&pState );
	     i != kInvalidObjectId && err == kNoError && scount > 0;
	     i = IterateObjectArrayNext( &gProcesses, i, (ObjectHeader **)&pState ))
	{
		if (processId == -1 || pState->fProcessId == processId)
		{
			for (x = 0; x < kConsoleCount; x++)
			{
				if (pState->fConsoleFds[ x ] != -1 &&
					FD_ISSET( pState->fConsoleFds[ x ],
						(x == kDSStdin) ? &writeFds : &readFds ))
				{
					/*
					** The same console pipe can be shared between stdin & stdout
					** and even between processes after fork(), so it may occur
					** multiple times in this loop.  Clear the ready status the first
					** time it is handled so we don't attempt to handle again.
					*/
					FD_CLR( pState->fConsoleFds[ x ],
						(x == kDSStdin) ? &writeFds : &readFds );

					__puts( "Console triggered " );
					__puthex8( pState->fProcessId );
					__puts( ":" );
					__puthex2( x );
					__puts( "\r\n" );
					count = kMaxConsoleIoLen;
					if (x != kDSStdin)
					{
						res = read( pState->fConsoleFds[ x ], buf, count );
						if (res > 0)
						{
							__puts( "Writing " );
							__puthex8( res );
							__puts( " to console " );
							__puthex8( pState->fProcessId );
							__puts( ":" );
							__puthex2( x );
							__puts( "\r\n" );
							count = res;
							err = SuppAccessFile( x, buf, &count, &ioResult, 
									true, false, pState->fConsoleParentId, 0 );
						}
						else
						{
							/*
							** This seems to happen sometimes after the target
							** process dies (the error is EIO).
							*/
							if (res == -1)
								__perror( "read from console pipe failed" );
							scount--;
						}
					}
					else if (!pState->fBlockStdin)
					{
						__puts( "Reading from console " );
						__puthex8( pState->fProcessId );
						__puts( ":" );
						__puthex2( x );
						__puts( "\r\n" );
						err = SuppAccessFile( x, buf, &count, &ioResult,
								true, true, pState->fConsoleParentId, 0 );
						if (err == kNoError && ioResult == kDSIONoError && count != 0)
						{
							write( pState->fConsoleFds[ x ], buf, count );
							/*
							** If we get less than requested, assume no more available.
							*/
							if (count < kMaxConsoleIoLen)
								pState->fBlockStdin = true;
						}
						else
						{
							pState->fBlockStdin = true;
						}
					}
				}
			}
		}
	}
	
	return( (err == kNoError) ? scount : -1 );
}

/****************************************************************************/
/*
 *	TargetConnect
 *
 *	Target-specific actions performed on connect.
 *
 */
/****************************************************************************/
DSError TargetConnect()
{
	return( kNoError );
}

/****************************************************************************/
/*
 *	TargetDisconnect
 *
 *	Target-specific actions performed on disconnect.
 *
 */
/****************************************************************************/
DSError TargetDisconnect()
{
	return( kNoError );
}

/****************************************************************************/
/*
 *	OsTargetModifyBreakpointThread
 *
 *	Associate the breakpoint with the given thread.
 *
 */
/****************************************************************************/
DSError OsTargetModifyBreakpointThread( Breakpoint *b, ui32 threadId )
{
	b->fThreadId = threadId;
	return( kUnsupportedError );
}

/****************************************************************************/
/*
 *  OsTargetMatchExecBreakpoint
 *
 *  Determines whether there are any target-specific criteria which
 *	would suggest that the given breakpoint overlaps with the given
 *	memory range.
 *	Note: a default implementation can simply return false.
 */
/****************************************************************************/
bool OsTargetMatchExecBreakpoint( Breakpoint *b, unsigned int start,
								unsigned int len,
								ui32 processId, ui32 threadId )
{

#if FOR_MIPS_ARCHITECTURE	
	return OsTargetMatchExecBreakpointForMips( b, start, len, processId, threadId );
#else
	return( false ); 
#endif

}

#if FOR_MIPS_ARCHITECTURE
#else
/****************************************************************************/
/*
 *	TargetValidateBreakpoint
 *
 *	Verify that the breakpoint parameters are OK on the target system.
 *	Note: a default implementation can simply return kNoError
 *
 */
/****************************************************************************/
DSError	TargetValidateBreakpoint( Breakpoint *b )
{
	return( kNoError );
}
#endif

/****************************************************************************/
/*
 *	TrkAttachNewProcessLinux
 *
 *	Attach to a new process which has just been created by fork()/vfork()/
 *	clone().  
 *
 */
/****************************************************************************/
#define ConsoleDup( x )		(((x) == -1) ? -1 : dup( (x) ))
static DSError	TrkAddChildProcessLinux( int processId )
{
	DSError err = kNoError;
	char buffer[1024];
	FILE *statFile;
	int parentId;
	int consoleFds[kConsoleCount];
	LinuxProcessState *parentProcessState;
	LinuxThreadState *parentThreadState;
	LinuxProcessState *pState;
	LinuxThreadState *tState;
	
	__puts( "Attaching to new child process " );
	__puthex8( processId );
	__puts( "\r\n" );
	
	//!!! if no follow-fork, then remove breakpoints (if mem
	//!!! not shared and parent controlled), and detach

	/*
	** Get the parent ID from the /proc stat file.
	*/

	parentId = -1;
	sprintf( buffer, "/proc/%lu/stat", (ui32)processId );
	statFile = fopen( buffer, "r" );
	if (statFile == NULL)
		err = kOsError;
		
	if (err == kNoError)
	{
		fscanf( statFile, "%*d (%*s %*c %d", &parentId );
		fclose( statFile );

		/*
		** The parent may be a thread within a process, so look it up by lwpid.
		*/
		
		if (FindLinuxThreadStateLwId( parentId, NULL, &parentThreadState ) == kNoError)
		{
			parentProcessState = TrkGetLinuxProcessState( parentThreadState->fProcessStateId );
			ASSERT( parentProcessState );
			
			/*
			** Add to the list of controlled processes, and copy the
			** console I/O pipes from the parent.  If stdout & stderr
			** are the same, they should use the same file descriptor.
			*/

			consoleFds[kDSStdin] = ConsoleDup( parentProcessState->fConsoleFds[kDSStdin] );
			consoleFds[kDSStdout] = ConsoleDup( parentProcessState->fConsoleFds[kDSStdout] );
			if (parentProcessState->fConsoleFds[kDSStderr] == parentProcessState->fConsoleFds[kDSStdout])
				consoleFds[kDSStderr] = consoleFds[kDSStdout];
			else
				consoleFds[kDSStderr] = ConsoleDup( parentProcessState->fConsoleFds[kDSStdout] );

			/*
			 * TRK associates a "console parent process ID" with each process.  
			 * All children should inherit the same such ID.
			 */
			err = OsTargetAddProcess( processId, consoleFds, parentProcessState->fConsoleParentId,
					&pState, &tState );
		
			if (err == kNoError)
			{
				/*
				** Copy the signal policy from the parent.
				*/
				memcpy( &pState->fStopMask, &parentProcessState->fStopMask,
					sizeof( TrkSignalMaskLinux ) );
				memcpy( &pState->fPassMask, &parentProcessState->fPassMask,
					sizeof( TrkSignalMaskLinux ) );
				
				/*
				** The child will inherit the parent's thread library state without
				** inheriting the threads.  Thus the inherited state will be bogus,
				** and the thread debug library will be fooled by it.  It'll be anarchy.
				** Nip it in the bud.
				*/
				
#ifndef __NotSupportThread__
				if (parentProcessState->fThreadAgent != NULL)
					pState->fAllowThreadDb = false;
#else
				pState->fAllowThreadDb = false;
#endif									

				/*
				** The child inherits all of the breakpoints from the parent.
				** Clear them all here.  New breakpoints will be set in the
				** child later as appropriate.
				*/
#ifndef __uClinux__
				ClearInheritedBreakpoints( parentProcessState->fProcessId,
						parentThreadState->fThreadId,
						processId, DS_MSG_BREAK_THREAD_ID_ALL );
#endif
				/*
				** Set up the shared library event breakpoint.  Then get the
				** shared object state for the new process without sending any
				** load/unload notifications to the host.
				*/
				
#ifndef __NotSupportSo__ 
				err = TrkSetSoBreakpointsLinux( pState, tState );
				if (err == kNoError)
					err = TrkUpdateSoStateLinux( pState, tState, false, false, NULL );
#endif					
			
			}

			
			/*
			** Notify the host of the new process.
			*/

			if (err == kNoError)
			{
				err = DoOsNotifyProcessCreated(processId, parentProcessState->fProcessId);
#ifdef FOR_ECLIPSE				
				OsTargetSetStopped( true, processId, 0 );
				DoNotifyStopped( kDSNotifyStopped, processId, 0 );
#endif			
			}
		}
#ifdef PPC
		else
		{
			/*
			** It seems that the parent was just killed
			*/
			err = ptrace(PTRACE_CONT, processId, 0, 0);
			if (err == -1)
			{
				__perror("continue the child ");
				return err;
			}
		}
#endif
	}

	//!!! what if they share memory (&parent controlled)? add as raw thread
	return( err );
}

/****************************************************************************/
/*
 *	TrkHandleExecEvent
 *
 *	Handle the case in which a process calls exec().
 *
 */
/****************************************************************************/
static DSError TrkHandleExecEvent( LinuxProcessState *pState, LinuxThreadState *tState )
{
	DSError err = kNoError;
	ui32 processId, newProcessId;
	int consoleFds[kConsoleCount];
	ui32 consoleParentId;
	int i;
		
#ifdef PPC
	/* 
	* See if the exec was executed from inside a thread 
	*/
	if (pState->fProcessId != tState->fLwProcessId) {
	
		newProcessId = tState->fLwProcessId;

		__puts("exec is from thread with lwpid: ");
		__puthex8(newProcessId);
		__puts("\r\n");

	} else {
	
		newProcessId = pState->fProcessId;

		__puts("exec if from main thread with lwpid: ");
		__puthex8(newProcessId);
		__puts("\r\n");

	}

#endif	
	
	tState->fLwProcessId=0;		/* This is done so that PTRACE_DETACH() 
								 * will NOT get invoked when an exec() function
								 * is called.  If PTRACE_DETACH()
								 * gets invoked, then current process
								 * image will not get changed due to exec()
								 * system as the PTRACE_DETACH() terminates
								 * process itself.
								 */

	
	/*
	** The only thing retained from the old process state is the process ID
	** and the console I/O pipes.  Set the console I/O pipes to -1 in the
	** old process state so PurgeProcess doesn't close them.
	*/

	processId = pState->fProcessId;
	for (i = 0; i < kConsoleCount; i++)
	{
		consoleFds[i] = pState->fConsoleFds[i];
		pState->fConsoleFds[i] = -1;
	}
	consoleParentId = pState->fConsoleParentId;

#ifdef PPC	

	/* 
	** Do cleanup depending of what exec type we have
	*/
	if ( newProcessId == processId )
	{
		err = PurgeProcess( pState->fProcessId );
//!!! clear old state (what about multi-threaded?) PurgeProcess should do it, except that it detaches from all threads if multithreaded
	} 
	else
	{
		err = PurgeProcessesOnly( pState->fProcessId );
		processId = newProcessId;
	}
	
#endif	
	/*
	** Create a fresh process state object and set up the new program for
	** debugging.
	*/

	if (err == kNoError)
		err = OsTargetAddProcess( processId, consoleFds, consoleParentId,
				&pState, &tState );
	if (err == kNoError)
		err = TrkSetupDebugProcess( pState, tState );

	/*
	** Notify the host that the exec occurred.
	*/

	if (err == kNoError)
		err = DoOsNotifyProcExecCreated( processId, pState->fExe );
	

	return( err );
}

/****************************************************************************/
/*
 *	OsTargetBlockFileIo
 *
 *	Block/unblock file IO requests from the target.
 *
 */
/****************************************************************************/
DSError OsTargetBlockFileIo( bool block )
{
	gBlockFileIo = block;
	return( kNoError );
}

/****************************************************************************/
/*
 *	TrkPrintMessage
 *
 *	Prints out a message to be seen by the user.  The message will be
 *	sent to both the console on the target and a debugger console window
 *	on the host.
 *
 */
/****************************************************************************/
DSError TrkPrintMessage( char *msg )
{
	size_t count;
	DSIOResult ioResult;
	DSError err;
	
	/* Print to the target console. */
	puts( msg );
	
	/* Print to the host debugger console window */
	count = TRK_strlen( msg );
	err = SuppAccessFile( kDSStdout, (ui8 *)msg, &count, &ioResult, true,
			false, TRK_PROCESS_ID, TRK_THREAD_ID );
	
	return( err );
}



DSError GetProcessInfo(ui32 pid, char* name, char* status, ui32* ppid, ui32* pgid, ui32* vms, ui32* rss)
{
	char statFileName[64];		// plenty, considering known path is /proc/<some_number>/stat
	char dontcare[64];			// buffer used when a parameter is NULL. 'name' is the largest needed; all others are 32 bits
	FILE * file = NULL;
	int length = 0;
	int count;	
	ui32 tmp;
	
	ASSERT(pid);

	/* caller may pass NULL for any field he's not interested in. We still need a valid ptr, though */	
	if (!name) 		name = dontcare;
	if (!status)	status = dontcare;
	if (!ppid)		ppid = (ui32*)dontcare;
	if (!pgid)		pgid = (ui32*)dontcare;	
	if (!vms)		vms = (ui32*)dontcare;	
	if (!rss)		rss = (ui32*)dontcare;	
	
	name[0] = '\0';				// init return params, just in case...
	status[0] = '\0';
	*ppid = 0;
	*pgid = 0;	
	*vms = 0;
	*rss = 0;

	sprintf(statFileName, "/proc/%lu/stat", pid);

	file = fopen(statFileName, "r");
	if (!file)
	{
		/* 
		 * Starting with Linux Kernel 2.14.20 (RedHat 9.0), all the lightweight 
		 * processes of a thread group except the main one are given a hidden
		 * directory in /proc
		 */	
		sprintf(statFileName, "/proc/.%lu/stat", pid);
		file = fopen(statFileName, "r");
		if (!file)
		{
			/* 
		 * The process doesn't exist. Caller may be using us to find out of the 
		 * main lwp for a thread-group is still alive, so this isn't necessarily
		 * indicative of an internal error (i.e., don't assert!).
		 */
		}
		return kOsError;
	}

	rewind(file);
	count = fscanf(file, "%*d (%s %c %d %d %d %d %d %lu %lu %lu %lu %lu %lu %lu %ld %ld %ld %ld %ld %ld %lu %lu %ld", name, status, ppid, pgid, \
		&tmp, &tmp, &tmp, &tmp, &tmp, &tmp, &tmp, &tmp, &tmp, &tmp, &tmp, \
		&tmp, &tmp, &tmp, &tmp, &tmp, &tmp, vms, rss);
	ASSERT(count == 23);
	count = count;		// shut up gcc warning about count being unused when ASSERT is not turned on
	
	fclose(file);

	/* removing the trailing ')'; but obviously not if the user didn't request the name */
	if (name != dontcare)
	{
		length = strlen(name);
		if (length)
		{
			ASSERT(name[length - 1] == ')');
			name[length - 1] = '\0';
		}
		else
			ASSERT(0);	// no name???
	}

	return kNoError;
}

/*
 * This function builds a list of lwpids found in the system based on the /proc entries.
 */
 
static OsLwpidItem* TRKBuildOsLwpidList(int* listCount)
{
	int lwpidCount;
	struct dirent** dList = NULL;
	
	ui32 pid;
	char name[512];
	ui8 state;
	ui32 ppid;
	ui32 pgid;
	ui32 vms;
	ui32 rss;
	
	int index;
	int tmp;
	
	OsLwpidItem* lwpidL;
	
	#if DEBUG_LWPID_LIST
		__puts("Building lwpid list\n");
	#endif
	
	lwpidCount = scandir("/proc", &dList, &SelectProcessDirent, &CompareProcessDirent);
	if (!lwpidCount)
		return (NULL);
	
	lwpidL = (OsLwpidItem*)malloc(lwpidCount*sizeof(OsLwpidItem));
	if (!lwpidL)
	{
		while(lwpidCount--)
			free(dList[lwpidCount]);
		free(dList);
		
		return (NULL);
	}
	memset(lwpidL, 0, lwpidCount*sizeof(OsLwpidItem));
	  	
	for (index = 0; index < lwpidCount; index++)
	{
	
		pid = (ui32)atol(dList[index]->d_name);

		if (pid == 0)
		{
			continue;
		}
		
		GetProcessInfo(pid, name, &state, &ppid, &pgid, &vms, &rss);

		lwpidL[index].pid = pid;
		lwpidL[index].state = state;
		lwpidL[index].name = (char*)strdup(name);
		lwpidL[index].ppid = ppid;
		lwpidL[index].pgid = pgid;
		lwpidL[index].vms = vms;
		lwpidL[index].rss = rss;


		#if DEBUG_LWPID_LIST
			
			__puts( "   pid: " );	
			__puthex8( pid );
			__puts( " 	name: " );
			__puts( name );
			__puts( " 	state: " );
			__putchar( state );
			__puts( " 	ppid: " );
			__puthex8( ppid );
			__puts( " 	pgid: " );
			__puthex8( pgid );
			__puts( " 	vms: " );
			__puthex8( vms );
			__puts( " 	rss: " );
			__puthex8( rss );
			__puts( "\r\n" );
		#endif
		
		free(dList[index]);
			
	}
	
	free(dList);
	
	*listCount = lwpidCount;
	return (lwpidL);
}

/*
 * This function builds a list of lwpids found in the system based on the /proc/<pid>/task entries.
 */
 
static OsLwpidItem* TRKBuildOsLwpidListTaskMode(int* listCount)
{
	int pidCount = 0, lwpidCount = 0;
	struct dirent** dList = NULL;
	struct dirent** dEntries = NULL;
	
	ui32 pid;
	char name[512];
	ui8 state;
	ui32 ppid;
	ui32 pgid;
	ui32 vms;
	ui32 rss;
	
	int index, i,j,size;
	int entries;
	char buffer[PATH_MAX];
		
	OsLwpidItem* lwpidL;

	#if DEBUG_LWPID_LIST
		__puts("Building lwpid list\n");
	#endif
	
	pidCount = scandir("/proc", &dList, &SelectProcessDirent, &CompareProcessDirent);
	if (pidCount <= 0)
		return (NULL);
	
	// assume that every process has maximum 2 threads
	lwpidCount = pidCount * 2;
	lwpidL = (OsLwpidItem*)calloc(lwpidCount, lwpidCount * sizeof(OsLwpidItem));
	
	if (!lwpidL)
	{
		while(lwpidCount--)
			free(dList[lwpidCount]);
		free(dList);
		
		return (NULL);
	}
	
	index = 0;
	for (i = 0; i < pidCount; i++)
	{
		pid = (ui32)atol(dList[i]->d_name);

		if (pid == 0)
			continue;
		
		memset(buffer, 0 , strlen(buffer));
		sprintf(buffer, "/proc/%lu/task", pid);
		
		entries = scandir(buffer, &dEntries, &SelectProcessDirent, &CompareProcessDirent);
		
		if (entries <= 0)
			continue;
		
		for (j = 0; j < entries; j++)
		{
			pid = (ui32)atol(dEntries[j]->d_name);
			
			if (pid == 0)
				continue;
			
			if (index >= lwpidCount)
			{
				// expand the lwpidL space in case we have more pids then the initial assumption
				lwpidCount += pidCount;
				lwpidL = (OsLwpidItem*)realloc(lwpidL, lwpidCount * sizeof(OsLwpidItem));
				memset(lwpidL + (lwpidCount - pidCount), 0, pidCount * sizeof(OsLwpidItem));
			}
			
			GetProcessInfo(pid, name, &state, &ppid, &pgid, &vms, &rss);

			lwpidL[index].pid = pid;
			lwpidL[index].state = state;
			lwpidL[index].name = (char*)strdup(name);
			lwpidL[index].ppid = ppid;
			lwpidL[index].pgid = pgid;
			lwpidL[index].vms = vms;
			lwpidL[index].rss = rss;

			// if the /proc/<pid>/task list contains more than one lwpid the other lwpids are subthreads of the first one
			if (j > 0)
			    lwpidL[index].isSubthread = true;

			#if DEBUG_LWPID_LIST
			
				__puts( "   pid: " );	
				__puthex8( pid );
				__puts( " 	name: " );
				__puts( name );
				__puts( " 	state: " );
				__putchar( state );
				__puts( " 	ppid: " );
				__puthex8( ppid );
				__puts( " 	pgid: " );
				__puthex8( pgid );
				__puts( " 	vms: " );
				__puthex8( vms );
				__puts( " 	rss: " );
				__puthex8( rss );
				__puts( "\r\n" );
			#endif

			index++;

			free(dEntries[j]);
		}
		free(dEntries);
		
		free(dList[i]);
	}
	
	free(dList);
	
	if (index < lwpidCount)
		lwpidL = (OsLwpidItem*)realloc(lwpidL, index * sizeof(OsLwpidItem));
	
	*listCount = index;
	return (lwpidL);
}

/*
 * This function gets as arguments a list of lwpids and parse it. For each lwpid
 * it will find it's parent and determine whether lwpid is a subthread of its parent.
 * It this is true then add the child in the parent's list of subthreads.
 */
  
static int TRKBuildOSLwpidRelationships(OsLwpidItem* lwpidList, int lwpidCount)
{
	int parent;
	int index;
	OsLwpidItem* entry;
	
	for (index = 0; index < lwpidCount; index++)
	{
		/*
		 * Loop to find the parent entry.
		 */
		for(parent = 0; parent < lwpidCount; parent++)
		{
			/*
			 * We have found the parent.
			 * For subthreads found on a /proc/<pid>/task list the condition to determine the parent is:
			 * - to have the same lwpid at parent PID and subthread PGID
			 * - to have the same PPID at parent and subthread
			 */
			if (lwpidList[index].isSubthread)
			{
			    if (lwpidList[parent].pid == lwpidList[index].pgid && lwpidList[parent].ppid == lwpidList[index].ppid)
				break;
			}
			else
			{
			    if (lwpidList[parent].pid == lwpidList[index].ppid)
				break;
			}	
		}

		/*
		 * If we do not find the parent it must be died. In this case
		 * we consider this lwpid as being a process.
		 */
		if (parent == lwpidCount)
				  continue;
		
		if (IsSubthreadOf(&lwpidList[parent], &lwpidList[index]))
		{
			  lwpidList[index].isSubthread = true;
				  
			  /*
			   * Add the thread in the parent's list of subthreads.
			   * We have to add the entry at the end of the list in order
			   * to get the right TIDs later on.
			   */
			  entry = lwpidList[parent].threads;
			  if (entry == NULL)
			  {
		  		lwpidList[parent].threads = &lwpidList[index];
			  }
			  else
			  {
				while(entry->next)
				{
					entry = entry->next;
				}

				entry->next = &lwpidList[index];
			  }		
		}
	}

	return (0);
}

/*
 * This function gets two lwpid entries and returned true if
 * the second is a subthread of the first.
 */
 
static int IsSubthreadOf(OsLwpidItem* parent, OsLwpidItem* child)
{
	
	if (parent->vms != child->vms)
			  return (0);
	
	if (parent->rss != child->rss)
			  return (0);

	return (1);
}

/****************************************************************************/
/*
 *	PostSupportEvent
 *
 *	Post a support event with the given info to the event queue.  This
 *	function can be called at any time, allowing concurrent execution to be
 *	synchronized through the event loop.
 */
/****************************************************************************/
static DSError PostSupportEvent( ui32 info0, ui32 info1, ui32 info2, ui32 info3, ui32 info4,
						ui32 info5, ui32 info6 )
{
	NubEvent event;

	ConstructEvent(&event, kSupportEvent);

	/* Store provided info in the event */
	event.fInfo[0] = info0;
	event.fInfo[1] = info1;
	event.fInfo[2] = info2;
	event.fInfo[3] = info3;
	event.fInfo[4] = info4;
	event.fInfo[5] = info5;
	event.fInfo[6] = info6;

	/* Post an event to be handled in the main loop */
	return PostEvent(&event);
}

/****************************************************************************/
/*
 *	PostSendInitConsoleToDebuggerEvent
 *
 *	In various locations, the TRK needs to send the debugger an initial
 *  (effectively empty) console message to the debugger so that it opens
 *  a log window.
 *	
 */
/****************************************************************************/
DSError PostSendInitConsoleToDebuggerEvent(ui32 processId)
{
	return PostSupportEvent(kSupportEvent_sendInitConsoleToDebugger, processId, 0, 0, 0, 0, 0);
}

/****************************************************************************/
/*
 *	TargetSupportRequest
 *
 *	Flexible mechanism for either servicing support request from the target 
 *  program or for deferring a particular action while handling another.
 *
 */
/****************************************************************************/
DSError TargetSupportRequest(NubEvent *event)
{
	DSError err = kNoError;
	
	switch (event->fInfo[0])
	{
		case kSupportEvent_sendInitConsoleToDebugger:
		{
			char str[] = "\r\n";
			size_t count = sizeof(str) - 1;
			DSIOResult io_result = 0;
			
			ui32 processId = event->fInfo[1];
			ASSERT(processId);

			/* Send as a WriteFile command to stdout. */
			return SuppAccessFile(kDSStdout, (ui8 *)str, &count, &io_result,
							true, false, processId, 0 );
		}
			
		default:
			__puts( "Unknown support event\r\n" );
			ASSERT(0);
			err = kUnsupportedError;
	}
	
	return err;
}
