/******************************************************************************************/
/*
	
DESCRIPTION

	Linux-specific implementation function prototypes and structures.

COPYRIGHT

	Copyright 2000-2006, 2009 Freescale Semiconductor, Inc.
	
*/
/******************************************************************************************/

#ifndef __LINUX_TARGIMPL_H__
#define __LINUX_TARGIMPL_H__

#include "target.h"
#include "dstypes.h"
#include "dserror.h"
#include "obarray.h"
#include "step.h"
#include "linux_msgtarg.h"

#include <sys/ptrace.h>
#include <link.h>
#include <thread_db.h>

/****************************************************************************/
/*
 *	Types
 */
/****************************************************************************/


#define kConsoleCount				3
#define kSectionCount				4


//Aditional ptrace() option used in AppTRK for capturing uClinux process events.
//They were introduced in 2.6 kernels and most toolchains do not define them in
//the sys/ptrace.h header. Define them here.
#ifndef PTRACE_SETOPTION
#define PTRACE_SETOPTIONS       0x4200
#endif
#ifndef PTRACE_GETEVENTMSG
#define PTRACE_GETEVENTMSG      0x4201
#endif

// options set using PTRACE_SETOPTIONS 
#ifndef PTRACE_O_TRACESYSGOOD
#define PTRACE_O_TRACESYSGOOD   0x00000001
#endif
#ifndef PTRACE_O_TRACEFORK
#define PTRACE_O_TRACEFORK      0x00000002
#endif
#ifndef PTRACE_O_TRACEVFORK
#define PTRACE_O_TRACEVFORK     0x00000004
#endif
#ifndef PTRACE_O_TRACECLONE
#define PTRACE_O_TRACECLONE     0x00000008
#endif
#ifndef PTRACE_O_TRACEEXEC
#define PTRACE_O_TRACEEXEC      0x00000010
#endif
#ifndef PTRACE_O_TRACEVFORKDONE
#define PTRACE_O_TRACEVFORKDONE 0x00000020
#endif
#ifndef PTRACE_O_TRACEEXIT
#define PTRACE_O_TRACEEXIT      0x00000040
#endif

// Wait extended result codes for the above trace options.
#ifndef PTRACE_EVENT_FORK
#define PTRACE_EVENT_FORK       1
#endif
#ifndef PTRACE_EVENT_FORK
#define PTRACE_EVENT_FORKK      2
#endif
#ifndef PTRACE_EVENT_CLONE
#define PTRACE_EVENT_CLONE      3
#endif
#ifndef PTRACE_EVENT_EXEC
#define PTRACE_EVENT_EXEC       4
#endif
#ifndef PTRACE_EVENT_VFORKDONE
#define PTRACE_EVENT_VFORKDONE  5
#endif
#ifndef PTRACE_EVENT_EXIT
#define PTRACE_EVENT_EXIT       6
#endif

typedef struct TrkSoInfo {
	ElfW(Addr)				l_addr;
	char					*l_name;
	struct TrkSoInfo		*l_next;
} TrkSoInfo;

/****************************************************************************/
/*
 *	Linked list element used to store the full list of lwpids in the
 *	OS. This list is only for internal use and the processes/threads list are based on it.
 */
/***************************************************************************/

typedef struct OsLwpidItem
{
		  ui32 pid;
		  char* name;
		  ui8 state;
		  ui32 ppid;
		  ui32 pgid;
		  ui32 vms;
		  ui32 rss;
		  struct OsLwpidItem* next;
		  struct OsLwpidItem* threads;
		  bool isSubthread;
} OsLwpidItem;


typedef struct LinuxProcessState {
	ObjectHeader			fHeader;			/* needed by obarray routines */
	ui32					fProcessId;			/* ID of the process */
	BreakpointId			fSoBreakpointId;	/* breakpoint used for shared-object debug */
	InvisibleRangeInfo		*fInvisibleStepRangeList;
	ui32					fInvisibleStepRangeCount;
	struct r_debug			*fSoDebugPtr;		/* pointer to r_debug struct (target ptr) */
	TrkSoInfo				*fSoList;			/* list of shared objects for the process */
	td_thragent_t			*fThreadAgent;		/* pointer to thread agent for the process */
	BreakpointId			fThreadCreateBreakpointId;	/* breakpoint triggered by thread creation event */
	BreakpointId			fThreadDeathBreakpointId;	/* breakpoint triggered by thread death */
	BreakpointId			fThreadSignalBreakpointId;	/* breakpoint triggered by thread signal */
	unsigned int			fNextThreadId;		/* next ID to be used for new thread */
	TrkSignalMaskLinux		fStopMask;			/* signals on which to stop */
	TrkSignalMaskLinux		fPassMask;			/* signals which should be passed on */
	int						fConsoleFds[kConsoleCount];	/* console pipe FDs for stdin, stdout, & stderr respectively */
	ui32					fConsoleParentId;	/* ID of the parent process for the console */
	bool					fBlockStdin;		/* block reads on stdin */
	bool					fPrimaryThreadDead;	/* is the thread whose lwpid==fProcessId dead? */
	bool					fAllowThreadDb;		/* OK to use multi-thread debug lib? */
	bool					fDlWarned;			/* Already warned user about dynamic loader symbols? */
	bool					fThreadLibWarned;	/* Already warned user about thread library symbols? */
	char					*fExe;				/* path to executable file for this process */

/* Delayed process attachment hack for RedHat 9.0. See explanation elsewhere; search on this macro */
#ifdef TRK_DELAY_ATTACHING_TO_PROCESS_THREADS
	bool					fReadyForDelayedProcThreadsAttach; /* set to true when it's safe to do the first TrkAttachProcessThreadsLinux call */
	bool					fDelayedProcThreadsAttachDone; /* set to true once we've done the delayed TrkAttachProcessThreadsLinux call */
#endif
	
} LinuxProcessState;

typedef struct OsThreadGroupItem
{
	ui32	tgid;	// thread group id; the lwpid of the first lwp created in the process (thread group)
	ui32*	lwpids;	// a null terminated array of the ids of the lwps in the process
} OsThreadGroupItem;

/****************************************************************************/
/*
 *	Globals
 */
/****************************************************************************/

extern const int *TrkDefaultIdToPtraceOffset;
extern const int *TrkDefaultIdToThreadDbOffset;

/****************************************************************************/
/*
 *	Function prototypes
 */
/****************************************************************************/

#include "linux_thread.h"

__extern_c

	struct MessageBuffer;

	ui32 TrkProcess();
	ui32 TrkThread();


	DSError OsTargetAdjustBreakPC( ui32 processId, ui32 threadId );
	bool OsTargetStepDone( ui32 processId, ui32 threadId );
	void OsTargetSetStopped( bool stop_state, ui32 processId, ui32 threadId );
	DSError PurgeProcess( ui32 processId );
	bool OsTargetCheckStep( ui32 processId, ui32 threadId );
	DSError OsTargetDoStep( ui32 processId, ui32 threadId );
	DSError OsTargetAddProcess( ui32 processId, int consoleFds[3], ui32 consoleParentId,
							LinuxProcessState **resultPstate, 
							LinuxThreadState **resultTstate );
	DSError TrkSetupDebugProcess( LinuxProcessState *pState, LinuxThreadState *tState );
	DSError OsTargetKillProcessLinux( ui32 processId );
	LinuxProcessState *TrkGetLinuxProcessState( ObjectId i );
	ObjectId TrkGetLinuxProcessStateId( LinuxProcessState *pState );
	DSError TrkHandleProcessDeathLinux( ui8 type, ui32 code,
								ui32 processId );
	DSError FindLinuxProcessStateId( ui32 processId,
								ObjectId *resultObId,
								LinuxProcessState **result );
	DSError TrkPrintMessage( char *msg );

	DSError TrkBuildOsThreadGroupList(OsThreadGroupItem ** list);
	DSError TrkFreeOsThreadGroupList(OsThreadGroupItem * list);
	
	DSError	TrkAccessDefaultSingleProcessLinux(unsigned int reg, DefaultType *value,
						bool read,
						ui32 processId);

	DSError GetProcessInfo(ui32 pid, char* name, char* status, ui32* ppid, ui32* pgid, ui32* vms, ui32* rss);
	
	void PostInterruptEvent( int status, ui32 processId, ui32 threadId );
	DSError OsTargetKillThreadLinux( ui32 processId, ui32 threadId );
	
	DSError TargetAddProcessList( struct MessageBuffer *b, ui8 options, ui32 filter,
							ui32 startIndex,
                            ui32 *returnedCount,
                            ui32 *totalCount );

	DSError	TrkContinueProcessLinux( ui32 processId, LinuxThreadState *tState );
	
	DSError TrkStopProcessLinux( ui32 processId );
	DSError TrkKillProcessLinux( ui32 processId );

	DSError PostSendInitConsoleToDebuggerEvent(ui32 processId);
	
	DSError OsTemporarilyStopThread(ui32 processId, LinuxThreadState *tState, bool * bStoppedByRequest);
		
__end_extern_c

/****************************************************************************/
/*
 * OS related vars and structures
 */
/****************************************************************************/


#endif /* __LINUX_TARGIMPL_H__ */
