/******************************************************************************************/
/*

DESCRIPTION

	Linux thread handling routines.
	
COPYRIGHT

	Copyright 2000, 2002-2004, 2006, 2009 Freescale Semiconductor, Inc.
		
*/
/******************************************************************************************/

#ifndef _LINUX_THREAD_H_
#define _LINUX_THREAD_H_

/******************************************************************************************/
/*
 *	Includes
 */
/******************************************************************************************/

#include "obarray.h"
#include "step.h"

#include <sys/ptrace.h>
#include <thread_db.h>

/******************************************************************************************/
/*
 *	Errors
 */
/******************************************************************************************/

/******************************************************************************************/
/*
 *	Types
 */
/******************************************************************************************/

typedef struct LinuxThreadState {
	ObjectHeader			fHeader;			/* needed by obarray routines */
	StepStatii				fStepStatii;		/* statii of current step commands if any */
	enum __ptrace_request	fContinueType;		/* continue type */
	ui32					fThreadId;			/* ID of the thread (assigned by TRK) */
	thread_t				fDbThreadId;		/* thread_db ID of the thread */
	lwpid_t					fLwProcessId;		/* light-weight process ID for the thread */
	unsigned long			fSignal;			/* last signal that stopped the thread */
	bool					fStopped;			/* stopped state of the thread */
	PCType					fLastPc;			/* previous PC value */
	ObjectId				fProcessStateId;	/* ID of associated LinuxProcessState object */
	void					*fThreadResult;		/* exit code for this thread */
} LinuxThreadState;
	
/******************************************************************************************/
/*
 *	Public Function Prototypes
 */
/******************************************************************************************/
#include "linux_targimpl.h"

__extern_c

	struct MessageBuffer;

	DSError TrkInitializeThreadSupportLinux( void );
	DSError TrkUpdateProcessThreadsLinux( LinuxProcessState *pState );
	DSError TrkPurgeProcessThreadsLinux( LinuxProcessState *pState );
	DSError TrkHandleThreadDeathLinux( void *threadResult,
								ui8 processDeathType, ui32 processDeathCode,
								ui32 processId, ui32 threadId );
	DSError FindLinuxThreadStateId( ui32 processId,
									ui32 threadId,
									ObjectId *resultObId,
									LinuxThreadState **result );
	DSError FindLinuxThreadStateDbId( ui32 processId,
									thread_t dbThreadId,
									ObjectId *resultObId,
									LinuxThreadState **result );
	DSError FindLinuxThreadStateLwId( lwpid_t lwProcessId,
									ObjectId *resultObId,
									LinuxThreadState **result );
	DSError FindLinuxThreadStateLwAccessible( ui32 processId,
									ObjectId *resultObId,
									LinuxThreadState **result );
	DSError FindLinuxThreadStateLwFirst( ui32 processId,
									ObjectId *resultObId,
									LinuxThreadState **result );
	DSError TrkAddThreadLinux( ObjectId pStateId, thread_t dbThreadId,
								lwpid_t lwProcessId, ui32 *threadId,
								ObjectId *threadStateId );
	LinuxThreadState *TrkGetLinuxThreadState( ObjectId i );
	DSError	TrkAccessDefaultSingleThreadLinux(unsigned int reg, DefaultType *value,
						bool read,
						LinuxThreadState *tState );
	
	DSError TargetAddThreadList( struct MessageBuffer *b, ui8 options, ui32 filter,
							ui32 startIndex,
                            ui32 *returnedCount,
                            ui32 *totalCount );
	
	DSError TrkInitializeProcessThreadsLinux( LinuxProcessState *pState );

	DSError	TrkAccessFPThreadLinux(unsigned int firstRegister, unsigned int lastRegister,
						struct MessageBuffer *b, size_t* registerStorageSize,
						bool read,
						LinuxThreadState *tState );

	DSError TrkContinueThreadLinux( LinuxThreadState *tState );
	DSError TrkStopThreadLinux( LinuxThreadState *tState );
	DSError TrkKillThreadLinux( LinuxThreadState *tState );
	
	DSError TrkHandleThreadCreateEventLinux( LinuxProcessState *pState );
	DSError TrkHandleThreadDeathEventLinux( LinuxProcessState *pState );
	
	DSError InitializeThreadArray();
	
	DSError TrkKillProcessThreadsLinux( ui32 processId );

	DSError TrkAttachProcessThreadsLinux( LinuxProcessState *pState );

__end_extern_c

#endif /* #ifndef _LINUX_THREAD_H_ */
