/******************************************************************************************/
/*
	
DESCRIPTION

	Dummy memory map for Linux.  This is referenced by some of the bare-board
	PPC code, but that particular code is not called, so this isn't used.
	Just define an empty memory map.

COPYRIGHT

	Copyright 2000, 2002, 2004, 2006, 2009 Freescale Semiconductor, Inc.
	
*/
/******************************************************************************************/

#ifndef __MEMMAP_H
#define __MEMMAP_H

#include "dstypes.h"

/******************************************************************************************/


typedef struct memRange {
	ui8		*start;
	ui8		*end;
	bool	readable;
	bool	writeable;
} memRange;


/*
** Specify valid memory ranges in this array.  When memory
** ranges overlap, they should be specified in order of
** checking priority.  A particular memory access will be
** accepted or rejected according the first entry in the
** table matching its address range.
**
** Examples:
**	1.	To disallow access everywhere except a range of
**		valid memory between addresses x and y, the table 
**		would be:
**
**		{
**			{x, y, true, true}
**		}
**
** 		Any memory range that falls within x..y would be
**		accepted.  Everything else would be rejected by
**		default.
**
**	2.	To allow access everywhere except a range of invalid
**		memory between addresses x and y, the table would
**		be:
**
**		{
**			{x, y, false, false},
**			{0x00000000, 0xffffffff, true, true}
**		}
**
**		Any memory range that falls within or overlaps x..y
**		would be rejected.  Everything else would fall within
**		0x00000000..0xffffffff and be accepted.
**
**
** A single entry covers the entire memory space so that 
** all memory accesses will be allowed.
**
*/


const memRange gMemMap[] =
{
	{(ui8 *)0x00000000, (ui8 *)0xffffffff, true, true },
};

#endif /* #ifndef __MEMMAP_H */
