/******************************************************************************************/
/*

DESCRIPTION

	Target-specific implementation info for debugging output.

	See the header files <usr_put.h>/<usr_put.c>, which contain
	the portable debug output routines and API.

COPYRIGHT

	Copyright 2000-2002, 2004-2006, 2009 Freescale Semiconductor, Inc.
	
*/
/*****************************************************************************************/

/*
** Define any combination of these two.  Note that serial debug output
** uses the same serial channel as the TRK commands themselves.  It is
** useful for manual debugging, but is not likely to work with the host
** plugin.
*/

#undef DEBUGIO_SERIAL			/* define for writing out to serial port */
#undef DEBUGIO_RAM				/* define for writing to memory */

/*
** If DEBUGIO_CUSTOM is defined, the the following must also be defined,
** either as functions or as macros:
**
**		void usr_put_initialize_custom( void );
**			This is called once at startup to initialize target-
**			specific debug output.
**
**		void usr_putchar_custom( int c );
**			This is called for each debug output character.
**
*/
#include <stdio.h>
#define usr_put_initialize_custom()
#define usr_putchar_custom( c )			{putchar( c ); fflush( stdout );}

#ifdef DEBUGIO_CUSTOM
	#include <errno.h>
	#include <string.h>
#if __GNUC__ < 3
	#define __perror( s )				{ \
											__puts( s ); \
											__puts( ": " ); \
											if (errno > 0 && errno < sys_nerr) \
												__puts( sys_errlist[ errno ] ); \
											__puts( "\r\n" ); \
										}
#else	
	#define __perror( s )			{ \
										  int errnum = errno; \
										  char *buf = strerror(errnum); \
										  __puts( s ); \
										  __puts( ": " ); \
										  if (errno == errnum) \
												{__puts( buf );} \
										}
#endif
#else
	#define __perror( s ) {}
#endif

/*
** When DEBUGIO_RAM is defined, then defining DB_RAM_CONSOLE_DUMP will
** cause the debug output in the RAM buffer to be periodically
** dumped to the console (stdout) via the kDSWriteFile command.  This
** is useful when using the host plugin to drive CodeWarriorTRK, since the
** debug output will appear in the debugger's console window.  Note,
** however, that there are only certain times that the console dump
** can reliably be done.  Currently it is only done when the debugger tells
** CodeWarriorTRK to step or execute the target program.  Debug output 
** in between these times is queued up until the next dump opportunity.
*/
#ifdef DEBUGIO_RAM
	#undef DB_RAM_CONSOLE_DUMP	/* define for dumping debug buffer to console */
#endif

/*
** Define a buffer to receive debugging output when DEBUGIO_RAM is
** defined.
*/

#ifdef DEBUGIO_RAM
	#define	DB_START ((char *) 0x007f0000)		/* start of debug buffer */
	#define	DB_END   ((char *) 0x00800000)		/* end of debug buffer non-inclusive */
#endif
