/****************************************************************************/
/*

DESCRIPTION

	Register numbers for PPC Linux.
	
	These register blocks are accessed via the kDSReadRegisters and
	kDSWriteRegisters messages.  The register number for a specific
	register is determined by its corresponding element number in these
	structures.  The register data in the messages must conform to
	the structures defined here in big-endian format.
	
AUTHORS

	Copyright 2002, 2006, 2009 Freescale Semiconductor, Inc.
	
*/
/****************************************************************************/

#ifndef _LINUX_ARM_REG_H_
#define _LINUX_ARM_REG_H_

#include "dstypes.h"
#include "arm_reg.h"	// Generic ARM Register Definitions
/****************************************************************************/

typedef struct Default_ARM
{
	DefaultType	GPR[11];
	DefaultType	FP;
	DefaultType	IP;
	DefaultType	SP;
	DefaultType	LR;
	DefaultType	PC;
	DefaultType	CPSR;
} Default_ARM;

/*
 * Info stored in NotifyStopped message.
 */


typedef struct StopInfo_PPC
{
	PCType	PC;
	InstructionType	PCInstruction;		/* instruction at PC */
	ExceptionCauseType	exceptionID;	/* same as vector #, e.g. 0x0200 */
} StopInfo_PPC;


/*
 * Info stored in NotifyException message.
 */
 
typedef StopInfo_PPC ExceptionInfo_PPC;

/*
** Info used by breakpoint code to restore (unset)
** breakpoints.  For software breakpoints it is just
** the saved instruction.
*/

typedef InstructionType BreakpointRestore;


/*
 *	full processor state
 */

typedef struct ProcessorState_PPC_Linux
{
	Default_ARM			Default;		/* Default register block */
#if TRK_TRANSPORT_INT_DRIVEN
	DefaultType 			transport_handler_saved_ra;
#endif
} ProcessorState_PPC_Linux;

#endif /* #ifndef _LINUX_ARM_REG_H_ */
