/****************************************************************************/
/*
	
DESCRIPTION

	Target-specific implementation for Linux on ColdFire/M68K
	

AUTHORS

	Copyright 2003, 2006, 2009 Freescale Semiconductor, Inc.
	
*/
/****************************************************************************/

/*
** TRK includes
*/

#include "trk_prefix.h"

#include "usr_put.h"		// must be included before msgcmd.h
#include "msg.h"
#include "targimpl.h"
#include "main_TRK.h"
#include "linux_targimpl.h"
#include "target.h"
#include "m68k_reg.h"
#include "m68k_instr.h"
#include "break.h"

/*
** Linux includes
*/
#include <errno.h>
#include <linux/ptrace.h>

/*
** Define this to 1 to get detailed debugging output when accessing
** the floating point registers.
*/
#define DEBUG_FLOAT_ACCESS		(1)


/*
** Map from CodeWarriorTRK register ID to Linux register offset.
** Note: PT_R0, etc. are defined in <asm/ptrace.h>
*/
#define LINUX_INVALID_REG_ID			((U32)-1)

const int TrkDefaultIdToPtraceOffsetArray[ TRK_DEFAULT_MAX_REGISTER + 1 ] =
{
	PT_D1*4,
	PT_D2*4,
	PT_D3*4,
	PT_D4*4,
	PT_D5*4,
	PT_D6*4,
	PT_D7*4,
	PT_A0*4,
	PT_A1*4,
	PT_A2*4,
	PT_A3*4,
	PT_A4*4,
	PT_A5*4,
	PT_A6*4,
	PT_D0*4,
	PT_USP*4,
	PT_ORIG_D0*4,
	PT_SR*4,
	PT_PC*4,
};

const int *TrkDefaultIdToPtraceOffset =
		TrkDefaultIdToPtraceOffsetArray;

const int *TrkDefaultIdToThreadDbOffset =
		TrkDefaultIdToPtraceOffsetArray;

/*
** The FPR's start at (PT_FPR0 * 4) and occupy 8 bytes each. 
*/

#define PTRACE_FPR_OFFSET( r )		((PT_FPR0 * 4) + ((r) * 8))

/****************************************************************************/
/*
 *	TrkAccessFPProcessLinux
 *
 *	R/W a range of floating point registers.  W/R the registers in the
 *  range directly to/from the given message buffer.  Returns the size of the
 *  requested range in bytes.
 *
 *	NOTE:This is a low-level implementation that can be used
 *	to access the light-weight process for a thread.  This means that
 *	processId may NOT be the main process ID for the given thread.  This
 *	is important, because many of the TRK functions which accept a
 *	processId will ONLY accept the main process ID.  Avoid calling other
 *	TRK functions with the given process ID.
 */
/****************************************************************************/
DSError	TrkAccessFPProcessLinux(unsigned int firstRegister, unsigned int lastRegister,
						MessageBuffer *b, size_t* registerStorageSize,
						bool read,
						ui32 processId )
{
	DSError err = kNoError;
	return( err );
}						


/****************************************************************************/
/*
 *	OsTargetAccessExtended1
 *
 *	R/W a range in the extended1 register block.  W/R the registers in the
 *  range directly to/from the given message buffer.  Returns the size of the
 *  requested range in bytes.
 */
/****************************************************************************/
DSError	OsTargetAccessExtended1(unsigned int firstRegister, unsigned int lastRegister,
						MessageBuffer *b, size_t* registerStorageSize,
						bool read,
						ui32 processId,
						ui32 threadId )
{
#if __MWERKS__
	#pragma unused(firstRegister, lastRegister, b, read, processId, threadId)
#endif /* __MWERKS__ */
	
	*registerStorageSize = 0;
	
	return kUnsupportedError;		// no extended1 regs under Linux
}						

/****************************************************************************/
/*
 *	OsTargetAccessExtended2
 *
 *	R/W a range in the extended2 register block.  W/R the registers in the
 *  range directly to/from the given message buffer.  Returns the size of the
 *  requested range in bytes.
 */
/****************************************************************************/
DSError	OsTargetAccessExtended2(unsigned int firstRegister, unsigned int lastRegister,
						MessageBuffer *b, size_t* registerStorageSize,
						bool read,
						ui32 processId,
						ui32 threadId )
{
#if __MWERKS__
	#pragma unused(firstRegister, lastRegister, b, read, processId, threadId)
#endif /* __MWERKS__ */
	
	*registerStorageSize = 0;
	
	return kUnsupportedError;		// no extended1 regs under Linux
}						


/****************************************************************************/
/*
 *	OsTargetGetPC
 *
 *	Get the PC for the specified thread.
 */
/****************************************************************************/
PCType OsTargetGetPC( ui32 processId, ui32 threadId )
{
	DefaultType value;

	OsTargetAccessDefaultSingle( TRK_DEFAULT_PC,
			&value, true, processId, threadId );

	return( (PCType)value );
}						



/****************************************************************************/
/*
 *	OsTargetAdjustBreakPC
 *
 *	When the target stops at a breakpoint, the PC may or may not be
 *	where we want it.  Adjust accordingly.
 *
 *  NOTE1: This should be called once after each breakpoint is hit,
 *		   before the PC is used for anything. In particular before
 *		   sending out a stop notification or performing a continue.
 *
 */
/****************************************************************************/
DSError OsTargetAdjustBreakPC( ui32 processId, ui32 threadId )
{
	/*
	** For ColdFire, nothing to do.
	*/
}

/****************************************************************************/
/*
 *	OsTargetSetBreakpoint
 *
 *	Sets a breakpoint according to the given breakpoint descriptor.
 */
/****************************************************************************/
DSError OsTargetSetBreakpoint( Breakpoint *b )
{
	DSError result = kNoError;
	InstructionType instr;
	
	if (b->fHardware)
	{
		__puts( "Attempting to set HW breakpoint: unsupported\r\n" );
		result = kUnsupportedError;
	}
	else
	{
		result = DefaultTargetSetSoftwareBreakpoint( b, m68k_BREAK );
	}
	
	return( result );
}

/****************************************************************************/
/*
 *	OsTargetClearBreakpoint
 *
 *	Clear the breakpoint described in the given breakpoint object.
 */
/****************************************************************************/
DSError OsTargetClearBreakpoint( Breakpoint *b )
{
	DSError result = kNoError;
	InstructionType instr;
	
	if (b->fHardware)
	{
		result = kUnsupportedError;
	}
	else
	{
		result = DefaultTargetClearSoftwareBreakpoint( b );
	}
	
	return( result );
}

/****************************************************************************/
/*
 *	OsTargetHideBreakpoint
 *
 *	Hide the breakpoint described in the given breakpoint object.
 */
/****************************************************************************/
DSError OsTargetHideBreakpoint( Breakpoint *b, void *data, size_t maxLen  )
{
	return( DefaultTargetHideBreakpoint( b, data, maxLen ) );
}

/****************************************************************************/
/*
 *	OsTargetAccessInstruction
 *
 *	Read instruction from memory, determining instruction size.  The
 *  storage pointed to by readData must be big enough to store the
 *  largest possible instruction.
 *
 *	Note: if maxSize is -1, the size is unlimited
 */
/****************************************************************************/
DSError OsTargetAccessInstruction(InstructionType* data,
								void* virtualAddress, size_t maxSize,
								MemoryAccessOptions accessOptions,
								bool read,
								ui32 processId, ui32 threadId)
{
	DSError err = kNoError;
	size_t length;
	size_t outlength;
	
	/*
	** Determine instruction size.  Instructions on CF are 32 bits.
	** Bound it by maxSize.
	*/

	length = sizeof( InstructionType );
	if (maxSize != -1 && maxSize < length)
		length = maxSize;


	/*
	** Read/write the instruction in memory.  If we're doing this in the
	** CodeWarriorTRK process, then just use memcpy.
	*/
	
	outlength = length;
	if (processId != getpid())
	{
		err = OsTargetAccessMemory( data, virtualAddress, &outlength,
									accessOptions, read,
									processId, threadId);
	}
	else
	{
		if (read)
			memcpy( data, virtualAddress, length );
		else
			memcpy( virtualAddress, data, length );
	}
	
	/*
	 * Verify that an entire instruction was read/written.
	 */
	 
	if (err == kNoError && length != outlength)
		err = kInvalidMemory;
		
	return( err );
}
