/****************************************************************************/
/*

DESCRIPTION

	PPC instruction constants & macros.

COPYRIGHT

	Copyright 2000, 2002, 2004, 2006, 2008, 2009 Freescale Semiconductor, Inc.

*/
/****************************************************************************/

#ifndef __ppc_instr_h__
#define __ppc_instr_h__


/*
** Sign-extend the lower 26-bit value of a 32-bit word by shifting
** it left 6 bits, and then doing an ARITHMETIC shift right
** by 6 bits.  Do the arithmetic shift using a division by 2^6.
*/

#define SIGN_EXTEND_26( x )			( ((signed long)(((unsigned long)(x)) << 6)) / 0x40)

/*
** Sign-extend the lower 16-bit value of a 32-bit word by shifting
** it left 16 bits, and then doing an ARITHMETIC shift right
** by 16 bits.  Do the arithmetic shift using a division by 2^16.
*/

#define SIGN_EXTEND_16( x )			( ((signed long)(((unsigned long)(x)) << 16)) / 0x10000)


/*
** Instruction used for breakpoint.
*/

#define	PPC_BREAK			0x00000000

/*
** Extract the primary opcode from an instruction.  It is in
** bits 0-5.
*/

#define PPC_OPCODE_MASK		0xfc000000
#define PPC_OPCODE( x )		(((ui32)(x) & PPC_OPCODE_MASK) >> 26)

/*
** Useful opcode values.
*/

#define OP_BX				18
#define OP_BCX				16

/*
** This opcode is used by both BCCTRX and BCLRX instructions.
** Additional testing is required to identify the specific
** instruction.
*/
#define OP_BCXX				19

/*
** Identify BCCTRX and BCLRX instructions.
*/

#define BCCTRX_MASK		0xfc00fffe
#define BCCTRX_KEY		0x4c000420
#define IS_BCCTRX( x )	(((ui32)(x) & BCCTRX_MASK) == \
									 BCCTRX_KEY)

#define BCCLRX_MASK		0xfc00fffe
#define BCCLRX_KEY		0x4c000020
#define IS_BCCLRX( x )	(((ui32)(x) & BCCLRX_MASK) == \
									 BCCLRX_KEY)

/*
** Identify an blr instruction blr <=> bclr 20,0 (unconditional branch to addr in LR)
*/

#define PPC_BLR_MASK		0x4e800020
#define IS_PPC_BLR( x )		(((ui32)(x) & PPC_BLR_MASK) == \
									PPC_BLR_MASK)

/*
** Extract the AA bit from an instruction.  This is valid for BX and BCX
** instructions.  It is in bit 30.
*/

#define PPC_B_AA_MASK		0x00000002
#define PPC_B_AA( x )		(((ui32)(x) & PPC_B_AA_MASK) >> 1)

/*
** Extract the LK bit from an instruction.  This is valid for BX, BCX,
** BCCTRX, and BCCLRX instructions.  It is in bit 31.
*/

#define PPC_B_LK_MASK		0x00000001
#define PPC_B_LK( x )		(((ui32)(x) & PPC_B_LK_MASK))


/*
** Extract the branch destination from a BX instruction.
**	i: The bx instruction
**	a: The address of the bx instruction
**
** The base value (LI) is in bits 6-29.  That value is left-shifted
** by 2 and sign-extended.  If the AA bit is not set, the address of
** the instruction is added.
*/

#define PPC_BX_LI_MASK		0x03fffffc
#define PPC_BX_DEST( i, a )	((ui32)\
								(SIGN_EXTEND_26( ((ui32)(i)) & PPC_BX_LI_MASK ) + \
								 (PPC_B_AA( i ) ? 0 : ((signed long)(a)))) \
							)

/*
** Extract the branch destination from a BCX instruction.
**	i: The bcx instruction
**	a: The address of the bcx instruction
**
** The base value (BD) is in bits 16-29.  That value is left-shifted
** by 2 and sign-extended.  If the AA bit is not set, the address of
** the instruction is added.
*/

#define PPC_BCX_LI_MASK			0x0000fffc
#define PPC_BCX_DEST( i, a )	((ui32)\
									(SIGN_EXTEND_16( ((ui32)(i)) & PPC_BCX_LI_MASK ) + \
									 (PPC_B_AA( i ) ? 0 : ((signed long)(a)))) \
								)


#endif	// #ifndef __ppc_instr_h__
