/****************************************************************************/
/*

DESCRIPTION

	PPC-specific implementation function prototypes and structues.
	
	Functions may only work for 32-bit (or smaller) processors.
	
COPYRIGHT

	Copyright 1999, 2000, 2002, 2006, 2008, 2009 Freescale Semiconductor, Inc.
	
*/
/****************************************************************************/

#ifndef __TARGIMPL_PPC_H__
#define __TARGIMPL_PPC_H__

#include "target.h"
#include "dstypes.h"
#include "dserror.h"
#include "msgcmd.h"

/****************************************************************************/
/*
 *	Function prototypes
 */
/****************************************************************************/


/****************************************************************************/
/*
 *	TRK processor state (prior to entering user code)
 */
/****************************************************************************/

typedef struct TRKState_PPC
{
	DefaultType		GPR[32];
	DefaultType		LR;
	DefaultType		CTR;
	DefaultType		XER;
	Extended1Type	MSR;
#ifdef _E500
	Extended1Type	DBCR0;
	Extended1Type	DBSR;
#else
	Extended1Type	DAR;
	Extended1Type	DSISR;
#endif
	bool			stopped;
#if TRK_TRANSPORT_INT_DRIVEN
	bool			inputActivated;
	volatile unsigned char *inputPendingPtr;
#endif /* TRK_TRANSPORT_INT_DRIVEN */
} TRKState_PPC;

#ifndef IS_SMP
extern TRKState_PPC 			gTRKState;
#endif

/****************************************************************************/
/*
 *	Target program's processor state (exception state)
 */
/****************************************************************************/
#ifndef IS_SMP
extern ProcessorState_PPC 	gCPUState;
#endif

/*
** Some registers are not normally restored to their pre-exception states
** by the TRK.  Typically all timer/counter registers fall into this 
** category, since they will continue to change as the TRK runs.  They 
** are only restored if they have been explicitly modified by the
** debugger.
*/

typedef struct ProcessorRestoreFlags_PPC
{
	ui8 TBR;
	ui8 DEC;
	ui8 linker_padding[7];	  /* The linker doesn't like anything smaller! */
} ProcessorRestoreFlags_PPC;

extern ProcessorRestoreFlags_PPC gRestoreFlags;

/*
** Separate storage for processor state used by transport
** interrupt handler.
*/

extern Default_PPC gSaveState;


__extern_c

	ui32 __get_MSR( void );
	void __set_MSR( ui32 val );
	ui32 __get_PVR( void );
	ui32 __get_DBCR0( void );

	void SaveExtended1Block( void );
	void RestoreExtended1Block( void );
	DSError PPCAccessSPR(void* value, unsigned int spr_register_num, bool read);
	void SwapAndGo( void );

	extern void flush_cache(ui32 address, ui32 size);
	
	DSError TargetReadInstruction(InstructionType* readData,
								void* virtualAddress);
	DSError HyperTargetReadInstruction(InstructionType* readData,
								void* virtualAddress);
	DSError OsTargetReadInstruction(InstructionType* readData,
								void* virtualAddress,
								ui32 processId, ui32 threadId);

	DSError TargetWriteInstruction(InstructionType *data, void* virtualAddress);
	DSError HyperTargetWriteInstruction(InstructionType *data, void* virtualAddress);
	DSError OsTargetWriteInstruction(InstructionType *data, void* virtualAddress,
								ui32 processId, ui32 threadId);

__end_extern_c


#if DS_PROTOCOL < DS_PROTOCOL_HYPER

#define XTargetReadInstruction		TargetReadInstruction
#define XTargetWriteInstruction		TargetWriteInstruction

#elif DS_PROTOCOL < DS_PROTOCOL_RTOS

#define XTargetReadInstruction		HyperTargetReadInstruction
#define XTargetWriteInstruction		HyperTargetWriteInstruction

#else

#define XTargetReadInstruction		OsTargetReadInstruction
#define XTargetWriteInstruction		OsTargetWriteInstruction

#endif /* #if DS_PROTOCOL < DS_PROTOCOL_RTOS */



#endif /* __TARGIMPL_H__ */
