/******************************************************************************************/
/*

DESCRIPTION

	Abstract interface to serial device (UART) for CodeWarriorTRK.
	For maximum portability, do not change this file.
	
	NOTE: This interface is obsolete.  The new interface is
	define in transport_clayer.h.  This interface is still
	supported in most cases for legacy code.

COPYRIGHT

	Copyright 1999-2002, 2006, 2007, 2009 Freescale Semiconductor, Inc.
	
*/
/******************************************************************************************/

/* #pragma once */
#ifndef uart_h_included
#define uart_h_included

#include "transport_clayer.h"

/****************************************************************************/

typedef int UARTError;

enum {
	kUARTDataError = -1,
	kUARTNoError = 0,
	kUARTUnknownBaudRate,
	kUARTConfigurationError,
	kUARTBufferOverflow,				/* specified buffer was too small */
	kUARTNoData							/* no data available from polling */
};

/****************************************************************************/

typedef enum {
	kBaudHWSet = -1,					/* use HW settings such as DIP switches */
	kBaud300 = 300,						/* valid baud rates */
	kBaud600 = 600,
	kBaud1200 = 1200,
	kBaud1800 = 1800,
	kBaud2000 = 2000,
	kBaud2400 = 2400,
	kBaud3600 = 3600,
	kBaud4800 = 4800,
	kBaud7200 = 7200,
	kBaud9600 = 9600,
	kBaud19200 = 19200,
	kBaud38400 = 38400,
	kBaud57600 = 57600,
	kBaud115200 = 115200,
	kBaud230400 = 230400
} UARTBaudRate;

/****************************************************************************/

typedef struct UartParams
{
	/* uart initialization param */
	UARTBaudRate baudRate;
	unsigned char intDrivenInput;
	unsigned char intDrivenOutput;
} UartParams;

/****************************************************************************/

UARTError InitializeUART(UARTBaudRate baudRate);
UARTError InitializeIntDrivenUART( UARTBaudRate baudRate, 
								   unsigned char intDrivenInput,
								   unsigned char intDrivenOutput,
								   volatile unsigned char **inputPendingPtrRef );
UARTError TerminateUART(void);

UARTError ReadUARTPoll(char* c);

UARTError ReadUART1(char* c);
UARTError ReadUARTN(void* bytes, unsigned long length);
UARTError ReadUARTString(char* string, unsigned long limit, char terminateChar);

UARTError WriteUART1(char c);
UARTError WriteUARTN(const void* bytes, unsigned long length);
UARTError WriteUARTString(const char* string);
UARTError WriteUARTFlush(void);

int PollUART( void );
void UARTInterruptHandler( void );

void TransportIrqHandler( void );

#endif /* uart_h_included */
