/******************************************************************************************/
/*

DESCRIPTION

	Default support-mask constant definitions.  This file should
	be included from "target.h" for each target.  If a particular
	target needs to define non-default values, copy this file into
	the target directory and modify it there.
	
	Note: This file uses the value of TRK_TRANSPORT_INT_DRIVEN,
	so it must be included after that constant is defined.

COPYRIGHT

	Copyright 1999, 2002, 2006, 2007, 2009 Freescale Semiconductor, Inc.
	
*/
/******************************************************************************************/

#pragma once

/*
** If TRK_TRANSPORT_INT_DRIVEN is not defined, assume that
** interrupt-driven communication is not supported.
*/

#ifndef TRK_TRANSPORT_INT_DRIVEN
	#define TRK_TRANSPORT_INT_DRIVEN		(0)
#endif

/*
 * Support Mask - Used to tell the host which commands are supported by
 * this implementation of CodeWarriorTRK.  See msgcmd.h for a list of commands
 * and a better explanation of the meaning of this mask.  The mask is
 * defined in 8-bit segments.
 */
 
#define DS_SUPPORT_MASK_00_07	0x7A	/* kDSConnect, kDSReset, kDSVersions, kDSSupportMask, kDSCPUType */
#define DS_SUPPORT_MASK_08_0F	0x00
#define DS_SUPPORT_MASK_10_17	0x4f	/* R/W Memory, R/W Registers, FlushCache */
#if TRK_TRANSPORT_INT_DRIVEN
	#define DS_SUPPORT_MASK_18_1F	0x07	/* kDSContinue, kDSStep, kDSStop */
#else
	#define DS_SUPPORT_MASK_18_1F	0x03	/* kDSContinue, kDSStep */
#endif
#define DS_SUPPORT_MASK_20_27	0x00
#define DS_SUPPORT_MASK_28_2F	0x00
#define DS_SUPPORT_MASK_30_37	0x00
#define DS_SUPPORT_MASK_38_3F	0x00
#define DS_SUPPORT_MASK_40_47	0x00
#define DS_SUPPORT_MASK_48_4F	0x00
#define DS_SUPPORT_MASK_50_57	0x00
#define DS_SUPPORT_MASK_58_5F	0x00
#define DS_SUPPORT_MASK_60_67	0x00
#define DS_SUPPORT_MASK_68_6F	0x00
#define DS_SUPPORT_MASK_70_77	0x00
#define DS_SUPPORT_MASK_78_7F	0x00
#define DS_SUPPORT_MASK_80_87	0x01	/* kDSReplyACK */
#define DS_SUPPORT_MASK_88_8F	0x00
#define DS_SUPPORT_MASK_90_97	0x03	/* kDSNotifyStopped, kDSNotifyException */
#define DS_SUPPORT_MASK_98_9F	0x00
#define DS_SUPPORT_MASK_A0_A7	0x00
#define DS_SUPPORT_MASK_A8_AF	0x00
#define DS_SUPPORT_MASK_B0_B7	0x00
#define DS_SUPPORT_MASK_B8_BF	0x00
#define DS_SUPPORT_MASK_C0_C7	0x00
#define DS_SUPPORT_MASK_C8_CF	0x00
#define DS_SUPPORT_MASK_D0_D7	0x03	/* kDSWriteFile, kDSReadFile */
#define DS_SUPPORT_MASK_D8_DF	0x00
#define DS_SUPPORT_MASK_E0_E7	0x00
#define DS_SUPPORT_MASK_E8_EF	0x00
#define DS_SUPPORT_MASK_F0_F7	0x00
#define DS_SUPPORT_MASK_F8_FF	0x80	/* kDSReplyNAK */
