/******************************************************************************************/
/*

DESCRIPTION

	Default transport configuration file.  The default configuration is
	a single communication channel via serial port using the default 
	CodeWarriorTRK message framing (rfc1662).
	
	To create a custom configuration for a specific board, copy this
	file to the board directory and modify the new copy.

COPYRIGHT

	Copyright 2002, 2003, 2006, 2007, 2009 Freescale Semiconductor, Inc.
	
*/
/******************************************************************************************/

#include "trk_prefix.h"
#include "target.h"
#include "transport.h"
#include "UART.h"
#include "rfc1662.h"

/****************************************************************************/
/* Define transport global variables                                        */
/****************************************************************************/


/****************************************************************************/
/* 
 *	Define UART layer & parameters.  Each UART library should have
 *	a global, gUartApi, which gives a CharLayer API implementation for
 *	that library.  That API should accept the default UartParams
 *	parameter struct.  Create an instance of a transport layer based
 *	on these.
 *
 */
/****************************************************************************/
extern CharLayerApi gUartApi;	/* should be defined in UART driver */

static UartParams transportUartParams = 
{
	TRK_BAUD_RATE,				/* baud rate */
	TRK_TRANSPORT_INT_DRIVEN,	/* interrupt driven input */
	0							/* interrupt driven output */
};

static CharLayer transportUartLayer =
{
	&transportUartParams,		/* params for this layer */
	&gUartApi,					/* api implementation for this layer */
};

/* use the default UART layer for debug output as well */
#if defined( DEBUGIO_SERIAL ) || defined( DB_RAM_RESET_DUMP )
CharLayer *gUsrPutLayer = &transportUartLayer;
#endif

/****************************************************************************/
/* 
	Define framing layer & parameters (used to frame serial messages).
	CodeWarriorTRK interfaces directly with this layer.  This layer uses
	the UART layer for the actual communication.
	Define framing layer & parameters.
*/
/****************************************************************************/
static Rfc1662Params transportRfc1662Params =
{
	&transportUartLayer			/* pointer to character-based sub-layer */
};

static MsgLayer transportRfc1662Layer =
{
	&transportRfc1662Params,	/* params for this layer */
	&gRfc1662Api,				/* api implementation for this layer */
};


/****************************************************************************/
/* Define the number of available communication ports                       */
/****************************************************************************/
#define TRK_TRANSPORT_NUM_COMM_PORTS			1
unsigned char gTrkTransportNumCommPorts			= TRK_TRANSPORT_NUM_COMM_PORTS;

/****************************************************************************/
/*
 * Define the number of communication ports which can remain active
 * at one time.
 */
/****************************************************************************/
unsigned char gTrkTransportNumCommPortsActive	= 1;

/****************************************************************************/
/*
 * gTransportPrimaryIndex is the index of the primary port.
 * Primary port is the fastest available port which CodeWarriorTRK will
 * use to send big messages (length > TRK_TRANSPORT_MSG_THRESHOLD (0x1000)).
 *
 * For example, if CodeWarriorTRK supports two types of communications ports:
 * serial (index = 0) and ethernet (index == 1), then
 * gTransportPrimaryIndex should be defined as 1.
 */
/****************************************************************************/
unsigned char gTrkTransportPrimaryIndex			= 0;


/****************************************************************************/
/*
 *	Define transport table
 *	
 *	In this case, there is only a single entry consisting of the rfc1662
 *	framing layer.  There is no direct reference to the UART interface
 *	at this level.  However, the parameters for the framing layer point
 *	to the UART layer underneath (see above).
 */
/****************************************************************************/
Transport gTransportTable[TRK_TRANSPORT_NUM_COMM_PORTS] =
{
	{
		true,						/* active */
		false,						/* used */
		kSerial,					/* port type */
		kChannel_0,					/* channel number */
		&transportRfc1662Layer		/* pointer to top communication layer */
	}
};
