/******************************************************************************************/
/*

DESCRIPTION

	Get input stream/message IO command


COPYRIGHT

	Copyright 2002-2004, 2006, 2007, 2009 Freescale Semiconductor, Inc.
	
*/
/******************************************************************************************/

#include "trk_prefix.h"
#include "input.h"
#include "msg.h"
#include "nubevent.h"
#include "usr_put.h"
#include "nubassrt.h"

#include "transport.h"
#include "transport_tbl.h"

#ifndef IS_SMP
extern int gCurTransport;
extern volatile unsigned char gInputPending;
#endif


/******************************************************************************/
/*
	GetInput
	
	Gets new input, if any, and puts it in the event queue.
*/
/******************************************************************************/
void GetInput(void)
{
	MessageBuffer*	msgBuffer;
	MessageBufferId	bufferId;
	MessageCommandID command;
	ui8	data;

#if TRK_TRANSPORT_INT_DRIVEN
	/*
	** Assume that no input is pending.  If the read does
	** not return a message, this assumption will be confirmed
	** so we can leave the value at 0.
	** Note: gInputPending can be set by transport interrupts.
	** This assumption/confirmation method conveniently avoids
	** the need to disable interrupts or do any fancy target-
	** specific synchronization.
	*/
	gInputPending = false;
#endif /* TRK_TRANSPORT_INT_DRIVEN */

	TransportAlternator();
	
    bufferId = kInvalidMessageBufferId;
    MSG_LAYER_READ( TRANSPORT_LAYER( gCurTransport ), &bufferId );
	if (bufferId != kInvalidMessageBufferId) {
		/*
		 *	message arrived
		 */

#if TRK_TRANSPORT_INT_DRIVEN
		/*
		** Our earlier assumption about pending input could not
		** be confirmed.  At this point, don't know whether or
		** not there is more input.  Assume yes.  If this is
		** wrong, the next call to GetInput() will correct the
		** situation.
		*/
		gInputPending = true;
#endif /* TRK_TRANSPORT_INT_DRIVEN */

		msgBuffer = GetBuffer(bufferId);

		SetBufferPosition( msgBuffer, 0, false );
		MessageGet_ui8( msgBuffer, &data );
		command = (MessageCommandID)data;
		
		if (command < kDSReplyACK) {	/* only non-ack commands */
		
			PutMsgOnEventQueue( bufferId );

		} else {
		
			/*
			** We aren't waiting for an ACK at this point.  If one
			** arrives, ignore it.
			*/
			__puts( "Ignoring spurious ACK/NAK\r\n" );
			ReleaseBuffer(bufferId);
		}
	}
}

/******************************************************************************/
/*
	PutMsgOnEventQueue
	
	Puts the given message in the event queue as an input (request)
	event.
*/
/******************************************************************************/
void PutMsgOnEventQueue( MessageBufferId bufferId )
{
	NubEvent event;
	
	ConstructEvent(&event, kRequestEvent);
	event.fMessageBufferId = bufferId;	/* event owns buffer here */
	
	__puts("posting event\r\n");

	/*
	 *	Post an event to be handled in the main loop
	 */
	PostEvent(&event);
}			

