/******************************************************************************************/
/*
	
DESCRIPTION

	Command handler prototypes.
	
	Handle messages sent from the host to TRK.
	
COPYRIGHT

	Copyright 1999-2004, 2006, 2007, 2009 Freescale Semiconductor, Inc.
	
*/
/******************************************************************************************/

#ifndef __MSGHNDLR_H__
#define __MSGHNDLR_H__


#include "target.h"
#include "msgcmd.h"
#include "msgbuf.h"

/*
** Location to save most recently sent ACK message.
*/
extern MessageBuffer *gPreviousAck;

/*
** Connected status.
*/
extern bool gTrkConnected;

__extern_c

	/*
	 *	Standard ACK/NAK (noncommands)
	 */
	void MessageIntoReply(MessageBuffer* b,
							ui8 ackCmd /* MessageCommandID*/,
							DSReplyError errSentInAck);
	DSError SendACK(MessageBuffer*);
	DSError StandardACK(MessageBuffer*, ui8 ackCmd /*MessageCommandID*/,
							DSReplyError errSentInACK);
	DSError DoError(MessageBuffer*);
	DSError DoUnsupported(MessageBuffer*);

	/*
	 *	meta command set
	 */
	DSError DoPing(MessageBuffer* b);
	DSError DoConnect(MessageBuffer*);
	DSError DoDisconnect(MessageBuffer*);
	DSError DoReset(MessageBuffer*);
	DSError DoVersions(MessageBuffer*);
	DSError DoSupportMask(MessageBuffer*);
	DSError DoCPUType(MessageBuffer* b);
	DSError DoConfigTransport( MessageBuffer *b );

	/*
	 *	target "state" command set
	 */
	 
	DSError DoReadMemory(MessageBuffer*);
	DSError DoWriteMemory(MessageBuffer*);
	DSError DoReadRegisters(MessageBuffer*);
	DSError DoWriteRegisters(MessageBuffer*);
	DSError DoFlushCache(MessageBuffer*);
	DSError DoSetBreak(MessageBuffer* b);
	DSError DoClearBreak(MessageBuffer* b);
	DSError DoModifyBreakThread(MessageBuffer* b);
	DSError DoCheckTargetState(MessageBuffer* b);
	
	/*
	 *	target "execution" command set
	 */
	 
	DSError DoContinue(MessageBuffer*);
	DSError DoStep(MessageBuffer*);
	DSError DoStop(MessageBuffer*);
	
	/*
	** host->target file access command set
	*/
	
	DSError DoOsWriteFile( MessageBuffer *b );
	DSError DoOsReadFile( MessageBuffer *b );
	DSError DoOsOpenFile( MessageBuffer *b );
	DSError DoOsCloseFile( MessageBuffer *b );
	DSError DoOsPositionFile( MessageBuffer *b );
	
	/*
	** host->target file IO management
	*/
	
	DSError DoNotifyFileInput( MessageBuffer *b );
	DSError DoBlockFileIo( MessageBuffer *b );

__end_extern_c

#endif /* ndef __MSGHNDLR_H__ */
