/******************************************************************************************/
/*
	
DESCRIPTION

	Event queue.

COPYRIGHT

	Copyright 1999-2002, 2004, 2006, 2007, 2009 Freescale Semiconductor, Inc.
	
*/
/******************************************************************************************/

#ifndef _NUBEVENT_H_
#define _NUBEVENT_H_

/****************************************************************************/
/*
 *	Includes
 */
/****************************************************************************/
#include "dstypes.h"
#include "dserror.h"
#include "msgbuf.h"
#include "target.h"

enum {								/* DSErroror; see dserror.h */
	kEventQueueFull = 0x0100
};

/****************************************************************************/
/*
 *	Types
 */
/****************************************************************************/

/****************************************************************************/
typedef unsigned long NubEventId;	/* unique ID for nub event */

enum {
	kInvalidEventId = 0			/* NubEventId */
};

/****************************************************************************/
typedef enum NubEventType {
	kNullEvent,					/* an event with no data */
	kShutdownEvent,				/* shutdown event */
	kRequestEvent,				/* packet from host */
	kBreakpointEvent,			/* breakpoint was hit */
	kExceptionEvent,			/* non-breakpoint exception */
	kSupportEvent,				/* target support services */
	kContinueEvent				/* continue executing a target process event */
} NubEventType;

/****************************************************************************/
typedef struct NubEvent {
	NubEventType		fType;
	NubEventId			fId;
	MessageBufferId		fMessageBufferId;
	ui32				fInfo[8];		/* for storing add'l type-specific info */
#if DS_PROTOCOL == DS_PROTOCOL_RTOS
	ui32				fProcessId;
	ui32				fThreadId;
#endif
} NubEvent;

/****************************************************************************/
/*
 *	Function Prototypes
 */
/****************************************************************************/
__extern_c

	DSError InitializeEventQueue( void );
	DSError TerminateEventQueue( void );
	bool GetNextEvent(NubEvent* copiedEvent);
	DSError PostEvent(const NubEvent* copiedEvent);
	void CopyEvent(NubEvent* dstEvent, const NubEvent* srcEvent);
	
	void ConstructEvent(NubEvent* newEvent, NubEventType type);
	void DestructEvent(NubEvent* oldEvent);

__end_extern_c

#endif /* _NUBEVENT_H_ */
