/******************************************************************************************/
/*
	
DESCRIPTION

	Portable step code for CodeWarriorTRK.

COPYRIGHT

	Copyright 2000-2004, 2006, 2007, 2009 Freescale Semiconductor, Inc.
	
*/
/******************************************************************************************/

#ifndef _STEP_H_
#define _STEP_H_

/****************************************************************************/
/*
 *	Includes
 */
/****************************************************************************/
#ifndef _NO_SYSTEM_INCLUDES
#include <stddef.h>
#endif
#include "dstypes.h"
#include "dserror.h"
#include "break.h"

/****************************************************************************/
/*
 *	Errors
 */
/****************************************************************************/

/****************************************************************************/
/*
 *	Types
 */
/****************************************************************************/

/*
** Used to specify memory ranges which the user does not want to see while
** stepping.
*/

typedef struct TrkMemRange {
	PCType	fStart;
	PCType	fEnd;
} TrkMemRange;


/*
** Used to create dynamic list for invisible step ranges  which the user does not want to see while
** stepping.
*/

typedef struct InvisibleRangeInfo {
	TrkMemRange							InvisibleStepRanges;
	struct InvisibleRangeInfo			*next;
}InvisibleRangeInfo;

	
/*
** All current implementations of single-instruction trace using
** breakpoints only require 2 breakpoints.
*/

#define kTraceBreakpointCount		2

typedef struct StepStatus {
	bool active;
	bool stepOver;
	DSMessageStepOptions type;
	unsigned count;
	ui32 rangeStart;
	ui32 rangeEnd;
	ui32 traceStart;
	BreakpointId tempDisableBreakpointId;
	BreakpointId traceBreakpoints[ kTraceBreakpointCount ];
} StepStatus;

/*
** Hopefully we never need a recursive step depth more
** than 2.  
*/

#define kStepMaxDepth				2
	
typedef struct StepStatii {
	ui8 current;
	StepStatus arr[ kStepMaxDepth ];
} StepStatii;
	


/****************************************************************************/
/*
 *	Public Function Prototypes
 */
/****************************************************************************/
__extern_c

	DSError TargetSingleStep( unsigned count, bool stepOver,
							ui32 processId,
							ui32 threadId );

	DSError TargetStepOutOfRange( ui32 start, ui32 end,
							bool stepOver,
							ui32 processId,
							ui32 threadId );

	bool TargetCheckStep( InvisibleRangeInfo	*fInvisibleStepRangeList,
							bool *invisibleStop, ui32 processId, ui32 threadId );

	DSError TargetSetupTempDisableStep( BreakpointId tdbId,
										ui32 processId, ui32 threadId );

	DSError TargetCancelStep( ui32 processId, ui32 threadId );

	DSError InitStepStatii( StepStatii *statii );
	DSError InitStepStatus( StepStatus *status );

__end_extern_c

#endif /* _STEP_H_ */
