/******************************************************************************************/
/*

DESCRIPTION

	Hardware interface (drivers) header file	

COPYRIGHT

	Copyright 2000-2003, 2006, 2007, 2009 Freescale Semiconductor, Inc.
	
*/
/******************************************************************************************/

#ifndef __TRANSPORT_H__
#define __TRANSPORT_H__

#include "dstypes.h"
#include "msgbuf.h"
#include "transport_clayer.h"
#include "transport_mlayer.h"
#include "usr_put.h"
#ifdef IS_SMP
#include "smp.h"
#endif

/*
** Message Threshold
*/
#define TRK_TRANSPORT_MSG_THRESHOLD		0x1000

/****************************************************************************/
/*                                                                          */
/* Type enum                                                                */
/*                                                                          */
/****************************************************************************/

typedef enum {
	kSerial		= 1,
	kEthernet 	= 2,
	kByteChan   = 3
} Port;

typedef enum {
	kChannel_0 = 0,
	kChannel_1 = 1,
	kChannel_2 = 2,
	kChannel_3 = 3,
	kChannel_4 = 4,
	kChannel_5 = 5,
	kChannel_6 = 6,
	kChannel_7 = 7
} Channel;


/****************************************************************************/
/*                                                                          */
/* Type definitions                                                         */
/*                                                                          */
/****************************************************************************/

/****************************************************************************/
/*                                                                          */
/* Structure Name...........:                                               */
/*		Transport		                                                    */
/*                                                                          */
/* Description..............: 									            */
/* 		The structure contains necessary communication port attributes.     */
/*                                                                          */
/* Fields...................:                                               */
/*      active..............: the port and channel is active                */
/*      used................: the port and channel is used by CodeWarriorTRK      */
/* 		port................: port type                                     */
/*      channel.............: channel number                                */
/*      funcPtr.............: pointer to the transport layer functions      */
/*                                                                          */
/****************************************************************************/

typedef struct Transport
{
	bool		active;
	bool		used;
	Port		port;
	Channel		channel;
	MsgLayer	*layer;
} Transport;

/****************************************************************************/
/*
 * Global Variables
 */
/****************************************************************************/
#ifndef IS_SMP
/* Transport table */
extern Transport gTransportTable[];

/* Index of current transport */
extern int gCurTransport;

extern unsigned char gTrkTransportNumCommPorts;
extern unsigned char gTrkTransportNumCommPortsActive;
extern unsigned char gTrkTransportPrimaryIndex;

#if TRK_TRANSPORT_INT_DRIVEN
extern volatile unsigned char gInputPending;
#endif /* TRK_TRANSPORT_INT_DRIVEN */

#else
#define gTransportTable ((Transport*)PRIVATE_DATA(TRANSPORT_TABLE_ENTRY))
#define gCurTransport PRIVATE_DATA(TRANSPORT_CURRENT)
#define gTrkTransportNumCommPorts PRIVATE_DATA(TRANSPORT_NUM_PORTS)
#define gTrkTransportNumCommPortsActive PRIVATE_DATA(TRANSPORT_NUM_PORTS_ACTIVE)
#define gTrkTransportPrimaryIndex PRIVATE_DATA(TRANSPORT_PRIMARY_INDEX)

#if TRK_TRANSPORT_INT_DRIVEN
#define gInputPending PRIVATE_DATA(INPUT_PENDING)
#endif /* TRK_TRANSPORT_INT_DRIVEN */

#endif /* IS_SMP */

/****************************************************************************/
/*                                                                          */
/* Function Prototypes                                                      */
/*                                                                          */
/****************************************************************************/

/* Core */

__extern_c

	DSError InitializeTransport(void);
	DSError TransportInitializeIndex( int index );
	DSError TransportTerminateIndex( int index );
	void TransportSelector(MessageBuffer* buffer);
	void TransportAlternator(void);
	void TransportIrq(void);

__end_extern_c

#endif /* __TRANSPORT_H__ */
