/******************************************************************************************/
/*

DESCRIPTION

	Message-based transport layer header.  The message-based
	layer implements I/O for complete messages and is the
	layer which interfaces directly with the CodeWarriorTRK core.

COPYRIGHT

	Copyright 2002, 2006, 2007, 2009 Freescale Semiconductor, Inc.
	
*/
/******************************************************************************************/

#ifndef __TRANSPORT_MLAYER_H__
#define __TRANSPORT_MLAYER_H__

#include "transport_clayer.h"
#include "msgbuf.h"

/****************************************************************************/
/*
 * Type enum
 */
/****************************************************************************/

/****************************************************************************/
/*
 * Type definitions
 */
/****************************************************************************/

struct MsgLayer;

/****************************************************************************/
/*
 * Structure Name...........:
 *		MsgLayerApi
 *
 * Description..............:
 * 		The structure contains message layer api (functional implementation).
 *
 * Fields...................:
 *
 *      The following structure fields contain function pointers
 *      to their respective functions of the specified message-based
 *		communication layer.  The required fields *must* contain
 *		valid pointers to appropriate functions.  The optional fields
 *		may be set to NULL if they are not implemented for a 
 *		particular layer.
 *
 *      init................: Initialize layer. Required.
 *      terminate...........: Terminate layer. Required.
 *      read................: Read a message, if any.  Non-blocking.
 *							  Required. 
 *      write...............: Write a message.  Required.
 *      irq.................: Handle layer communication interrupt.  Return
 *							  0 if no new input arrived, 1 otherwise.  If in
 *							  doubt, return 1.
 *							  Optional.
 */
/****************************************************************************/

typedef DSError (MsgLayerReadFunc)( LayerParams *params, 
									MessageBufferId *bufferId );
typedef DSError (MsgLayerWriteFunc)( LayerParams *params, 
									MessageBuffer *buffer );

typedef struct MsgLayerApi
{
	LayerInitFunc			*init;
	LayerTerminateFunc		*terminate;
	MsgLayerReadFunc		*read;
	MsgLayerWriteFunc		*write;
	LayerIrqFunc			*irq;
} MsgLayerApi;


/****************************************************************************/
/*
 * Structure Name...........:
 *		MsgLayer
 *
 * Description..............:
 * 		The structure contains a message-based layer instance
 *
 * Fields...................:
 *
 *		The following structure fields contain information about a specific
 *		message-based layer instance.
 *
 *      param...............: layer parameters
 *		api.................: layer api (functional implementation)
 */
/****************************************************************************/
typedef struct MsgLayer
{
	LayerParams		*params;
	MsgLayerApi		*api;
} MsgLayer;


/****************************************************************************/
/*
 * Macros to access layer
 */
/****************************************************************************/
#define MSG_LAYER_READ(layer,b)			(*((layer)->api->read))((layer)->params,b)
#define MSG_LAYER_WRITE(layer,b)		(*((layer)->api->write))((layer)->params,b)


/****************************************************************************/
/*
 * Function Prototypes
 */
/****************************************************************************/

#endif /* __TRANSPORT_MLAYER_H__ */
